/*
pGina plugin for RSA SecurID ACE server with Domain logon support
Copyright (C) 2003 NASA Glenn Research Center
Web: http://www.grc.nasa.gov

Author:  Norbert Seidel
Email: pgina@cherryoakservices.com
Web: http://www.cherryoakservices.com

Note:  This code also contains free sample software from RSA ACE example
programs (async.c), MicroSoft crypt examples, ldap and dummy plugins.

$Log: DummyPlugin.cpp,v $
Revision 1.7  2003/08/04 14:22:11  nseidel
New pGina sets defaultDomain to CR?  Check for this.

Revision 1.6  2003/08/01 17:40:49  nseidel
change email and web address

Revision 1.5  2003/03/10 20:20:10  nseidel
iteration 5 - ACE prompts, chg windows password, pgina 1.6.2

Revision 1.4  2003/02/11 19:51:11  nseidel
iter 4 - SDK 1.6 upgrade, handle no domains

Revision 1.3  2003/02/05 20:49:33  nseidel
windows password prompt/expire support

Revision 1.2  2003/02/03 14:57:05  nseidel
Encrypt Windows password

Revision 1.1  2003/01/16 16:53:06  nseidel
New

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// DummyPlugin.cpp : Defines the entry point for the DLL application.
// pGina Plugin Specifications Document or visit us at: http://pgina.cs.plu.edu

#include "stdafx.h"
#include "DummyPlugin.h"
#include <wchar.h>
#include <tchar.h>

//nls - crypt/password
#include <wincrypt.h>
#define MY_ENCODING_TYPE  (PKCS_7_ASN_ENCODING | X509_ASN_ENCODING)
#include <Lm.h>
#include "resource.h"

//nls - ACE mods follow
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#include <windows.h>
#define sleep(x)    Sleep(x*1000)
#else /* UNIX */
#include <sys/time.h>
#include <pthread.h>
#include <errno.h>
#endif

// in NT this allows use of the aceclnt.lib file to link the DLL
#define USE_ACE_AGENT_API_PROTOTYPES
#include "acexport.h"

#ifdef WIN32

typedef HANDLE CONDVAR;
typedef HANDLE MUTEX;

#else

#if !defined(HPUX) || (_POSIX_C_SOURCE - 0 >= 199506L)
#   define COND_TIMEOUT_ERROR(s) ((s) == ETIMEDOUT)
#   define pthread_mutexattr_default NULL
#   define pthread_condattr_default NULL
#else
#   define COND_TIMEOUT_ERROR(s) (((s) != 0) && (errno == EAGAIN))
#endif

#define INFINITE 0
#define WINAPI

typedef unsigned long DWORD;
typedef unsigned long UINT32;
typedef pthread_cond_t CONDVAR;
typedef pthread_mutex_t MUTEX;
typedef void * HANDLE;
#define CALLBACK
#define NO_ERROR 0

#endif

/*nls - moved to dialogs
#define AUTH_CHALLENGE_USERNAME_STR \
  "Enter USERNAME: "

#define AUTH_CHALLENGE_PASSCODE_STR \
  "Enter PASSCODE: "

#define AUTH_CHALLENGE_NEXT_CODE_STR \
  "Wait for the tokencode to change,\n" \
  "then enter the new tokencode: "

#define AUTH_CHALLENGE_NEW_PIN_STR \
  "You must select a new PIN.\n" \
  "Do you want the system to generate\n" \
  "your new PIN? (y/n) [n] "
*/

#define AUTH_CHALLENGE_NEW_PIN_ALPHA_SAME_STR \
  "Enter a new PIN of %d alphanumeric characters: "

#define AUTH_CHALLENGE_NEW_PIN_ALPHA_STR \
  "Enter a new PIN between %d and %d alphanumeric characters: "

#define AUTH_CHALLENGE_NEW_PIN_DIGITS_SAME_STR \
  "Enter a new PIN of %d digits: "

#define AUTH_CHALLENGE_NEW_PIN_DIGITS_STR \
  "Enter a new PIN between %d and %d digits: "

/*nls - moved to dialogs
#define AUTH_CHALLENGE_NEW_PIN_SYS_STR \
  "To continue, you must accept a new PIN generated\n" \
  "by the system. Are you ready to have the\n" \
  "system generate your PIN? (y/n) [n] "

#define AUTH_CHALLENGE_CONFIRM_PIN_STR \
  "Re-enter new PIN to confirm: "

#define AUTH_CHALLENGE_NEW_SYS_PIN_PASSCODE_STR \
  "Wait for the tokencode to change,\n" \
  "then enter a new PASSCODE: "

#define AUTH_CHALLENGE_NEW_USER_PIN_PASSCODE_STR \
  "PIN accepted. Wait for the tokencode to\n" \
  "change, then enter a new PASSCODE: "

#define AUTH_CHALLENGE_SUCCESS_STR \
  "PASSCODE accepted.\n"

#define AUTH_CHALLENGE_ACCESS_DENIED_STR \
  "Access denied.\n"

#define AUTH_ERROR_MISC_STR \
  "Unexpected authentication error.\n"

#define AUTH_ERROR_BAD_PASSCODE_STR \
  "Access denied.\n"

#define AUTH_ERROR_BAD_TOKENCODE_STR \
  "Access denied.\n"

#define AUTH_ERROR_INVALID_PIN_STR \
  "Invalid PIN.\n"

#define AUTH_ERROR_CONFIRM_PIN_STR \
  "PIN did not match confirmation. Press Enter to continue.\n"

#define AUTH_ERROR_INVALID_ARG_STR \
  "Invalid argument.\n"

#define AUTH_CHALLENGE_NEW_PIN_USER_STR \
  "To continue you must enter a new PIN.\n" \
  "Are you ready to enter a new PIN? (y/n) [n]"
*/

#define AUTH_CHALLENGE_NEW_SYS_PIN_DISPLAY_STR \
  "Your new PIN is: %s"
/*nls
  "\n\nYour screen will automatically clear in 10 seconds.\n" \
  "Your new PIN is: %s\n"
*/

// a waitable event object
typedef struct
{
  CONDVAR CondVar;    // an event object in NT, a condition variable in UNIX
#ifdef UNIX
  MUTEX Mutex;        // NULL in NT, mutex in UNIX
  int var;            // not used in NT, flag variable in UNIX
#endif
} SDEVENT_S, *P_SDEVENT_S;

// event notification functions
static SD_I32  sdCreateEvent(P_SDEVENT_S);
static void    sdDeleteEvent(P_SDEVENT_S);
static SD_I32  sdWaitForEvent(P_SDEVENT_S, DWORD);
static void    sdSetEvent(P_SDEVENT_S);
static void    sdResetEvent(P_SDEVENT_S);

typedef struct
{
  SDEVENT_S Event;
  int asynchAceRet;
  SDI_HANDLE aceHdl;
  char user[LENUSERNAME+1];
  char prn[LENPRNST+1];
} sEventData;

static void WINAPI aceCB(SDI_HANDLE hdl);

//nls static int Authenticate(void *);
static int Authenticate(LPTSTR Username, LPTSTR Password);

//nls - add the following
static BOOL getWindowsPassword(pGinaInfo *settingsInfo);
static BOOL setWindowsPassword(LPTSTR Username, LPTSTR Password);
static BOOL passwordPrompt(pGinaInfo *settingsInfo, LPTSTR domain);
VOID CenterWindow(HWND hwnd);
BOOL CALLBACK promptDialogProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam);

#ifdef _DEBUG
static FILE *fp;	// debug
#endif
// Users' ACE pin is stored here for encryption/decryption
static DATA_BLOB _DataEntropy;
// true if user prompted for new windows password.  This occurs the
// first time on a system or after password has expired.
static BOOL _saveMSPassword;
// true if windows password found in registry.  This is used to check
// if the user could logon with this password, e.g. has it expired?
static BOOL _foundMSPassword;
// Last user who attempted logon, used when windows password wrong.
static LPTSTR _lastUsername;
// needed for dialogs
HINSTANCE _AuthInstance;
// prompt dialog prompt
static  char _prompt[256];
// prompt dialog edit value
static TCHAR _promptString[1024];

// This isn't strictly necessary, but can be handy if you are going to add Dialog's etc
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
  )
{
  switch (ul_reason_for_call)
  {
  case DLL_PROCESS_ATTACH:
    _AuthInstance = (struct HINSTANCE__ *)hModule;		  // Save our parents hInstance
    break;
  case DLL_THREAD_ATTACH:
  case DLL_THREAD_DETACH:
  case DLL_PROCESS_DETACH:
    break;
  }
  return TRUE;
}

/* This is the function that pGina calls to find out whether a user is authorized or not
	it is also the place to put changes in per-user configurations etc. */
DUMMYPLUGIN_API BOOL UserLogin(LPTSTR Username, LPTSTR Password, pGinaInfo *settingsInfo)
{
  //nls - start ACE

  settingsInfo->errorString = NULL;

  // With Domains:
  //   This user/pswd will result in a successful pGina login, but fail
  // at the domain logon prompt.  Allowing a person to enter their Windows
  // userid and password to get in.  Only used for debugging purposes.
#ifdef _DEBUG
  if(!_tcscmp(Username,TEXT("security")))
  {
    settingsInfo->isAdmin = false;
    _saveMSPassword = false;
    fp=fopen("ACEPluginAccess.log", "w");
    fprintf(fp, "ACE passcode logon skipped, security access occurred.\n");
    fclose(fp);
    return true;
  }
#endif

#ifdef _DEBUG
  fp=fopen("ACEPlugin.log", "w");
  fprintf(fp, "ACEPlugin UserLogin called\n");
#endif
  
  // AceInitialize is now available and must be called in both Unix and NT
  if (!AceInitialize())
  {
#ifdef _DEBUG
    fprintf(fp, "\nAceInitialize failed\n");
    fclose(fp);
#endif
    // make sure domain logon attempt fails
    settingsInfo->Password = _tcsdup(TEXT("Failed"));
    settingsInfo->errorString = _tcsdup(TEXT("Could not access SecurID authentication server - AceInitialize failed."));
    return false;
  }
    
  if (Authenticate(Username, Password) == ACM_OK)
  {
#ifdef _DEBUG
    fprintf(fp, "Authentication successful.\n");
#endif
    TCHAR DDN[2048];
    DWORD dwDDN = 2048;
    HKEY    hKey;
    LONG    lResult;

    // Need domain for password checks
    lResult = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
			    _T("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon"),
			    0, KEY_QUERY_VALUE, &hKey );

    if(lResult == ERROR_SUCCESS)
    {
      lResult = RegQueryValueEx(hKey,_T("DefaultDomainName"),
				NULL, NULL, (LPBYTE) DDN, &dwDDN);
      RegCloseKey( hKey );
    }

    // This is need by getWindowsPassword to decrypt existing password
    // and by LoginHook later to encrypt password if a save password needed.
    BYTE *pbDataEntropy = (BYTE*) _tcsdup(Password);
    _DataEntropy.pbData = pbDataEntropy;
    _DataEntropy.cbData = 8; // 4*2 for UNICODE
#ifdef _DEBUG
    //_ftprintf(fp, TEXT("The entropy to use for encryption is: %4.4s\n"),_DataEntropy.pbData);
    _ftprintf(fp, TEXT("The entropy to use for encryption is: hidden\n"));
#endif

    if (_saveMSPassword)
    {
      // if not the same user, don't force a windows password prompt
      if(_tcscmp(_lastUsername,settingsInfo->Username))
      {
	_saveMSPassword = false;
      }
    }

    // If found=true then the windows password in the registry is no longer
    // valid and we need to force a prompt.
    if (_foundMSPassword)
    {
      // Is this the same user?
      if(!_tcscmp(_lastUsername,settingsInfo->Username))
      {
	// Should do.. lResult = RegDeleteValue(hKey,"pGinaDataUSERNAME"));
	_foundMSPassword = false;
	_saveMSPassword = true; // this will cause a prompt
      }
      else
      {
	_foundMSPassword = false;
      }
    }

    // If saveMSPassword=true then we were prompted last login and
    // entered the wrong windows password - shouldn't happen anymore
    // OR If failed to find saved MS password in registry
    // Either way, prompt user for new password.
    if (_saveMSPassword || ! getWindowsPassword(settingsInfo))
    {
#ifdef _DEBUG
      fprintf(fp, "getWindowsPassword failed\n");
#endif
      // Did user not enter a new password?
      if (! passwordPrompt(settingsInfo, DDN))
      {
	settingsInfo->errorString = _tcsdup(TEXT("Login failed - Windows password was incorrect."));
#ifdef _DEBUG
	fclose(fp);
#endif
	return false;
      }
#ifdef _DEBUG
      _ftprintf(fp, TEXT("settingsInfo->Password - hidden\n"));
#endif
    }
    else
    {
      // found password (keep in mind it may have expired!)
      HANDLE hUser;

      // test to see if password is OK
      if(LogonUser(settingsInfo->Username,DDN,settingsInfo->Password,LOGON32_LOGON_INTERACTIVE,LOGON32_PROVIDER_DEFAULT, &hUser))
      {
#ifdef _DEBUG
	fprintf(fp, "Found password, Windows password check succeeded.\n");
#endif
	CloseHandle(hUser);
	_foundMSPassword = true;
	_lastUsername = _tcsdup(settingsInfo->Username);
      }
      else
      {
	// Did user not enter a new password?
	if (! passwordPrompt(settingsInfo, DDN))
	{
	  settingsInfo->errorString = _tcsdup(TEXT("Login failed - Windows password has been changed or has expired."));
#ifdef _DEBUG
	  fclose(fp);
#endif
	  return false;
	}
      }
    }

    // DEFECT (using domains):
    // In the pGina configurator window, the user could set the domain to
    // the hostname or any bad domain.  This will cause a logon error
    // and bring up a Windows logon prompt.  This is a sneaky way to logon
    // to a Windows account other than the one ACE authenticated.
    
    //----- Determine if domain was overridden

    BOOL domainOverride = false;
    dwDDN = 2048;

    lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina"),
			   0,
			   KEY_QUERY_VALUE,
			   &hKey);

    if(lResult == ERROR_SUCCESS)
    {
      lResult = RegQueryValueEx(hKey,_T("defaultDomain"),
				NULL,
				NULL,
				(LPBYTE) DDN,
				&dwDDN);
      RegCloseKey( hKey );

      // if blank or just CR, then no domain
      if(dwDDN <= 2)
      {
	lResult = ERROR_GEN_FAILURE;
      }

      // configure override of domain?
      if(lResult == ERROR_SUCCESS)
      {
	settingsInfo->errorString = _tcsdup(TEXT("Login failed - setting a domain not permitted."));
	domainOverride = true;
      }
    }

#ifdef _DEBUG
    fclose(fp);
#endif
    if (domainOverride)
      return false; // DEFECT:  prevent backdoor logon
    else
      // This will fall through to the domain successfully if the user/pswd
      // are correct for Windows - not sure why though!
      return true;
  }

#ifdef _DEBUG
  fprintf(fp, "Authentication failed.\n");
  fclose(fp);
#endif

  // No one else gets in - isRequired is turned on
  settingsInfo->errorString = _tcsdup(TEXT("Authentication of passcode failed."));
  return false;
}

DUMMYPLUGIN_API BOOL ChangeUserPassword(LPTSTR Username,LPTSTR OldPassword,LPTSTR NewPassword)
{
/* Possible Errors:
//  The specified network password is not correct.
#define ERROR_INVALID_PASSWORD           86L
//  Configuration information could not be read from the domain controller, either because the machine is unavailable, or access has been denied.
Note: also if new password not valid, e.g. enough characters, numbers, etc
#define ERROR_CANT_ACCESS_DOMAIN_INFO    1351L
*/

  NET_API_STATUS nStatus;
  BOOL	  rc = false;

  nStatus = NetUserChangePassword(NULL, NULL, OldPassword, NewPassword);

#ifdef _DEBUG
  fp=fopen("ACEPlugin.log", "a");
  fprintf(fp, "User password rc - %d\n", nStatus);
  fclose(fp);
#endif

  //
  // If the call succeeds, inform the user.
  //
  if (nStatus == NERR_Success)
  {
    if (setWindowsPassword(Username, NewPassword))
    {
      return true;
    }
    else
    {
      MessageBox(NULL,TEXT("Failed to save password"),TEXT("pGina Change Password"),MB_ICONERROR | MB_OK | MB_SYSTEMMODAL);
      return false;
    }
  }
  else
  {
    if (nStatus == ERROR_INVALID_PASSWORD)
      MessageBox(NULL,TEXT("Invalid password"),TEXT("pGina Change Password"),MB_ICONERROR | MB_OK | MB_SYSTEMMODAL);
    else if (nStatus == ERROR_CANT_ACCESS_DOMAIN_INFO)
      MessageBox(NULL,TEXT("New password is invalid or domain not accessable"),TEXT("pGina Change Password"),MB_ICONERROR | MB_OK | MB_SYSTEMMODAL);
    else
      MessageBox(NULL,TEXT("Could not change password"),TEXT("pGina Change Password"),MB_ICONERROR | MB_OK | MB_SYSTEMMODAL);

    return false;
  }
}

DUMMYPLUGIN_API LPCTSTR AboutPlugin(void)
{
  // Return a sample descriptions string
  return TEXT("Enter the SecurID PIN and TOKENCODE for the PASSWORD.  ACE plugin with Domain logon support (UNICODE). $Revision: 1.7 $");
}
DUMMYPLUGIN_API void ChangePluginSettings(void)
{
  // Describe in a messagebox what this would do
  MessageBox(NULL,TEXT("This would bring up settings... but this is a simple plugin, so no settings."),TEXT("ChangePluginSettings()"),MB_OK);
}
DUMMYPLUGIN_API void LoginHook(pGinaInfo *settingsInfo)
{
#ifdef _DEBUG
  fp=fopen("ACEPlugin.log", "a");
  fprintf(fp, "ACEPlugin LoginHook called\n");
  fclose(fp);
#endif

  _foundMSPassword = false; // found MS password was OK

  // Skip saving (i.e. no changes)?
  if (! _saveMSPassword)
    return;

  // If not found, then user was prompted, we need to encrypt and save in reg 
  setWindowsPassword(settingsInfo->Username, settingsInfo->Password);
}

DUMMYPLUGIN_API void LogoutHook(pGinaInfo *settingsInfo)
{
#ifdef _DEBUG
  fp=fopen("ACEPlugin.log", "a");
  fprintf(fp, "ACEPlugin LogoutHook called\n");
  fclose(fp);
#endif
}

DUMMYPLUGIN_API BOOL IsRequired(void)
{
  return true;
}

// ===========================================================================
// Windows password code

static BOOL getWindowsPassword(pGinaInfo *settingsInfo)
{
  TCHAR MSPassword[2048];
  DWORD dwMSPassword = 2048;
  DWORD size = sizeof(DWORD);

  LONG lResult;
  HKEY hKey;
  BOOL rc = false;

//--------------------------------------------------------------------
// Declare and initialize variables.

  DATA_BLOB DataOut;
  DATA_BLOB DataVerify;
  LPWSTR pDescrOut = NULL;
  BYTE *pbDataOutput;

//--------------------------------------------------------------------
//  Begin processing.

  lResult = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
			  _T("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon"),
			  0,
			  KEY_QUERY_VALUE,
			  &hKey);

  // Get encrypted Windows password
  if(lResult == ERROR_SUCCESS)
  {
    // Size of password when decrypted!
    TCHAR tempRegSize[25];
    _stprintf(tempRegSize, TEXT("%s%s"), TEXT("pGinaSize"), settingsInfo->Username);
    lResult = RegQueryValueEx(hKey,tempRegSize,
			      NULL,
			      NULL,
			      (LPBYTE) &DataOut.cbData,
			      &size);

    if(lResult == ERROR_SUCCESS)
    {
#ifdef _DEBUG
      fprintf(fp, "The encrypted message size is %d. \n", DataOut.cbData);
#endif
      TCHAR tempRegData[25];
      _stprintf(tempRegData, TEXT("%s%s"), TEXT("pGinaData"), settingsInfo->Username);
      lResult = RegQueryValueEx( hKey, tempRegData,
				 NULL,
				 NULL,
				 (LPBYTE) MSPassword,
				 &dwMSPassword);

      if(lResult == ERROR_SUCCESS)
      {
	pbDataOutput = (BYTE*) MSPassword;
	DataOut.pbData = pbDataOutput;
#ifdef _DEBUG
	fprintf(fp, "The encrypted blob is %s.\n",(LPSTR)pbDataOutput);
#endif
      }
    }
  }

  RegCloseKey( hKey );

  // Couldn't get password?
  if(lResult != ERROR_SUCCESS)
  {
    return rc;
  }

  // Save Windows password for pGina to use with domains
#ifdef _DEBUG
  fprintf(fp, "Encrypted pGinaDataUSERNAME found\n");
#endif

//-----------------------------------------------------------------
//   Begin unprotect phase.

  if (CryptUnprotectData(
    &DataOut,
    &pDescrOut,
    &_DataEntropy,                 // Optional entropy
    NULL,                 // Reserved
    NULL,        // Optional PromptStruct
    0,
    &DataVerify))
  {
#ifdef _DEBUG
    _ftprintf(fp, TEXT("The decrypted data is: hidden"));
    fprintf(fp, "\n");
    fprintf(fp, "The description of the data was: %S\n",pDescrOut);
#endif
  }
  else
  {
    return rc;
  }

  settingsInfo->Password = _tcsdup((const unsigned short*) DataVerify.pbData);
  rc = true;

  LocalFree(pDescrOut);
  LocalFree(DataOut.pbData);
  LocalFree(DataVerify.pbData);

  return rc;
}

static BOOL setWindowsPassword(LPTSTR Username, LPTSTR Password)
{
  HKEY    hKey;
  LONG    lResult;
  BOOL	  rc = false;

// Encrypt data from DATA_BLOB DataIn to DATA_BLOB DataOut.
// Then decrypt to DATA_BLOB DataVerify.

//--------------------------------------------------------------------
// Declare and initialize variables.

  DATA_BLOB DataIn;
  DATA_BLOB DataOut;
  BYTE *pbDataInput;
  DWORD cbDataInput;

//--------------------------------------------------------------------
//  Begin processing.

  _saveMSPassword = false; // let plugin know the entered password was OK

  pbDataInput = (BYTE*) _tcsdup(Password);
  cbDataInput = (_tcslen(Password)+1) * 2; // *2 for UNICODE
  DataIn.pbData = pbDataInput;    
  DataIn.cbData = cbDataInput;

#ifdef _DEBUG
  fp=fopen("ACEPlugin.log", "a");
  fprintf(fp, "ACEPlugin setWindowsPassword called\n");
  _ftprintf(fp, TEXT("The data to be encrypted is: hidden\n"));
  fprintf(fp, "MSPassword size %d\n",cbDataInput);
#endif

//--------------------------------------------------------------------
//  Begin protect phase.

  if(CryptProtectData(
    &DataIn,
    L"Encrypted MS Password.", // A description string. 
    &_DataEntropy,                               // Optional entropy used.
    NULL,                               // Reserved.
    NULL,                      // Pass a PromptStruct.
    0,
    &DataOut))
  {
#ifdef _DEBUG
    fprintf(fp, "The encryption phase worked. \n");
#endif
    rc = true;
  }
  else
  {
    rc = false;
  }

//-------------------------------------------------------------------

  if (rc)
  {
    lResult = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
			    _T("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon"),
			    0, KEY_WRITE, &hKey );

    if(lResult == ERROR_SUCCESS)
    {
      // save encrypted length and data with the unique name of the userid

      TCHAR tempRegData[25];
      TCHAR tempRegSize[25];
      _stprintf(tempRegData, TEXT("%s%s"), TEXT("pGinaData"), Username);
      _stprintf(tempRegSize, TEXT("%s%s"), TEXT("pGinaSize"), Username);

      lResult = RegSetValueEx(hKey,tempRegSize,0,REG_DWORD,(LPBYTE)&DataOut.cbData,sizeof(DWORD));

      if(lResult == ERROR_SUCCESS)
      {
	lResult = RegSetValueEx(hKey,tempRegData,0,REG_SZ,(CONST BYTE *) DataOut.pbData, DataOut.cbData);
      }
    }

    RegCloseKey( hKey );

    if(lResult != ERROR_SUCCESS)
      rc = false;
  }

#ifdef _DEBUG
  fclose(fp);
#endif

  LocalFree(DataOut.pbData);
  return rc;
}

/*
 * Prompt user for a new Windows password to save
 */
static BOOL passwordPrompt(pGinaInfo *settingsInfo, LPTSTR domain)
{
  HANDLE hUser;
  _saveMSPassword = false;

  // try until user enters correct password (or gives up)
  while (! _saveMSPassword)
  {
    if (IDOK == DialogBox(_AuthInstance, MAKEINTRESOURCE(IDD_DIALOG_WINDOWS_PASSWORD), NULL, (DLGPROC)promptDialogProc))
    {
      settingsInfo->Password = _tcsdup(_promptString);

      // test to see if password is OK
      if(LogonUser(settingsInfo->Username,domain,settingsInfo->Password,LOGON32_LOGON_INTERACTIVE,LOGON32_PROVIDER_DEFAULT, &hUser))
      {
#ifdef _DEBUG
	fprintf(fp, "passwordPrompt - Windows password check succeeded.\n");
#endif
	CloseHandle(hUser);
	_saveMSPassword = true; // LoginHook will save password if login OK
	_lastUsername = _tcsdup(settingsInfo->Username);
      }
      else
      {
#ifdef _DEBUG
	fprintf(fp, "passwordPrompt - Windows password check failed.\n");
#endif
      }
    }
    else
    {
      // prompt cancelled
#ifdef _DEBUG
      fprintf(fp, "passwordPrompt - User cancelled entering a Windows password.\n");
#endif
      return false;
    }
  }

  return true; // good password, save later
}

/*
	@author Deakin University
	
	Taken from NIS Gina project from deakin university, centers a given window on the 
	screen.
*/
VOID CenterWindow(HWND hwnd)
{
  RECT    rect;
  LONG    dx, dy;
  LONG    dxParent, dyParent;
  LONG    Style;

  GetWindowRect(hwnd, &rect);

  dx = rect.right - rect.left;
  dy = rect.bottom - rect.top;

  Style = GetWindowLong(hwnd, GWL_STYLE);
  if ((Style & WS_CHILD) == 0) 
  {
    dxParent = GetSystemMetrics(SM_CXSCREEN);
    dyParent = GetSystemMetrics(SM_CYSCREEN);
  } 
  else 
  {
    HWND    hwndParent;
    RECT    rectParent;

    hwndParent = GetParent(hwnd);
    if (hwndParent == NULL) 
    {
      hwndParent = GetDesktopWindow();
    }

    GetWindowRect(hwndParent, &rectParent);

    dxParent = rectParent.right - rectParent.left;
    dyParent = rectParent.bottom - rectParent.top;
  }

  rect.left = (dxParent - dx) / 2;
  rect.top  = (dyParent - dy) / 3;

  SetWindowPos(hwnd, HWND_TOPMOST, rect.left, rect.top, 0, 0, SWP_NOSIZE);

  SetForegroundWindow(hwnd);
}

// ===========================================================================
// ACE code

/* Prompt with dialog
 * Replace the "safe prompt function" from ACE async example with a dialog.
 * The result of the prompt is stored in _promptString for later retrieval.
 *
 * Globals:
 *	_promptString - user data from IDC_EDIT field
 *	_prompt - label to set in IDC_PROMPT_TEXT field
 * Dialog Fields
 * 	IDOK - simple message dialog
 * 	IDYES - yes/no confirmation dialog
 * 	IDENTER - input dialog, get user data from IDC_EDIT
 * 	IDC_PROMPT_TEXT - set this label field with _prompt value if lParam = 1
 * 	IDC_EDIT - user data field, see IDENTER
 */
BOOL CALLBACK promptDialogProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
  bool success = false;

  switch(message)
  {
  case WM_INITDIALOG:
    if (lParam == 1)
    {
      SetDlgItemText(hwndDlg,IDC_PROMPT_TEXT,(LPCTSTR)_prompt);
    }
    CenterWindow(hwndDlg);
    success = true;
    break;

  case WM_COMMAND:
    switch(LOWORD(wParam))
    {
      // User has entered something and hit OK
    case IDENTER:
      GetDlgItemText(hwndDlg,IDC_EDIT,_promptString,sizeof(_promptString));
    case IDOK:
      // This was a message dialog
    case IDYES:
      // This was a yes/no dialog
      EndDialog(hwndDlg, IDOK);
      success = true;
      break;

    case IDNO:
    case IDCANCEL:
      EndDialog(hwndDlg, IDCANCEL);
      success = true;
      break;

    default:
      success = false;
      break;
    }
  }
  return success;
}

/* PIN prompting function
 * Replace ACE async code with a series of dialogs.
 */
static int promptForPIN( SDI_HANDLE aceHdl, char * buf, const int bufsize, SD_BOOL * bSystemPIN )
{
  SD_PIN      SdPin;
  char        pinCompare[LENMAXPIN];

  *bSystemPIN = SD_FALSE;

// retrieve PIN parameters in one call
  AceGetPinParams(aceHdl, &SdPin);

  switch (SdPin.Selectable)
  {
  case CANNOT_CHOOSE_PIN:
// did they choose to accept system PIN?
    if (IDOK == DialogBox(_AuthInstance, MAKEINTRESOURCE(IDD_DIALOG_NEW_PIN_SYS_STR), NULL, (DLGPROC)promptDialogProc))
    {
      strncpy(buf, SdPin.System, bufsize);
      *bSystemPIN = SD_TRUE;
      return 1;
    }
    
// force a failure
    return 0;

  case USER_SELECTABLE:
// did they choose to accept system PIN?
    if (IDOK == DialogBox(_AuthInstance, MAKEINTRESOURCE(IDD_DIALOG_NEW_PIN_STR), NULL, (DLGPROC)promptDialogProc))
    {
      strncpy(buf, SdPin.System, bufsize);
      *bSystemPIN = SD_TRUE;
      return 1;
    }
// let them enter a PIN
    break;

  case MUST_CHOOSE_PIN:
// did they choose to enter a PIN?
    if (IDOK != DialogBox(_AuthInstance, MAKEINTRESOURCE(IDD_DIALOG_NEW_PIN_USER_STR), NULL, (DLGPROC)promptDialogProc))
    {
      // force a failure, did not enter a PIN
      return 0;
    }
    break;
  }

  if (SdPin.Alphanumeric)
  {
    if (SdPin.Min == SdPin.Max)
      sprintf(_prompt, AUTH_CHALLENGE_NEW_PIN_ALPHA_SAME_STR, SdPin.Max);
    else
      sprintf(_prompt, AUTH_CHALLENGE_NEW_PIN_ALPHA_STR, SdPin.Min, SdPin.Max);
  }
  else
  {
    if (SdPin.Min == SdPin.Max)
      sprintf(_prompt, AUTH_CHALLENGE_NEW_PIN_DIGITS_SAME_STR, SdPin.Max);
    else
      sprintf(_prompt, AUTH_CHALLENGE_NEW_PIN_DIGITS_STR, SdPin.Min, SdPin.Max);
  }
// goto label
  Repeat_PIN:
  // tell dialog to set prompt text from _prompt
  if (IDOK != DialogBoxParam(_AuthInstance, MAKEINTRESOURCE(IDD_DIALOG_NEW_PIN), NULL, (DLGPROC)promptDialogProc, 1))
    return 0;
  WideCharToMultiByte(CP_ACP, 0, _promptString, -1, buf, bufsize, NULL, NULL);

  if (IDOK != DialogBox(_AuthInstance, MAKEINTRESOURCE(IDD_DIALOG_CONFIRM_PIN_STR), NULL, (DLGPROC)promptDialogProc))
    return 0;
  WideCharToMultiByte(CP_ACP, 0, _promptString, -1, pinCompare, sizeof(pinCompare), NULL, NULL);

  if (strcmp(buf, pinCompare) != 0)
  {
    if (IDOK != DialogBox(_AuthInstance, MAKEINTRESOURCE(IDD_DIALOG_ERROR_CONFIRM_PIN_STR), NULL, (DLGPROC)promptDialogProc))
      return 0;
    goto Repeat_PIN;
  }

  return 1;
}

// function where each thread begins execution

#define Thread_Exit(retVal) \
    if (EventData.aceHdl != SDI_HANDLE_NONE) \
    { \
        AceClose(EventData.aceHdl, aceCB); \
        sdWaitForEvent(&EventData.Event, INFINITE); \
    } \
    return (retVal)

static int Authenticate(LPTSTR Username, LPTSTR Password)
//static int Authenticate(void * arg)
{
  int retVal;
  sEventData EventData;
    
  memset(&EventData, 0, sizeof(EventData));
  EventData.aceHdl = SDI_HANDLE_NONE;
    
  sdCreateEvent(&EventData.Event);
    
  // init server communication
    
  retVal = AceInit(&EventData.aceHdl,  (UINT32BIT) &EventData, aceCB);
  if (retVal != ACE_PROCESSING)
  {
#ifdef _DEBUG
    fprintf(fp, "AceInit failed. Returned %d\n",retVal);
#endif
    Thread_Exit(retVal);
  }
#ifdef _DEBUG
  fprintf(fp, "AceInit successful.\n");
#endif
    
  sdWaitForEvent(&EventData.Event, INFINITE);
    
  if ( EventData.asynchAceRet != ACM_OK)
  {
#ifdef _DEBUG
    fprintf(fp, "No response from the ACE/Server.\n");
#endif
    retVal = EventData.asynchAceRet;
    Thread_Exit(EventData.asynchAceRet);
  }

  //nls - convert unicode to ace can handle it
  WideCharToMultiByte(CP_ACP, 0, Username, -1, EventData.user, LENUSERNAME+1, NULL, NULL);
#ifdef _DEBUG
  fprintf(fp, "EventData.user %s\n",EventData.user);
#endif
  retVal = AceSetUsername(EventData.aceHdl, EventData.user);
  if (retVal != ACE_SUCCESS)
  {
#ifdef _DEBUG
    fprintf(fp, "AceSetUsername failed. Returned %d\n",retVal);
#endif
    Thread_Exit(retVal);
  }
#ifdef _DEBUG
  fprintf(fp, "AceSetUsername successful.\n");
#endif

  retVal = AceLock(EventData.aceHdl, aceCB);
    
  if (retVal != ACE_PROCESSING)
  {
#ifdef _DEBUG
    fprintf(fp, "AceLock failed. Returned %d\n",retVal);
#endif
    Thread_Exit(retVal);
  }
#ifdef _DEBUG
  fprintf(fp, "AceSetLock successful.\n");
#endif

  sdWaitForEvent(&EventData.Event, INFINITE);
    
  if ( EventData.asynchAceRet != ACM_OK)
  {
#ifdef _DEBUG
    fprintf(fp, "No response from the ACE/Server.\n");
#endif
    retVal = EventData.asynchAceRet;
    Thread_Exit(EventData.asynchAceRet);
  }
    
  //nls - convert unicode to ace can handle it
  WideCharToMultiByte(CP_ACP, 0, Password, -1, EventData.prn, LENPRNST+1, NULL, NULL);
#ifdef _DEBUG
  fprintf(fp, "EventData.prn hidden\n");
#endif
  retVal = AceSetPasscode(EventData.aceHdl, EventData.prn);
  if (retVal != ACE_SUCCESS)
  {
#ifdef _DEBUG
    fprintf(fp, "AceSetPasscode failed. Returned %d\n",retVal);
#endif
    Thread_Exit(retVal);
  }
#ifdef _DEBUG
  fprintf(fp, "AceSetPasscode successful.\n");
#endif
    
  retVal = AceCheck(EventData.aceHdl, aceCB);
  if (retVal != ACE_PROCESSING)
  {
#ifdef _DEBUG
    fprintf(fp, "AceCheck failed. Returned %d\n",retVal);
#endif
    Thread_Exit(retVal);
  }
#ifdef _DEBUG
  fprintf(fp, "AceCheck successful.\n");
#endif
    
  sdWaitForEvent(&EventData.Event, INFINITE);
  retVal = EventData.asynchAceRet;
    
  switch (retVal)
  {
  case ACM_OK:  // authentication succeeded
#ifdef _DEBUG
    fprintf (fp, "ACM_OK for %s\n", EventData.user);
#endif
    Thread_Exit(retVal);
    break;
        
  case ACM_ACCESS_DENIED:  // authentication was denied
#ifdef _DEBUG
    fprintf (fp, "ACM_ACCESS_DENIED for %s\n", EventData.user);
#endif
    Thread_Exit(retVal);
    break;
        
  case ACM_NEXT_CODE_REQUIRED: // need next tokencode
#ifdef _DEBUG
    fprintf (fp, "ACM_NEXT_CODE_REQUIRED for %s\n", EventData.user);
#endif

    if (IDOK != DialogBox(_AuthInstance, MAKEINTRESOURCE(IDD_DIALOG_NEXT_CODE), NULL, (DLGPROC)promptDialogProc))
    {
      Thread_Exit(ACM_ACCESS_DENIED);
    }
    WideCharToMultiByte(CP_ACP, 0, _promptString, -1, EventData.prn, LENPRNST+1, NULL, NULL);

    retVal = AceSetNextPasscode(EventData.aceHdl, EventData.prn);
    if (retVal != ACE_SUCCESS)
    {
      Thread_Exit(retVal);
    }
        
    retVal = AceSendNextPasscode(EventData.aceHdl, aceCB);
    if (retVal != ACE_PROCESSING)
    {
      Thread_Exit(retVal);
    }
        
    sdWaitForEvent(&EventData.Event, INFINITE);
    retVal = EventData.asynchAceRet;
        
#ifdef _DEBUG
    if (retVal ==  ACM_OK)  // authentication succeeded
      fprintf (fp, "ACM_OK for %s\n", EventData.user);
    else
      fprintf (fp, "ACM_ACCESS_DENIED for %s\n", EventData.user);
#endif
        
    Thread_Exit(retVal);
    break;
        
  case ACM_NEW_PIN_REQUIRED:
  {
#ifdef _DEBUG
    fprintf (fp, "ACM_NEW_PIN_REQUIRED for %s\n", EventData.user);
#endif
    SD_BOOL bSystemPIN = SD_FALSE;
    char PIN[LENMAXPIN];

    if ( 0 == promptForPIN( EventData.aceHdl, PIN, sizeof(PIN), &bSystemPIN ) )
    {
      AceCancelPin(EventData.aceHdl, aceCB);
      sdWaitForEvent(&EventData.Event, INFINITE);
      Thread_Exit(ACM_ACCESS_DENIED);
    }
            
    retVal = AceSetPin(EventData.aceHdl, PIN);
    if (retVal != ACE_SUCCESS)
    {
      AceCancelPin(EventData.aceHdl, aceCB);
      sdWaitForEvent(&EventData.Event, INFINITE);
      Thread_Exit(retVal);
    }
            
    retVal = AceSendPin(EventData.aceHdl,  aceCB);
    if (retVal != ACE_PROCESSING)
    {
      AceCancelPin(EventData.aceHdl, aceCB);
      sdWaitForEvent(&EventData.Event, INFINITE);
      Thread_Exit(retVal);
    }
            
    sdWaitForEvent(&EventData.Event, INFINITE);
    retVal = EventData.asynchAceRet;
            
    if ( retVal != ACM_NEW_PIN_ACCEPTED)
    {
#ifdef _DEBUG
      fprintf(fp, "Failed to set new PIN.\n");
#endif
      Thread_Exit(retVal);
    }

    // if a system PIN has been set display it
    if (bSystemPIN)
    {
      sprintf(_prompt, AUTH_CHALLENGE_NEW_SYS_PIN_DISPLAY_STR, PIN);
      // tell dialog to set prompt text from _prompt
      DialogBoxParam(_AuthInstance, MAKEINTRESOURCE(IDD_DIALOG_NEW_SYS_PIN_DISPLAY_STR), NULL, (DLGPROC)promptDialogProc, 1);
    }
    // lock the username to ask for passcode again
    retVal = AceLock(EventData.aceHdl, aceCB);
            
    if (retVal != ACE_PROCESSING)
    {
      Thread_Exit(retVal);
    }
    sdWaitForEvent(&EventData.Event, INFINITE);
            
    if (IDOK != DialogBox(_AuthInstance, MAKEINTRESOURCE(IDD_DIALOG_NEW_SYS_PIN_PASSCODE_STR), NULL, (DLGPROC)promptDialogProc))
    {
      Thread_Exit(ACM_ACCESS_DENIED);
    }
    WideCharToMultiByte(CP_ACP, 0, _promptString, -1, EventData.prn, LENPRNST+1, NULL, NULL);
            
    retVal = AceSetPasscode(EventData.aceHdl, EventData.prn);
    if (retVal != ACE_SUCCESS)
    {
      Thread_Exit(retVal);
    }
            
    retVal = AceCheck(EventData.aceHdl, aceCB);
    if (retVal != ACE_PROCESSING)
    {
      Thread_Exit(retVal);
    }
            
    sdWaitForEvent(&EventData.Event, INFINITE);
            
#ifdef _DEBUG
    if (EventData.asynchAceRet == ACM_OK)
      fprintf (fp, "ACM_OK for %s\n", EventData.user);
    else
      fprintf (fp, "ACM_ACCESS_DENIED for %s\n", EventData.user);
#endif
            
    retVal = EventData.asynchAceRet;
    Thread_Exit(retVal);
            
    break;
  }
        
  default:
    break;
  }     

  retVal = EventData.asynchAceRet;
  Thread_Exit(retVal);
}

static void WINAPI aceCB(SDI_HANDLE hdl)
{
  sEventData *proc;
    
  //nls if(AceGetUserData(hdl, (UINT32 *) &proc) != ACE_SUCCESS)
  if(AceGetUserData(hdl, (unsigned long *) &proc) != ACE_SUCCESS)
  {
#ifdef _DEBUG
    fprintf(fp, "\n AceGetUserData failed..\n");
#endif
  }
    
  if(AceGetAuthenticationStatus(hdl,(INT32BIT *) &proc->asynchAceRet) != ACE_SUCCESS)
  {
#ifdef _DEBUG
    fprintf(fp, "\n GetAuthenticationStatus failed...\n");
#endif
  }
    
  sdSetEvent(&proc->Event);
}

static SD_I32 sdCreateEvent(P_SDEVENT_S Event)
{
#ifdef WIN32
  // create an EVENT_S object specific to this authentication request
  Event->CondVar = CreateEvent(NULL, SD_FALSE, SD_FALSE, NULL);
  if (! Event->CondVar)
  {
    return ACE_EVENT_CREATE_FAIL;
  }
#endif
#ifdef UNIX
  // create condition variable specific to this request
  if (pthread_cond_init(&Event->CondVar,
			pthread_condattr_default) != 0)
  {
    return ACE_PTHREADCONDVAR_CREATE_FAIL;
  }
    
  // create mutex specific to this request
  if (pthread_mutex_init(&Event->Mutex,
			 pthread_mutexattr_default) != 0)
  {
    return ACE_PTHREADMUTEX_CREATE_FAIL;
  }
  // set event to not signaled at creation
  Event->var = 0;
#endif
  return 0;
}

static void sdDeleteEvent(P_SDEVENT_S Event)
{
#ifdef UNIX
  pthread_cond_destroy(&Event->CondVar);
  pthread_mutex_destroy(&Event->Mutex);
#endif
#ifdef WIN32
  if (Event->CondVar)
    CloseHandle(Event->CondVar);
#endif
}

static SD_I32 sdWaitForEvent(P_SDEVENT_S Event, DWORD ms)
{
#ifdef WIN32
  return (WaitForSingleObject(Event->CondVar, ms));
#endif
    
#ifdef UNIX
    
  int ret = NO_ERROR;
    
  if (pthread_mutex_lock(&Event->Mutex) != 0)
  {
    return 1;
  }
  // if INFINITE timeout then wait for ever
  if (INFINITE == ms)
  {
    // did the event get cleared?
    while (! Event->var)
    {
      // always wait until the flag is clear
      ret = pthread_cond_wait(&Event->CondVar, &Event->Mutex);
      if (ret == NO_ERROR)
      {
	break;
      }
    }
    // clear event always
    Event->var = 0;
  }
  // wait until the timeout time
  else
  {
        
#define MILLISEC_PER_SEC       1000L
#define NANOSEC_PER_MILLISEC   1000000L
#define NANOSEC_PER_SEC        1000000000L
        
    struct timespec delta, timeout;
    struct timeval now;
    struct timezone tz;

    // get current time
    gettimeofday(&now, &tz);
    // calculate the delta used for timeout
    delta.tv_sec = ms / MILLISEC_PER_SEC;
    delta.tv_nsec = (ms % MILLISEC_PER_SEC) * NANOSEC_PER_MILLISEC;
    // get timeout for timed wait
    timeout.tv_sec = now.tv_sec + delta.tv_sec;
    timeout.tv_nsec = (now.tv_usec * 1000) + delta.tv_nsec;
    // if more than a second
    if (timeout.tv_nsec > NANOSEC_PER_SEC)
    {
      timeout.tv_sec ++;              // bump up by one second
      timeout.tv_nsec -= NANOSEC_PER_SEC;
    }
    
    // wait until condition or timeout
    while (! Event->var)
    {
      ret = pthread_cond_timedwait(&Event->CondVar, &Event->Mutex, &timeout);
      // success, or timeout
      if ((ret == 0) || COND_TIMEOUT_ERROR(ret))
      {
	break;
      }
    }
    // success, clear event
    if (ret == NO_ERROR)
    {
      Event->var = 0;
    }
  }
    
  pthread_mutex_unlock(&Event->Mutex);
    
  return ret;
    
#endif
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//
// sdSetEvent
//   This function sets an event object to the signalled state.
//   On UNIX, we used a mutex and a condition variable to "build"
//   the event object.
//
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
static void sdSetEvent(P_SDEVENT_S Event)
{
#ifdef WIN32
  SetEvent(Event->CondVar);
#endif
    
#ifdef UNIX
  int ret;
  if ((ret = pthread_mutex_lock(&Event->Mutex)) != 0)
  {
#ifdef _DEBUG
    fprintf(fp, "\n pthread_mutex_lock return %d\n", ret);
#endif
    return;
  }
    
  Event->var = 1;
  if (pthread_cond_signal(&Event->CondVar) != 0)
  {
    return;
  }
  if (pthread_mutex_unlock(&Event->Mutex) != 0)
  {
    return;
  }
    
#endif
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//
// sdResetEvent
//   This function resets an event object to the non-signalled state.
//   On UNIX, we used a mutex and a condition variable to "build"
//   the event object.
//
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
static void sdResetEvent(P_SDEVENT_S Event)
{
#ifdef WIN32
  ResetEvent(Event->CondVar);
#endif
    
#ifdef UNIX
  if (pthread_mutex_lock(&Event->Mutex) != 0)
  {
    return;
  }
  Event->var = 0;
  if (pthread_mutex_unlock(&Event->Mutex) != 0)
  {
    return;
  }
    
#endif
}
