/*
 *  EmailAuth.cpp
 *
 *  Copyright (C) 2005  Ahmed Obied
 *	
 *  This program is free software; you can redistribute it and/or	
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *  This product includes software developed by the OpenSSL Project
 *  for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 *  In addition, as a special exception, the copyright holders give
 *  permission to link the code of portions of this program with the
 *  OpenSSL library under certain conditions as described in each
 *  individual source file, and distribute linked combinations
 *  including the two.
 *
 *  You must obey the GNU General Public License in all respects
 *  for all of the code used other than OpenSSL.  If you modify
 *  file(s) with this exception, you may extend this exception to your
 *  version of the file(s), but you are not obligated to do so.  If you
 *  do not wish to do so, delete this exception statement from your
 *  version.  If you delete this exception statement from all source
 *  files in the program, then also delete it here.
 *
 */

#include "EmailAuth.h"
#include "resource.h"

HINSTANCE hInstance;
LPTSTR registry_data[REG_DATA_NUM] = {_T("Host"),_T("Port"),_T("Type"),_T("UseSSL")};
LPTSTR protocol_types[PROTOCOL_NUM] = {_T("POP3"),_T("IMAP")};

static PCHAR convert_wchar_to_char(PTCHAR wcstr)
{
	PCHAR cstr;
		
	cstr = (PCHAR)GlobalAlloc(NULL,MAX_BYTES);
	WideCharToMultiByte(CP_ACP,NULL,(LPCWSTR)wcstr,-1,cstr,MAX_BYTES,NULL,NULL);	
	return cstr;
}

static LPTSTR convert_char_to_wchar(PCHAR cstr)
{
	LPTSTR wcstr;
	INT wcstr_size;

	wcstr_size = lstrlenA(cstr);
	wcstr = (LPTSTR)SysAllocStringLen(NULL,wcstr_size);
	MultiByteToWideChar(CP_ACP,0,cstr,wcstr_size,wcstr,wcstr_size);
	return wcstr;
}

static LPTSTR read_registry_string(LPTSTR location)
{
	LONG l_result;
	HKEY h_key;
	DWORD bytes_read = MAX_BYTES;
	TCHAR result[MAX_BYTES];
	TCHAR *ret;
	
	l_result = RegOpenKeyEx(HKEY_LOCAL_MACHINE,REGISTRY_KEY,0,KEY_QUERY_VALUE,&h_key);
	if(l_result == ERROR_SUCCESS) {
    		l_result = RegQueryValueEx(h_key,location,NULL,NULL,(LPBYTE)&result,&bytes_read);
		if(l_result == ERROR_SUCCESS) {
			if(bytes_read > 0) {
				if(!(_tcscmp(result,EMPTY) == 0)) {
					ret = _tcsdup(result);
					RegCloseKey(h_key);
					return ret;
				}
			}
		}
	}
	RegCloseKey(h_key);
	return NULL;
}

static DWORD read_registry_dword(LPTSTR location)
{
	LONG l_result;
	HKEY h_key;
	DWORD value = 0;
	DWORD size = sizeof(DWORD);

	l_result = RegOpenKeyEx(HKEY_LOCAL_MACHINE,REGISTRY_KEY,0,KEY_QUERY_VALUE,&h_key);
	if(l_result == ERROR_SUCCESS) {
    		l_result = RegQueryValueEx(h_key,location,NULL,NULL,(LPBYTE)&value,&size);
			if(l_result == ERROR_SUCCESS) {
				RegCloseKey(h_key);
			        return value;
			}
	}
	RegCloseKey(h_key);
	return 0;
}

static BOOL write_registry_string(LPTSTR value, LPTSTR location)
{
	HKEY h_key;

	if(RegCreateKeyEx(HKEY_LOCAL_MACHINE,REGISTRY_KEY,0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,
		&h_key,NULL) == ERROR_SUCCESS) {
		if(value != NULL)
			if(RegSetValueEx(h_key,location,0,REG_SZ,(CONST BYTE *)value,
				(DWORD)(_tcslen(value)*sizeof(TCHAR))) == ERROR_SUCCESS) {
				RegFlushKey(h_key);
				RegCloseKey(h_key);
				return TRUE;
			 }
	}
	RegFlushKey(h_key);
	RegCloseKey(h_key);
	return TRUE;
}

static BOOL write_registry_dword(DWORD value, LPTSTR location)
{
	HKEY h_key;
	
	if(RegCreateKeyEx(HKEY_LOCAL_MACHINE,REGISTRY_KEY,0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,
		&h_key,NULL) == ERROR_SUCCESS) {
		if(RegSetValueEx(h_key,location,0,REG_DWORD,(LPBYTE)&value,sizeof(DWORD)) == ERROR_SUCCESS) {
			RegFlushKey(h_key);
			RegCloseKey(h_key);
			return TRUE;
		}
	}
	RegFlushKey(h_key);
	RegCloseKey(h_key);
	return FALSE;
}

static VOID restore_settings(HWND c_dlg)
{
	PCHAR tmp;
	LPTSTR host, t_port, type;
	DWORD port, use_ssl;

	tmp = (PCHAR)malloc(MAX_BYTES);
	memset(tmp,STR_TERM,MAX_BYTES);
	if ((host = read_registry_string(registry_data[HOST])) != NULL) 
		SetDlgItemText(c_dlg,IDC_HOST_FIELD,host);
	if ((port = read_registry_dword(registry_data[PORT])) != 0) {
		sprintf(tmp,"%d",port);
		t_port = convert_char_to_wchar(tmp);
		SetDlgItemText(c_dlg,IDC_PORT_FIELD,t_port);
		free(tmp);
	}
	if ((type = read_registry_string(registry_data[TYPE])) != NULL) {
		if (_tcscmp(type,protocol_types[POP3]) == 0) 
			CheckRadioButton(c_dlg,IDC_POP3_RADIO,IDC_IMAP_RADIO,IDC_POP3_RADIO);
		else
		if (_tcscmp(type,protocol_types[IMAP]) == 0) 
			CheckRadioButton(c_dlg,IDC_POP3_RADIO,IDC_IMAP_RADIO,IDC_IMAP_RADIO);
	}
	use_ssl = read_registry_dword(registry_data[USE_SSL]);
	switch (use_ssl) {
		case 0:
			CheckDlgButton(c_dlg,IDC_SSL_CHECK,BST_UNCHECKED);
			break;
		case 1:
			CheckDlgButton(c_dlg,IDC_SSL_CHECK,BST_CHECKED);
	}
}

static BOOL save_settings(HWND c_dlg, WPARAM w_param)
{
	LPTSTR host, port;
	DWORD d_port, use_ssl, protocol_check;

	host = (LPTSTR)malloc(MAX_BYTES);
	port = (LPTSTR)malloc(MAX_BYTES);
	memset(host,STR_TERM,MAX_BYTES);
	memset(port,STR_TERM,MAX_BYTES);
	GetDlgItemText(c_dlg,IDC_HOST_FIELD,host,MAX_BYTES);
	if (_tcscmp(host,EMPTY) == 0) {
		error("Please enter a valid host in the server settings.");
		return FALSE;
	}
	GetDlgItemText(c_dlg,IDC_PORT_FIELD,port,MAX_BYTES);
	if (_tcscmp(port,EMPTY) == 0) {
		error("Please enter a valid port in the server settings.");
		return FALSE;
	}
	d_port = atoi(convert_wchar_to_char(port));
	if (!((d_port >= MIN_PORT_NUM) && (d_port <= MAX_PORT_NUM))) {
		error("You have entered an invalid port number.");
		return FALSE;
	}
	if (!write_registry_string(host,registry_data[HOST])) {
		error("An error has occurred ... \n\nCannot save host data in the registry.");
		return FALSE;
	}
	if (!write_registry_dword(d_port,registry_data[PORT])) {
		error("An error has occurred ... \n\nCannot save port data in the registry.");
		return FALSE;
	}
	protocol_check = ERR;
	if (IsDlgButtonChecked(c_dlg,IDC_POP3_RADIO) == BST_CHECKED) {
		protocol_check = POP3;
		if (!write_registry_string(protocol_types[POP3],registry_data[TYPE])) {
			error("An error has occurred ... \n\nCannot save pop3 type in the registry.");
			return FALSE;
		}
	}
	else
	if (IsDlgButtonChecked(c_dlg,IDC_IMAP_RADIO) == BST_CHECKED) {
		protocol_check = IMAP;
		if (!write_registry_string(protocol_types[IMAP],registry_data[TYPE])) {
			error("An error has occurred ... \n\nCannot save imap type in the registry.");
			return FALSE;
		}
	}
	if (protocol_check == ERR) {
		error("Please select a protocol type.");
		return FALSE;
	}
	if (IsDlgButtonChecked(c_dlg,IDC_SSL_CHECK) == BST_UNCHECKED)
		use_ssl = 0;
	else
		use_ssl = 1;
	if (!write_registry_dword(use_ssl,registry_data[USE_SSL])) {
		error("An error has occurred ... \n\nCannot save ssl option in the registry.");
		return FALSE;
	}
	free(host);
	free(port);
	EndDialog(c_dlg,LOWORD(w_param));
	return TRUE;
}

INT_PTR CALLBACK server_settings(HWND c_dlg, UINT msg, WPARAM w_param, LPARAM l_param)
{
	switch(msg) {
		case WM_INITDIALOG:
			restore_settings(c_dlg);
		break;
		case WM_COMMAND: {
			switch (LOWORD(w_param)) {
				case IDC_SAVE_BUTTON: {
					if (save_settings(c_dlg,LOWORD(w_param))) {
						EndDialog(c_dlg,LOWORD(w_param));
						MessageBox(GetForegroundWindow(),
						_T("Settings were successfully saved in the registry"),
						NAME,
						MB_OK);
					}
				}
				break;
				case IDC_CANCEL_BUTTON:
					EndDialog(c_dlg,LOWORD(w_param));
				break;
			}
		}
		break;
		case WM_CLOSE:
			EndDialog(c_dlg,LOWORD(w_param));
	}
	return FALSE;
}

static LPTSTR get_pop3_auth_msg(LPTSTR username, LPTSTR password)
{
	LPTSTR msg;

	msg = (LPTSTR)malloc(MAX_BYTES);
	memset(msg,STR_TERM,MAX_BYTES);
	if (password == NULL) {
		_tcscat(msg,_T("USER"));
		_tcscat(msg,SPACE);
		_tcscat(msg,username);
	}
	else
	if (username == NULL) {
		_tcscat(msg,_T("PASS"));
		_tcscat(msg,SPACE);
		_tcscat(msg,password);
	}
	_tcscat(msg,CMD_END);
	return msg;
}

static LPTSTR get_imap_auth_msg(LPTSTR username, LPTSTR password) 
{
	LPTSTR msg;

	msg = (LPTSTR)malloc(MAX_BYTES);
	memset(msg,STR_TERM,MAX_BYTES);
	_tcscat(msg,IMAP_MSG_ID);
	_tcscat(msg,SPACE);
	_tcscat(msg,_T("LOGIN"));
	_tcscat(msg,SPACE);
	_tcscat(msg,username);
	_tcscat(msg,SPACE);
	_tcscat(msg,password);
	_tcscat(msg,CMD_END);
	return msg;
}

static BOOL send_packet_to_server(DWORD sock_ds, PCHAR data, DWORD use_ssl, SSL *ssl)
{
	DWORD err;

	if (use_ssl) {
		err = SSL_write(ssl,data,strlen(data));
		if (err <= 0) {
			error("An error has occurred ... \n\nSSL_write call failed.\n");
			return FALSE;
		}
	}
	else {
		err = send(sock_ds,data,strlen(data),0);
		if (err == SOCKET_ERROR) {
			error("An error has occurred ... \n\nsend call failed.\n");
			return FALSE;
		}
	}
	return TRUE;
}

static BOOL recv_packet_from_server(int sock_ds, PCHAR data, DWORD use_ssl, SSL *ssl)
{
	DWORD err;

	memset(data,STR_TERM,MAX_BYTES);
	if (use_ssl) {
		err = SSL_read(ssl,data,MAX_BYTES);
		if (err <= 0) {
			error("An error has occurred ... \n\nSSL_read call failed.\n");
			return FALSE;
		}
	}
	else {
		err = recv(sock_ds,data,MAX_BYTES,0);
		if (err == SOCKET_ERROR) {
			error("An error has occurred ... \n\nrecv call failed.\n");
			return FALSE;
		}
	}
	return TRUE;
}

BOOL APIENTRY DllMain(HANDLE handle, DWORD reason, LPVOID reserved)
{
	if (reason == DLL_PROCESS_ATTACH) {
		hInstance = (HINSTANCE)handle;
	}
	return TRUE;
}

EMAILAUTH_API BOOL UserLogin(LPTSTR username, LPTSTR password, pGinaInfo *settingsInfo)
{
	BOOL auth_check;
	WSADATA wsa_data;
	SOCKET sock_ds;
	DWORD err, port, use_ssl;
	PCHAR str_port, recv_packet;
	LPTSTR host, protocol, send_packet;
	ADDRINFO *addr_info;
	FD_SET sock_set;
	TIMEVAL timeout;
	SSL *ssl;
	SSL_CTX *ssl_ctx;
	SSL_METHOD *ssl_method;

	auth_check = FALSE;
	host = (LPTSTR)malloc(MAX_BYTES);
	str_port = (PCHAR)malloc(MAX_BYTES);
	recv_packet = (PCHAR)malloc(MAX_BYTES);
	memset(str_port,STR_TERM,MAX_BYTES);
	memset(host,STR_TERM,MAX_BYTES);
	if ((host = read_registry_string(registry_data[HOST])) == NULL)
		return FALSE;
	if ((port = read_registry_dword(registry_data[PORT])) == 0)
		return FALSE;
	if ((protocol = read_registry_string(registry_data[TYPE])) == NULL)
		return FALSE;
	use_ssl = read_registry_dword(registry_data[USE_SSL]);
	sprintf(str_port,"%d",port);
	err = WSAStartup(WINSOCK_VERSION,&wsa_data);
	if (err != 0) {
		error("An error has occurred ... \n\nWSAStartup call failed.\n");
		return FALSE;
	}
	err = getaddrinfo(convert_wchar_to_char(host),str_port,NULL,&addr_info);
	if (err != 0) {
		error("An error has occurred ... \n\ngetaddrinfo call failed.\n");
		return FALSE;
	}
	sock_ds = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
	if (sock_ds == INVALID_SOCKET) {
		error("An error has occurred ... \n\nsocket call failed.\n");
		return FALSE;
	}
	err = connect(sock_ds,addr_info->ai_addr,addr_info->ai_addrlen);
	if (err == SOCKET_ERROR) {
		error("An error has occurred ... \n\nconnect call failed.\n");
		return FALSE;
	}
	if (use_ssl) {
		char *seed_file;
		const char *path_to_seed_file;

		SSL_library_init();
		ssl_method = SSLv23_client_method();
		if (ssl_method == NULL) {
			error("An error has occurred ... \n\nSSLv23_client_method call failed.\n");
			return FALSE;
		}
		ssl_ctx = SSL_CTX_new(ssl_method);
		if (ssl_ctx == NULL) {
			error("An error has occurred ... \n\nSSL_CTX_new call failed.\n");
			return FALSE;
		}
		seed_file = (char *)malloc(MAX_BYTES);
		memset(seed_file,STR_TERM,MAX_BYTES);
		path_to_seed_file = RAND_file_name(seed_file,MAX_BYTES);
		if (path_to_seed_file == NULL) {
			error("An error has occurred ... \n\nRAND_file_name call failed.\n");
			return FALSE;
		}
		if (RAND_load_file(path_to_seed_file,-1) < 0) {
			error("An error has occurred ... \n\nRAND_load_file call failed.\n");
			return FALSE;
		}
		ssl = SSL_new(ssl_ctx);
		if (ssl == NULL) {
			error("An error has occurred ... \n\nSSL_new call failed.\n");
			return FALSE;
		}
		if (!SSL_set_fd(ssl,sock_ds)) {
			error("An error has occurred ... \n\nSSL_set_fd call failed.\n");
			return FALSE;
		}
		if (SSL_connect(ssl) <= 0) {
			error("An error has occurred ... \n\nSSL_connect call failed.\n");
			return FALSE;
		}
	}
	FD_ZERO(&sock_set);
	FD_SET(sock_ds,&sock_set);
	timeout.tv_sec = TIMEOUT;
	timeout.tv_usec = 0;
	err = select(0,&sock_set,NULL,NULL,&timeout);
	if (err == SOCKET_ERROR) {
		error("An error has occurred ... \n\nselect call failed.\n");
		return FALSE;
	}
	if (FD_ISSET(sock_ds,&sock_set)) {
		if (!recv_packet_from_server(sock_ds,recv_packet,use_ssl,ssl))
			return FALSE;
	}
	else {
		error("Operation Timeout ...\n\nYou are attempting to connect to either POP3s or IMAPs without using SSL");
		return FALSE;
	}
	if (_tcscmp(protocol,protocol_types[POP3]) == 0) {
		send_packet = get_pop3_auth_msg(username,NULL);
		if (!send_packet_to_server(sock_ds,convert_wchar_to_char(send_packet),use_ssl,ssl))
			return FALSE;
		if (!recv_packet_from_server(sock_ds,recv_packet,use_ssl,ssl))
			return FALSE;
		free(send_packet);
		send_packet = get_pop3_auth_msg(NULL,password);
		if (!send_packet_to_server(sock_ds,convert_wchar_to_char(send_packet),use_ssl,ssl))
			return FALSE;
		if (!recv_packet_from_server(sock_ds,recv_packet,use_ssl,ssl))
			return FALSE;
		free(send_packet);
		if ((recv_packet[0] == '+') && (recv_packet[1] == 'O') && (recv_packet[2] == 'K'))
			auth_check = TRUE;
	}
	else
	if (_tcscmp(protocol,protocol_types[IMAP]) == 0) {
		send_packet = get_imap_auth_msg(username,password);
		if (!send_packet_to_server(sock_ds,convert_wchar_to_char(send_packet),use_ssl,ssl))
			return FALSE;
		if (!recv_packet_from_server(sock_ds,recv_packet,use_ssl,ssl))
			return FALSE;
		free(send_packet);
		if ((recv_packet[6] == 'O') && (recv_packet[7] == 'K'))
			auth_check = TRUE;
	}
	err = WSACleanup();
	if (err == SOCKET_ERROR) {
		error("An error has occurred ... \n\nWSACleanup call failed.\n");
		return FALSE;
	}
	if (use_ssl) {
		SSL_free(ssl);
		SSL_CTX_free(ssl_ctx);
	}
	free(recv_packet);
	free(host);
	free(str_port);
	return auth_check;
}

EMAILAUTH_API BOOL ChangeUserPassword(LPTSTR username, LPTSTR old_pwd, LPTSTR new_pwd)
{
	return FALSE;
}

EMAILAUTH_API LPCTSTR AboutPlugin()
{
	LPCTSTR about = _T("Email Authentication plugin for pGINA by Ahmed Obied (ahmed.obied@gmail.com)");
	return about;
} 

EMAILAUTH_API VOID ChangePluginSettings()
{
	DialogBox(hInstance,MAKEINTRESOURCE(IDD_DIALOG),NULL,server_settings);	
}

EMAILAUTH_API VOID LoginHook(pGinaInfo *settings_info)
{
	
}

EMAILAUTH_API BOOL IsRequired()
{
	return FALSE;
}

EMAILAUTH_API VOID LogoutHook(pGinaInfo *settings_info)
{

}