/*
 *  EmailAuth.h
 *
 *  Copyright (C) 2005  Ahmed Obied
 *	
 *  This program is free software; you can redistribute it and/or	
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *  This product includes software developed by the OpenSSL Project
 *  for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 *  In addition, as a special exception, the copyright holders give
 *  permission to link the code of portions of this program with the
 *  OpenSSL library under certain conditions as described in each
 *  individual source file, and distribute linked combinations
 *  including the two.
 *
 *  You must obey the GNU General Public License in all respects
 *  for all of the code used other than OpenSSL.  If you modify
 *  file(s) with this exception, you may extend this exception to your
 *  version of the file(s), but you are not obligated to do so.  If you
 *  do not wish to do so, delete this exception statement from your
 *  version.  If you delete this exception statement from all source
 *  files in the program, then also delete it here.
 *
 */

#ifndef _EMAIL_AUTH_HEADER_
#define _EMAIL_AUTH_HEADER_

#include <winsock2.h>
#include <ws2tcpip.h>
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <tchar.h>
#include <openssl/ssl.h>
#include <openssl/rand.h>

#ifdef EMAILAUTH_EXPORTS
#define EMAILAUTH_API extern "C" __declspec(dllexport)
#else
#define EMAILAUTH_API extern "C" __declspec(dllimport)
#endif

#define MAX_BYTES		1024
#define MIN_PORT_NUM		1
#define MAX_PORT_NUM		65535
#define REG_DATA_NUM		4
#define PROTOCOL_NUM		2
#define ERR			-1
#define TIMEOUT			10
#define REGISTRY_KEY		_T("SOFTWARE\\pGina\\EmailAuth")
#define STR_TERM		_T('\0')
#define EMPTY			_T("")
#define SPACE			_T(" ")
#define NAME			_T("Email Authentication")
#define CMD_END			_T("\r\n")
#define IMAP_MSG_ID		_T("a0001")

#define error(x) { \
	MessageBox(GetForegroundWindow(),_T(x),NAME,MB_ICONERROR); \
}

enum PROTOCOL_TYPES
{
	POP3 = 0,
	IMAP = 1
};

enum REG_DATA
{
	HOST	= 0,
	PORT	= 1,
	TYPE	= 2,
	USE_SSL = 3
};

typedef struct pGinaInfo {
	LPTSTR pathMSGina;
	LPTSTR pathPlugin;
	LPTSTR pathProfile;
	LPTSTR mapPaths;
	LPTSTR Username;
	LPTSTR Password;
	LPTSTR homeDir;
	BOOL isAdmin;
	BOOL disabled;
	INT authType;
	HANDLE hUser;
	LPTSTR userGroups;
	LPTSTR userDescription;
	LPTSTR userFullName;
	BOOL allowPassChange;
	LPTSTR errorString;
	LPTSTR defaultDomain;
	BOOL Reserved3;
	BOOL Reserved4;
} pGinaInfo;

EMAILAUTH_API BOOL UserLogin(LPTSTR,LPTSTR,pGinaInfo *);			
EMAILAUTH_API BOOL ChangeUserPassword(LPTSTR,LPTSTR,LPTSTR);
EMAILAUTH_API LPCTSTR AboutPlugin();
EMAILAUTH_API VOID ChangePluginSettings();
EMAILAUTH_API VOID LoginHook(pGinaInfo *);
EMAILAUTH_API VOID LogoutHook(pGinaInfo *);
EMAILAUTH_API BOOL IsRequired();

#endif