#include <stdafx.h>
#include "LDAPGroupPlugin.h"

LDAP *ldapConnection;

bool connect_to_ldap(void) {
	char *server = convert_to_char(regReadString(_T("server")));
	char *port = convert_to_char(regReadString(_T("port")));
	int port_num = atoi(port);
    int ldap_version = LDAP_VERSION3;
	
	// Set our options accordingly (for version and initial time out)
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &ldap_version);

    // Set it up and initialize the connection
    if((ldapConnection = ldap_init(server,port_num)) == NULL) {
        return false;
    }

	if(ldap_simple_bind_s( ldapConnection, NULL, NULL ) != LDAP_SUCCESS) {
		return false;
    } 

	return true;
}

void disconnect(void) {
	ldap_unbind_s(ldapConnection);
}

LDAPMessage *GetGroups(char *username) {
	int return_code = 0;
    LDAPMessage *search_results = NULL;

	char *base_dn = convert_to_char(regReadString(_T("base_dn")));
	char *filter = convert_to_char(regReadString(_T("filter")));
	char filter_buffer[2048];
	memset(filter_buffer,0,sizeof(filter_buffer));
	sprintf(filter_buffer,"(%s=%s)",filter,username);
	// Search on the group container for all groups that have a MEMBER with USERNAME
    return_code = ldap_search_s(ldapConnection,     // LDAP Connection
								base_dn,     // Base DN
                                LDAP_SCOPE_SUBTREE,              // Scope
                                filter_buffer,     // Filter
                                NULL,               // Specific attributes, NULL = all
                                0,                  // Give me both attributes AND values
                                &search_results);   // Somewhere to store the results

    
    if(return_code != LDAP_SUCCESS) {
        return NULL;
    }

    return search_results;
}

char *getPrimaryGroup(char *username) {
	LDAPMessage *search_results = NULL;
	char *users_gid = NULL;
	char *users_dn = NULL;
	BerElement *ber = NULL;
	int return_code = 0;
	LDAPMessage *entry = NULL;
	char *attribute = NULL;
	char **values = NULL;
	char *attribute_list[] = { convert_to_char(regReadString(_T("groupAttribute"))) , NULL };
	
	// Get the users container
	char filter_buffer[1024];
	sprintf(filter_buffer,"(%s=%s)",convert_to_char(regReadString(_T("user_attribute"))),username);

	return_code = ldap_search_s(ldapConnection,     // LDAP Connection
								convert_to_char(regReadString(_T("user_container"))),     // Base DN
                                LDAP_SCOPE_SUBTREE,              // Scope
                                filter_buffer,     // Filter
                                attribute_list,               // Specific attributes, NULL = all
                                0,                  // Give me both attributes AND values
                                &search_results);   // Somewhere to store the results

	if(return_code != LDAP_SUCCESS) {
        return NULL;
    }

	// We only look at the first return.. there *should* only be one anyway
	if(search_results == NULL) 
		return NULL;

	entry = ldap_first_entry(ldapConnection, search_results);
	
	if(entry == NULL) {
		ldap_msgfree(search_results);
		return NULL;
	}

	users_dn = ldap_get_dn(ldapConnection, entry);
	
	if(users_dn == NULL) {
		ldap_msgfree(search_results);
		return NULL;
	}

	attribute = ldap_first_attribute(ldapConnection,entry,&ber);

	if(attribute == NULL) {
		ldap_msgfree(search_results);
		return NULL;
	}

	values = ldap_get_values(ldapConnection,entry,attribute);

	if(values == NULL) {
		ldap_msgfree(search_results);
		ldap_memfree(attribute);
		ber_free(ber,0);
		return NULL;
	}

	users_gid = strdup(values[0]);
	ldap_value_free(values);
	ldap_memfree(attribute);
	ber_free(ber, 0);

	// Now that we have the GID, search for the group DN with that GID
	memset(filter_buffer,0,sizeof(filter_buffer));
	sprintf(filter_buffer,"(%s=%s)",convert_to_char(regReadString(_T("groupAttribute"))),users_gid);
	return_code = ldap_search_s(ldapConnection,
								convert_to_char(regReadString(_T("base_dn"))),
								LDAP_SCOPE_SUBTREE,
								filter_buffer,
								NULL,
								0,
								&search_results);

	if(search_results == NULL)
		return NULL;

	entry = ldap_first_entry(ldapConnection, search_results);
	
	if(entry == NULL) {
		ldap_msgfree(search_results);
		return NULL;
	}

	users_dn = ldap_get_dn(ldapConnection, entry);
	
	if(users_dn == NULL) {
		ldap_msgfree(search_results);
		return NULL;
	}
	
	return users_dn;
}

