#include <stdafx.h>
#include "LDAPGroupPlugin.h"


// Search the adminGroup registry entries for an entry that matches that passed
bool isAdmin(char *group_dn) {
	TCHAR key_name[2048];
	char *key_value = NULL;
	int key_number = 0;
	bool found_flag = false;

	for(;key_number < 256; key_number++) {
		// Format for our next key
		wsprintf(key_name,_T("adminGroup%d"),key_number);
		key_value = convert_to_char(regReadString(key_name));

		// Make sure it has a value
		if(key_value == NULL)
			goto Finish;

		// See if it compares
		if(strcmp(key_value,group_dn) == 0) {
			found_flag = true;
			goto Finish;
		}
	}

Finish:
	return found_flag;
}

// Search userGroup entries for the given dn
bool isUser(char *group_dn) {
	TCHAR key_name[2048];
	char *key_value = NULL;
	int key_number = 0;
	bool found_flag = false;

	for(;key_number < 256; key_number++) {
		// Format for our next key
		wsprintf(key_name,_T("userGroup%d"),key_number);
		key_value = convert_to_char(regReadString(key_name));

		// Make sure it has a value
		if(key_value == NULL)
			goto Finish;

		// See if it compares
		if(strcmp(key_value,group_dn) == 0) {
			found_flag = true;
			goto Finish;
		}
	}

Finish:
	return found_flag;
}

/**
 * Read a string value from the HKLM\\Software\\pGina\\USBKeyPlugin registry.
 * 
 * @param location Name of the registry entry to read
 * @return A TCHAR buffer containing the value, or NULL if empty/error
 */
TCHAR * regReadString(LPCTSTR location)
{
    LONG lResult;
	HKEY hKey;
    DWORD dwBytesRead = 2048;
    TCHAR result[2048];
    TCHAR *ret;

    lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\LDAPGroupPlugin"),0,KEY_QUERY_VALUE,&hKey);
    
    if(lResult == ERROR_SUCCESS)
    {
    	lResult = RegQueryValueEx(hKey,location,NULL,NULL,(LPBYTE)&result,&dwBytesRead);
        
        if(lResult == ERROR_SUCCESS)
        {
            if(dwBytesRead > 0)
            {
                if(!_tcscmp(result,TEXT(""))) // If its just a space
				{
					RegCloseKey(hKey);
				    return NULL;                                                                                 
				}
				else
				{
					ret = _tcsdup(result);
					RegCloseKey(hKey);
					return ret;
				}
            }
            else
            {
                RegCloseKey(hKey);
                return NULL;
            }
        }
        
    }
    RegCloseKey(hKey);
    return NULL;
}

/**
 * Write a string to the HKLM\\Software\\pGina\\USBKeyPlugin registry.
 * 
 * @param value Value to write (string)
 * @param location Name of the value to write
 *
 * @return True if written, false otherwise
 */
BOOL regWriteString(LPTSTR value,LPCTSTR location)
{
    HKEY hKey;
    
    if(RegCreateKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\LDAPGroupPlugin"),0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&hKey,NULL) == ERROR_SUCCESS)
    {
        if(value != NULL) {
            if(RegSetValueEx(hKey,location,0,REG_SZ,(CONST BYTE *) value, _tcslen(value)*sizeof(TCHAR)) == ERROR_SUCCESS)
            {
                RegFlushKey(hKey);
                RegCloseKey(hKey);
                return true;
            }
        } else {
            if(RegSetValueEx(hKey,location,0,REG_SZ,(CONST BYTE *) _T("\0"), _tcslen(_T("\0"))*sizeof(TCHAR)) == ERROR_SUCCESS)
            {
                RegFlushKey(hKey);
                RegCloseKey(hKey);
                return true;
            }
        }
    }
    
    RegFlushKey(hKey);
    RegCloseKey(hKey);
    return false;
}


/**
 * Read a boolean value (DWORD) from HKLM\\Software\\pGina\\USBKeyPlugin registry.
 *  DWORD values of 1 are considered "True", all else are "False"
 *
 * @param Name of the value to read
 * @return True or False
 */
BOOL regReadBool(LPCTSTR location)
{
    LONG lResult;
	HKEY hKey;
    DWORD value = 0;
    DWORD size = sizeof(DWORD);

    lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\LDAPGroupPlugin"),0,KEY_QUERY_VALUE,&hKey);
    
    if(lResult == ERROR_SUCCESS)
    {
    	lResult = RegQueryValueEx(hKey,location,NULL,NULL,(LPBYTE)&value,&size);
        
        if(lResult == ERROR_SUCCESS)
        {
			RegCloseKey(hKey);
            if(value == 1) 
                return true;
            else
                return false;
        }
        
    }
    RegCloseKey(hKey);
    return false;
}

/** 
 * Write a booean value as a DWORD into the HKLM\\Software\\pGina\\USBKeyPlugin registry.
 *
 * @param value Boolean value to write
 * @param location name of the value to write
 */
BOOL regWriteBool(BOOL value,LPCTSTR location)
{
    HKEY hKey;
    DWORD write;

    write = (value ? 1 : 0);
    
    if(RegCreateKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\LDAPGroupPlugin"),0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,  &hKey,NULL) == ERROR_SUCCESS)
        if(RegSetValueEx(hKey,location,0,REG_DWORD,(const BYTE*)&write,sizeof(DWORD)) == ERROR_SUCCESS)
        {
            RegFlushKey(hKey);
            RegCloseKey(hKey);
            return true;
        }
    RegFlushKey(hKey);
    RegCloseKey(hKey);
    return false;
}


/** 
 * Convert from a single byte (ANSI-ASCII) to a Multibyte (UNICODE) string
 * 
 * @param from The single byte string to convert
 * @return A unicode string, or NULL
 */
LPTSTR convert_to_wchar(const char *from)
{
	TCHAR *to;
	if(from == NULL)
		return NULL;
	
	to = (TCHAR *) malloc(strlen(from)*sizeof(TCHAR) + 1);
	MultiByteToWideChar(CP_ACP,
						NULL,
						(LPCSTR)from,
						strlen(from),
						(LPWSTR)to,
						strlen(from)*sizeof(TCHAR));					

	return to;
}

/** 
 * Convert from a multi-byte (unicode) to a single byte (ascii) string.
 * 
 * @param from The multibyte string to convert
 * @return An ascii string (char *), or NULL
 */
char * convert_to_char(const TCHAR *from)
{
	if(from == NULL)
		return NULL;

	char *to = (char *) GlobalAlloc(NULL,1024);
	
	WideCharToMultiByte(CP_ACP,
						NULL,
						(LPCWSTR)from,
						-1,
						to,
						1024,
						NULL,
						NULL);
	
	return to;
}