/*
pGina PAM Plugin for pGina
Copyright (C) 2003 Nathan Yocom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: nate.yocom@xpasystems.com
Web: http://pgina.xpasystems.com
Snail Mail:
  Nathan Yocom 
  9 Evergreen Farms Rd.
  Scarborough, ME 04074
  Phone: 207-450-4948
*/

// PAMPlugin.cpp : Defines the initialization routines for the DLL.
//

#include "stdafx.h"
#include "PAMPlugin.h"
#include "PAMPluginConfig.h"
#include "ChangePass.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//#include <wchar.h>
//#include <tchar.h>

#include <openssl/ssl.h>

//#include <string>

// Our messages for sending to the server
#define AUTH_ACTION "AUTH"
#define SUCCESS "SUCC"
#define EXPIRED "EXP"
#define ERROR "ERR"
#define CHANGE_PASS "CHPS"
#define GET_GROUP "GGRP"
#define LOCKEDOUT "LKD"
#define DONE "!"

char * glPassword;

CTestApp theApp;

int check_login(LPTSTR Username,LPTSTR Password,pGinaInfo *settingsInfo)
{
	SSL *ssl;
	char *result;	
	char *admin_groups;
	char *current_group;
	bool authenticated = false;

	settingsInfo->allowPassChange = false;
	settingsInfo->isAdmin = false;

	ssl = connect_to_server(settingsInfo);
	
	if(!ssl)
	{
		return 0;
	}
	// If we get this far, we are connected and can begin our authentication

	// Tell the server what action we are doing
	send_string(ssl,AUTH_ACTION);
	// Then send username and password (seperatley)
	send_string(ssl,convert_to_char(settingsInfo->Username));
	send_string(ssl,convert_to_char(settingsInfo->Password));
	
	result = read_string(ssl);
	

	if(!strcmp(result,SUCCESS))
	{
		// Now process group membership etc
		if(regReadBool("reflect_group") || regReadBool("create_admins"))
		{
			send_string(ssl,GET_GROUP);
			send_string(ssl,convert_to_char(settingsInfo->Username));
			result = read_string(ssl); // Get the resulting group

			if(strcmp(result,ERROR) && regReadBool("reflect_group")) 
				settingsInfo->userGroups = convert_to_wchar(result);

			if(strcmp(result,ERROR) && regReadBool("create_admins"))
			{
				// Check to see if the resulting group is in the list of admin groups
				//  admin_groups looks like: group1,group2,group3
				//  probably a pretty massive mem leak here.. but hey, we get unloaded later right? :)

				// First read the entyr
				admin_groups = regReadString("admin_group");
				// As long as we got something
				if(admin_groups != NULL) 
				{
					// Get the first group
					current_group = strtok(admin_groups,",");
					// So long as there are groups to get
					while(current_group != NULL) {
						// If this group matches the one the user is in, they are an admin
						if(strcmp(result,current_group) == 0)
						{
							settingsInfo->isAdmin = true;
							settingsInfo->allowPassChange = true;
						}
						current_group = strtok(NULL,",");
					}
				}
				
				/* The Old Way
				if(!strcmp(result,regReadString("admin_group")))
				{
					settingsInfo->isAdmin = true;
					settingsInfo->allowPassChange = true;
				}*/
			}

			if(regReadBool("change_pass"))
				settingsInfo->allowPassChange = true;
		}

		send_string(ssl,DONE);
		SSL_shutdown(ssl);
		return 1;
	}
	else
	{
		if(!strcmp(result,EXPIRED))
		{
			send_string(ssl,DONE);
			SSL_shutdown(ssl);
			return -1; // Return expired
		}
		else
		{
			send_string(ssl,DONE); // Tell the server we need nothing more right now
			SSL_shutdown(ssl);
			return 0;
		}
	}
}

/* This is the function that pGina calls to find out whether a user is authorized or not
	it is also the place to put changes in per-user configurations etc. */
PAMPLUGIN_API BOOL UserLogin(LPTSTR Username, LPTSTR Password, pGinaInfo *settingsInfo)
{
	int retval = 0;
	pGinaInfo pInfo;
	retval = check_login(Username,Password,settingsInfo);

	switch(retval) {
	case -1:
		glPassword = convert_to_char(settingsInfo->Password);
		show_change_pass();
		if(glPassword != NULL)
		{
			if(change_pass(Username,Password,glPassword))
			{
				// Messageboxes just end up behind pgina.. when it works they'll know
				//MessageBox(NULL,"You password was changed and updated successfully","Success",MB_OK|MB_ICONINFORMATION);
				pInfo.Username = settingsInfo->Username;
				pInfo.Password = convert_to_wchar(glPassword);
				retval = check_login(Username,convert_to_wchar(glPassword),&pInfo);
			}
			else 
			{
				settingsInfo->errorString = convert_to_wchar("Unable to reset your password.  Please contact a system administrator for details");
				retval = 0;
			}
		}
		else
		{
			settingsInfo->errorString = convert_to_wchar("You need to reset your password before you can be admitted.");
			retval = 0;
		}

		if(retval != 1)
			return false;
		else
		{
			// If we get here, then pInfo holds the info we want
			settingsInfo->isAdmin = pInfo.isAdmin;
			settingsInfo->allowPassChange = pInfo.allowPassChange;
			settingsInfo->Password = convert_to_wchar(glPassword);
			return true;
		}
		break;
	case 0:
		return false;
		break;
	case 1:
		return true;
		break;
	}
	
	return false;
}

void show_change_pass(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());	
	CChangePass MyDialog;
	MyDialog.DoModal();
}

bool change_pass(LPTSTR Username, LPTSTR OldPassword,char *NewPassword)
{
	pGinaInfo settingsInfo;
	SSL *ssl = connect_to_server(&settingsInfo);
	char *result = NULL;

	if(!ssl)
		return false;  // Had a problem, fail out

	send_string(ssl,CHANGE_PASS); // Tell the server what we want to do
	send_string(ssl,convert_to_char(Username)); // Now we need to auth the username and pass
	send_string(ssl,convert_to_char(OldPassword)); // Old password to verify who we are
	result = read_string(ssl);	// Find out what the server says
	send_string(ssl,NewPassword); // Send our new pass
	result = read_string(ssl); // Find out whether the change works or not
	send_string(ssl,DONE);
	SSL_shutdown(ssl);

	if(!strcmp(result,SUCCESS))
		return true;
	else
	{
		return false;
	}
	
}
PAMPLUGIN_API BOOL ChangeUserPassword(LPTSTR Username,LPTSTR OldPassword,LPTSTR NewPassword)
{
	pGinaInfo settingsInfo;
	SSL *ssl = connect_to_server(&settingsInfo);
	char *result = NULL;

	if(!ssl)
		return false;  // Had a problem, fail out

	send_string(ssl,CHANGE_PASS); // Tell the server what we want to do
	send_string(ssl,convert_to_char(Username)); // Now we need to auth the username and pass
	send_string(ssl,convert_to_char(OldPassword)); // Old password to verify who we are
	result = read_string(ssl);	// Find out what the server says

	if(!strcmp(result,SUCCESS))
	{
		send_string(ssl,convert_to_char(NewPassword)); // Send our new pass
		result = read_string(ssl); // Find out whether the change works or not
		send_string(ssl,DONE);
		SSL_shutdown(ssl);

		if(!strcmp(result,SUCCESS))
			return true;
		else
		{
			return false;
		}
	}
	else
	{
		// We werent able to prove our identity
		send_string(ssl,DONE);
		SSL_shutdown(ssl);
		return false;
	}
}

PAMPLUGIN_API LPCTSTR AboutPlugin(void)
{
	// Return a sample descriptions string
	return convert_to_wchar(PAMPLUGIN_ABOUT);
}
PAMPLUGIN_API void ChangePluginSettings(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());	
	PAMPluginConfig MyDialog;
	MyDialog.DoModal();
}
PAMPLUGIN_API void LoginHook(pGinaInfo *settingsInfo)
{
	
}
PAMPLUGIN_API void LogoutHook(pGinaInfo *settingsInfo)
{

}

PAMPLUGIN_API BOOL IsRequired(void)
{
	if(regReadBool("am_i_required"))
		return true;
	else
		return false;
}

char * regReadString(LPCSTR location)
{
    LONG lResult;
	HKEY hKey;
    DWORD dwBytesRead = 2048;
    char result[2048];
    char *ret;

    lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\PAMPlugin"),0,KEY_QUERY_VALUE,&hKey);
    
    if(lResult == ERROR_SUCCESS)
    {
    	lResult = RegQueryValueEx(hKey,location,NULL,NULL,(LPBYTE)&result,&dwBytesRead);
        
        if(lResult == ERROR_SUCCESS)
        {
            if(dwBytesRead != 0)
            {
                //ret = (TCHAR *)malloc((sizeof(TCHAR) * dwBytesRead) + 1);
                ret = strdup(result);
                RegCloseKey(hKey);
                return ret;
            }
            else
            {
                return NULL;
            }
        }
    }
    RegCloseKey(hKey);
    return NULL;
}

void regWriteString(LPCSTR value,LPCSTR location)
{
    HKEY hKey;
    
    if(RegCreateKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\PAMPlugin"),0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&hKey,NULL) == ERROR_SUCCESS)
    {
        if(value != NULL)
            if(RegSetValueEx(hKey,location,0,REG_SZ,(CONST BYTE *) value, strlen(value)*sizeof(char)) == ERROR_SUCCESS)
            {
                RegFlushKey(hKey);
                RegCloseKey(hKey);
                return;
            }
    }
    
    RegFlushKey(hKey);
    RegCloseKey(hKey);
    return;
}

void regWriteBool(BOOL value,LPCSTR location)
{
    HKEY hKey;
    DWORD write;

    write = (value ? 1 : 0);
    
    if(RegCreateKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\PAMPlugin"),0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,  &hKey,NULL) == ERROR_SUCCESS)
		if(RegSetValueEx(hKey,location,0,REG_DWORD,(LPBYTE)&write,sizeof(DWORD)) == ERROR_SUCCESS)
        {
            RegFlushKey(hKey);
            RegCloseKey(hKey);
			return;
        }
    RegFlushKey(hKey);
    RegCloseKey(hKey);
    return;
}

BOOL regReadBool(LPCSTR location)
{
    LONG lResult;
	HKEY hKey;
    DWORD value = 0;
    DWORD size = sizeof(DWORD);

    lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\PAMPlugin"),0,KEY_QUERY_VALUE,&hKey);
    
    if(lResult == ERROR_SUCCESS)
    {
    	lResult = RegQueryValueEx(hKey,location,NULL,NULL,(LPBYTE)&value,&size);
        
        if(lResult == ERROR_SUCCESS)
        {
            RegCloseKey(hKey);
            if(value == 1) 
                return true;
            else
                return false;
        }
    }
    RegCloseKey(hKey);
    return false;
}



char * convert_to_char(const TCHAR *from)
{
	char *to = (char *) GlobalAlloc(NULL,1024);
	
	WideCharToMultiByte(CP_ACP,
						NULL,
						(LPCWSTR)from,
						-1,
						to,
						1024,
						NULL,
						NULL);
	
	return to;
}

TCHAR* convert_to_wchar(const char *from)
{
	TCHAR *to;
	
	to = (TCHAR *) GlobalAlloc(NULL,1024*sizeof(TCHAR)+1);
	MultiByteToWideChar(CP_ACP,
						NULL,
						(LPCSTR)from,
						strlen(from)+1,
						(LPWSTR)to,
						1024*sizeof(TCHAR)+1);					

	return to;
	
}

void send_string(SSL *mySSL, char *message)
{
	int error = 0, wrote = 0;

	for(wrote = 0; wrote <= strlen(message); wrote++)
	{
		error = SSL_write(mySSL,&message[wrote],1);
		
		if(error <= 0)
			break;
	}
	return;
}

char * read_string(SSL *mySSL)
{
	char buffer[1024];
	char *ret;
	char this_one;
	int error = 0, read_in = 0;

	memset(buffer,'\0',sizeof(buffer));

	for(read_in = 0; read_in < sizeof(buffer); read_in += error)
	{
		error = SSL_read(mySSL,&this_one,1);
		buffer[read_in] = this_one;

		if(error <= 0 || this_one == '\0' || this_one == '\n')
			break;
	}
	
	// Buffer is good here
	ret = (char *) malloc(strlen(buffer) + 1);
	strcpy(ret,buffer);

	return ret;
}

SSL * connect_to_server(pGinaInfo *settingsInfo)
{
	SSL *ssl;
	SSL_CTX *ctx;
	BIO *conn;
	char *pam_host;

	// Init OpenSSL
	OpenSSL_add_all_algorithms();
	SSL_load_error_strings();

	// Init context
	ctx = SSL_CTX_new(TLSv1_method());
	if(!ctx)
	{
		settingsInfo->errorString = convert_to_wchar("PAMPlugin: Error creating context.");
		return NULL;
	}

	// Init SSL
	ssl = SSL_new(ctx);
	if(!ssl)
	{
		settingsInfo->errorString = convert_to_wchar("PAMPlugin: Error creating SSL.");
		return NULL;
	}

	pam_host = regReadString(TEXT("PAMHost"));
	conn = BIO_new_connect(pam_host);
	
	if(!conn)
	{
		settingsInfo->errorString = convert_to_wchar("PAMPlugin: Error in BIO_new_connect.");
		return NULL;
	}

	if(BIO_do_connect(conn) <= 0)
	{
		settingsInfo->errorString = convert_to_wchar("PAMPlugin: Error in BIO_do_connect.");
		return NULL;
	}

	SSL_set_bio(ssl,conn,conn);
	
	if(SSL_connect(ssl) <= 0)
	{
		settingsInfo->errorString = convert_to_wchar("PAMPlugin: Error in SSL handshake.");
		return NULL;
	}

	return ssl;
}


BEGIN_MESSAGE_MAP(CTestApp, CWinApp)
	//{{AFX_MSG_MAP(CTestApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestApp construction

CTestApp::CTestApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CTestApp object

