/*
 *  SSHAuth.cpp
 *
 *  Copyright (C) 2005-2006  Ahmed Obied
 *	
 *  This program is free software; you can redistribute it and/or	
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *  This product includes software developed by the libSSH2 Project
 *  (http://libssh2.sourceforge.net/)
 *
 *  This product includes software developed by the OpenSSL Project
 *  for use in the OpenSSL Toolkit. (http://www.openssl.org/)
 *
 *  In addition, as a special exception, the copyright holders give
 *  permission to link the code of portions of this program with the
 *  OpenSSL library under certain conditions as described in each
 *  individual source file, and distribute linked combinations
 *  including the two.
 *
 *  You must obey the GNU General Public License in all respects
 *  for all of the code used other than OpenSSL.  If you modify
 *  file(s) with this exception, you may extend this exception to your
 *  version of the file(s), but you are not obligated to do so.  If you
 *  do not wish to do so, delete this exception statement from your
 *  version.  If you delete this exception statement from all source
 *  files in the program, then also delete it here.
 *
 */

#include "SSHAuth.h"
#include "resource.h"

HINSTANCE hInstance;
LPTSTR registry_data[REG_DATA_NUM] = {_T("Host"),_T("Port")};

static PCHAR convert_wchar_to_char(PTCHAR wcstr)
{
	PCHAR cstr;
		
	cstr = (PCHAR)GlobalAlloc(NULL,MAX_BYTES);
	WideCharToMultiByte(CP_ACP,NULL,(LPCWSTR)wcstr,-1,cstr,MAX_BYTES,NULL,NULL);	
	return cstr;
}

static LPTSTR convert_char_to_wchar(PCHAR cstr)
{
	LPTSTR wcstr;
	INT wcstr_size;

	wcstr_size = lstrlenA(cstr);
	wcstr = (LPTSTR)SysAllocStringLen(NULL,wcstr_size);
	MultiByteToWideChar(CP_ACP,0,cstr,wcstr_size,wcstr,wcstr_size);
	return wcstr;
}

static LPTSTR read_registry_string(LPTSTR location)
{
	LONG l_result;
	HKEY h_key;
	DWORD bytes_read = MAX_BYTES;
	TCHAR result[MAX_BYTES];
	TCHAR *ret;
	
	l_result = RegOpenKeyEx(HKEY_LOCAL_MACHINE,REGISTRY_KEY,0,KEY_QUERY_VALUE,&h_key);
	if(l_result == ERROR_SUCCESS) {
    		l_result = RegQueryValueEx(h_key,location,NULL,NULL,(LPBYTE)&result,&bytes_read);
		if(l_result == ERROR_SUCCESS) {
			if(bytes_read > 0) {
				if(!(_tcscmp(result,EMPTY) == 0)) {
					ret = _tcsdup(result);
					RegCloseKey(h_key);
					return ret;
				}
			}
		}
	}
	RegCloseKey(h_key);
	return NULL;
}

static DWORD read_registry_dword(LPTSTR location)
{
	LONG l_result;
	HKEY h_key;
	DWORD value = 0;
	DWORD size = sizeof(DWORD);

	l_result = RegOpenKeyEx(HKEY_LOCAL_MACHINE,REGISTRY_KEY,0,KEY_QUERY_VALUE,&h_key);
	if(l_result == ERROR_SUCCESS) {
    		l_result = RegQueryValueEx(h_key,location,NULL,NULL,(LPBYTE)&value,&size);
			if(l_result == ERROR_SUCCESS) {
				RegCloseKey(h_key);
			        return value;
			}
	}
	RegCloseKey(h_key);
	return 0;
}

static BOOL write_registry_string(LPTSTR value, LPTSTR location)
{
	HKEY h_key;

	if(RegCreateKeyEx(HKEY_LOCAL_MACHINE,REGISTRY_KEY,0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,
		&h_key,NULL) == ERROR_SUCCESS) {
		if(value != NULL)
			if(RegSetValueEx(h_key,location,0,REG_SZ,(CONST BYTE *)value,
				(DWORD)(_tcslen(value)*sizeof(TCHAR))) == ERROR_SUCCESS) {
				RegFlushKey(h_key);
				RegCloseKey(h_key);
				return TRUE;
			 }
	}
	RegFlushKey(h_key);
	RegCloseKey(h_key);
	return TRUE;
}

static BOOL write_registry_dword(DWORD value, LPTSTR location)
{
	HKEY h_key;
	
	if(RegCreateKeyEx(HKEY_LOCAL_MACHINE,REGISTRY_KEY,0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,
		&h_key,NULL) == ERROR_SUCCESS) {
		if(RegSetValueEx(h_key,location,0,REG_DWORD,(LPBYTE)&value,sizeof(DWORD)) == ERROR_SUCCESS) {
			RegFlushKey(h_key);
			RegCloseKey(h_key);
			return TRUE;
		}
	}
	RegFlushKey(h_key);
	RegCloseKey(h_key);
	return FALSE;
}

static VOID restore_settings(HWND c_dlg)
{
	PCHAR tmp;
	LPTSTR host, t_port;
	DWORD port;

	tmp = (PCHAR)malloc(MAX_BYTES);
	host = (LPTSTR)malloc(MAX_BYTES);
	memset(tmp,STR_TERM,MAX_BYTES);
	memset(host,STR_TERM,MAX_BYTES);
	if ((host = read_registry_string(registry_data[HOST])) != NULL) {
		SetDlgItemText(c_dlg,IDC_HOST_FIELD,host);
		free(host);
	}
	if ((port = read_registry_dword(registry_data[PORT])) != 0) {
		sprintf(tmp,"%d",port);
		t_port = convert_char_to_wchar(tmp);
		SetDlgItemText(c_dlg,IDC_PORT_FIELD,t_port);
		free(tmp);
	}
}

static BOOL save_settings(HWND c_dlg, WPARAM w_param)
{
	LPTSTR host, port;
	DWORD d_port;

	host = (LPTSTR)malloc(MAX_BYTES);
	port = (LPTSTR)malloc(MAX_BYTES);
	memset(host,STR_TERM,MAX_BYTES);
	memset(port,STR_TERM,MAX_BYTES);
	GetDlgItemText(c_dlg,IDC_HOST_FIELD,host,MAX_BYTES);
	if (_tcscmp(host,EMPTY) == 0) {
		error("Please enter a valid host in the server settings.");
		return FALSE;
	}
	GetDlgItemText(c_dlg,IDC_PORT_FIELD,port,MAX_BYTES);
	if (_tcscmp(port,EMPTY) == 0) {
		error("Please enter a valid port in the server settings.");
		return FALSE;
	}
	d_port = atoi(convert_wchar_to_char(port));
	if (!((d_port >= MIN_PORT_NUM) && (d_port <= MAX_PORT_NUM))) {
		error("You have entered an invalid port number.");
		return FALSE;
	}
	if (!write_registry_string(host,registry_data[HOST])) {
		error("An error has occurred ... \n\nCannot save host data in the registry.");
		return FALSE;
	}
	if (!write_registry_dword(d_port,registry_data[PORT])) {
		error("An error has occurred ... \n\nCannot save port data in the registry.");
		return FALSE;
	}
	free(host);
	free(port);
	EndDialog(c_dlg,LOWORD(w_param));
	return TRUE;
}

INT_PTR CALLBACK server_settings(HWND c_dlg, UINT msg, WPARAM w_param, LPARAM l_param)
{
	switch(msg) {
		case WM_INITDIALOG:
			restore_settings(c_dlg);
		break;
		case WM_COMMAND: {
			switch (LOWORD(w_param)) {
				case IDC_SAVE_BUTTON: {
					if (save_settings(c_dlg,LOWORD(w_param))) {
						EndDialog(c_dlg,LOWORD(w_param));
						MessageBox(GetForegroundWindow(),
						_T("Settings were successfully saved in the registry"),
						NAME,
						MB_OK);
					}
				}
				break;
				case IDC_CANCEL_BUTTON:
					EndDialog(c_dlg,LOWORD(w_param));
				break;
			}
		}
		break;
		case WM_CLOSE:
			EndDialog(c_dlg,LOWORD(w_param));
	}
	return FALSE;
}

BOOL APIENTRY DllMain(HANDLE handle, DWORD reason, LPVOID reserved)
{
	if (reason == DLL_PROCESS_ATTACH) {
		hInstance = (HINSTANCE)handle;
	}
	return TRUE;
}

SSHAUTH_API BOOL UserLogin(LPTSTR username, LPTSTR password, pGinaInfo *settingsInfo)
{
	LIBSSH2_SESSION *ssh_session;
	BOOL auth_check;
	WSADATA wsa_data;
	SOCKET sock_ds;
	DWORD err, port;
	PCHAR user, pwd, str_port;
	LPTSTR host;
	ADDRINFO *addr_info;

	auth_check = FALSE;
	user = convert_wchar_to_char(username);
	pwd = convert_wchar_to_char(password);
	host = (LPTSTR)malloc(MAX_BYTES);
	str_port = (PCHAR)malloc(MAX_BYTES);
	memset(str_port,STR_TERM,MAX_BYTES);
	memset(host,STR_TERM,MAX_BYTES);
	if ((host = read_registry_string(registry_data[HOST])) == NULL)
		return FALSE;
	if ((port = read_registry_dword(registry_data[PORT])) == 0)
		return FALSE;
	sprintf(str_port,"%d",port);
	err = WSAStartup(WINSOCK_VERSION,&wsa_data);
	if (err != 0) {
		error("An error has occurred ... \n\nWSAStartup call failed.\n");
		return FALSE;
	}
	err = getaddrinfo(convert_wchar_to_char(host),str_port,NULL,&addr_info);
	if (err != 0) {
		error("An error has occurred ... \n\ngetaddrinfo call failed.\n");
		return FALSE;
	}
	sock_ds = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
	if (sock_ds == INVALID_SOCKET) {
		error("An error has occurred ... \n\nsocket call failed.\n");
		return FALSE;
	}
	err = connect(sock_ds,addr_info->ai_addr,addr_info->ai_addrlen);
	if (err == SOCKET_ERROR) {
		error("An error has occurred ... \n\nconnect call failed.\n");
		return FALSE;
	}
	ssh_session = libssh2_session_init();
	if (ssh_session == NULL) {
		error("An error has occurred ... \n\nlibssh2_session_init call failed.\n");
		return FALSE;
	}
	err = libssh2_session_startup(ssh_session,(int)sock_ds);
	if (err != 0) {
		error("An error has occurred ...\n\nlibssh2_session_startup call failed.\n");
		return FALSE;
	}
	err = libssh2_userauth_password(ssh_session,user,pwd);
	if (err != 0)
		auth_check = FALSE;
	else
		auth_check = TRUE;
	libssh2_session_free(ssh_session);
	closesocket(sock_ds);
	GlobalFree(user);
	GlobalFree(pwd);
	free(host);
	free(str_port);
	return auth_check;
}

SSHAUTH_API BOOL ChangeUserPassword(LPTSTR username, LPTSTR old_pwd, LPTSTR new_pwd)
{
	return FALSE;
}

SSHAUTH_API LPCTSTR AboutPlugin()
{
	LPCTSTR about = _T("SSH Authentication plugin for pGINA by Ahmed Obied (ahmed.obied@gmail.com)");
	return about;
} 

SSHAUTH_API VOID ChangePluginSettings()
{
	DialogBox(hInstance,MAKEINTRESOURCE(IDD_DIALOG),NULL,server_settings);	
}

SSHAUTH_API VOID LoginHook(pGinaInfo *settings_info)
{
	
}

SSHAUTH_API BOOL IsRequired()
{
	return FALSE;
}

SSHAUTH_API VOID LogoutHook(pGinaInfo *settings_info)
{

}