/*
Symbol Modification Plugin for pGina
Copyright (C) 2004 Nathan Yocom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: nate.yocom@xpasystems.com
Web: http://pgina.xpasystems.com
Snail Mail:
  Nathan Yocom 
  19 Foxwell Drive
  Scarborough, ME 04074
  Phone: 207-450-4948
*/

/**************************************************************************************
 * Development of this plugin was sponsored by an anonymous donor and I would
 *  like to acknowledge their support and thank them for pursuing its creation.
 **************************************************************************************/



#include "stdafx.h"
#include "SymbolMod.h"
#include "resource.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			gHandle = hModule;
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


SYMBOLMOD_API BOOL UserLogin(LPTSTR Username, LPTSTR Password, pGinaInfo *settingsInfo) {
	TCHAR *from_string = regReadString(TEXT("fromString"));
	TCHAR *to_string = regReadString(TEXT("toString"));
	TCHAR *truncate_string = regReadString(TEXT("truncate"));
	int truncate_length = 0;
	TCHAR *at = NULL;

	// Original and replacement have to be same size
	if(wcslen(from_string) == wcslen(to_string)) {
		// make sure it exists at least once
		if(wcsstr(Username,from_string)) {
			at = wcsstr(Username,from_string);
			while(at) {
				memcpy(at,to_string,wcslen(to_string)*sizeof(TCHAR));
				at = wcsstr(Username,from_string);
			}
		}
	}

	if(truncate_string) {
		truncate_length = wcstol(truncate_string,NULL,10);
		if(truncate_length > 0) {
			if(wcslen(Username) > truncate_length) {
				Username[truncate_length] = NULL;
			}
		}
	}
	settingsInfo->Username = _tcsdup(Username);
	
	return true;
}

SYMBOLMOD_API BOOL ChangeUserPassword(LPTSTR string1, LPTSTR string2, LPTSTR string3) {
	return true;
}

SYMBOLMOD_API LPCTSTR AboutPlugin(void) {
	return TEXT("Symbol Modification Plugin - Written by Nathan Yocom (nate@xpasystems.com) - Sponsored by an Anonymous donor");
}

SYMBOLMOD_API void ChangePluginSettings(void) {
	DialogBox((HINSTANCE)gHandle,MAKEINTRESOURCE(IDD_CONFIG),NULL,(DLGPROC)ConfigBoxProc);
}

SYMBOLMOD_API void LoginHook(pGinaInfo *settingsInfo) {
	
}

SYMBOLMOD_API void LogoutHook(pGinaInfo *settingsInfo) {
	
}

SYMBOLMOD_API BOOL IsRequired(void) {
	return false;
}


/**
 * Registry handling/settings functions
 */

TCHAR * regReadString(LPCTSTR location)
{
	LONG lResult;
	HKEY hKey;
	DWORD dwBytesRead = 2048;
	TCHAR result[2048];
	TCHAR *ret;

	lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\SymbolMod"),0,KEY_QUERY_VALUE,&hKey);

	if (lResult == ERROR_SUCCESS) {
		lResult = RegQueryValueEx(hKey,location,NULL,NULL,(LPBYTE)&result,&dwBytesRead);

		if (lResult == ERROR_SUCCESS) {
			if (dwBytesRead > 0) {
				if (_tcscmp(result,L"") != 0) {	// If its just a space
					ret = _tcsdup(result);
					RegCloseKey(hKey);
					return ret;
				}
			}
		}
	}
	RegCloseKey(hKey);
	return NULL;
}

TCHAR * regReadStringDefault(LPCTSTR location, TCHAR * defaultVal)
{
	TCHAR *ret = regReadString(location);

	if ((NULL == ret) && (NULL != defaultVal)) {
		ret = _tcsdup(defaultVal);
	}
	return ret;
}

BOOL regWriteString(LPTSTR value,LPCTSTR location)
{
	HKEY hKey;

	if (RegCreateKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\SymbolMod"),0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&hKey,NULL) == ERROR_SUCCESS) {
		if (value != NULL)
			if (RegSetValueEx(hKey,location,0,REG_SZ,(CONST BYTE *) value, (DWORD)(_tcslen(value)*sizeof(TCHAR))) == ERROR_SUCCESS) {
				RegFlushKey(hKey);
				RegCloseKey(hKey);
				return true;
			}
	}

	RegFlushKey(hKey);
	RegCloseKey(hKey);
	return false;
}

BOOL regWriteDword(DWORD value,LPCTSTR location)
{
	HKEY hKey;
	DWORD write;

	write = value;

	if (RegCreateKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\SymbolMod"),0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,  &hKey,NULL) == ERROR_SUCCESS)
		if (RegSetValueEx(hKey,location,0,REG_DWORD,(LPBYTE)&write,sizeof(DWORD)) == ERROR_SUCCESS) {
			RegFlushKey(hKey);
			RegCloseKey(hKey);
			return true;
		}
	RegFlushKey(hKey);
	RegCloseKey(hKey);
	return false;
}

DWORD regReadDword(LPCTSTR location)
{
	LONG lResult;
	HKEY hKey;
	DWORD value = 0;
	DWORD size = sizeof(DWORD);

	lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\SymbolMod"),0,KEY_QUERY_VALUE,&hKey);

	if (lResult == ERROR_SUCCESS) {
		lResult = RegQueryValueEx(hKey,location,NULL,NULL,(LPBYTE)&value,&size);

		if (lResult == ERROR_SUCCESS) {
			RegCloseKey(hKey);
			return value;
		}
	}
	RegCloseKey(hKey);
	return 0;
}


void regDelValue(LPCTSTR location)
{
	HKEY    hKey;
	LONG    lResult;

	lResult = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
							_T("SOFTWARE\\pGina\\SymbolMod"),
							0, KEY_WRITE, &hKey);

	if (lResult == ERROR_SUCCESS) {
		lResult = RegDeleteValue(hKey,location);
		RegFlushKey(hKey);
	}

	RegCloseKey(hKey);
}

BOOL regReadBool(LPCWSTR location)
{
	LONG lResult;
	HKEY hKey;
	DWORD value = 0;
	DWORD size = sizeof(DWORD);

	lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,L"SOFTWARE\\pGina\\SymbolMod",0,KEY_QUERY_VALUE,&hKey);

	if (lResult == ERROR_SUCCESS) {
		lResult = RegQueryValueEx(hKey,location,NULL,NULL,(LPBYTE)&value,&size);

		if (lResult == ERROR_SUCCESS) {
			RegCloseKey(hKey);
			if (value == 1)
				return true;
			else
				return false;
		}
	}
	RegCloseKey(hKey);
	return false;
}

BOOL regWriteBool(BOOL value,LPCTSTR location)
{
	HKEY hKey;
	DWORD write;

	write = value ? 1 : 0;

	if (RegCreateKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina\\SymbolMod"),0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,  &hKey,NULL) == ERROR_SUCCESS)
		if (RegSetValueEx(hKey,location,0,REG_DWORD,(LPBYTE)&write,sizeof(DWORD)) == ERROR_SUCCESS) {
			RegFlushKey(hKey);
			RegCloseKey(hKey);
			return true;
		}
	RegFlushKey(hKey);
	RegCloseKey(hKey);
	return false;
}

/**
 * Configuration Dialog Callback Function
 */
BOOL CALLBACK ConfigBoxProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	TCHAR Buffer[2048];
	switch (message) {
	case WM_INITDIALOG:
		CenterWindow(hwndDlg);
		SetDlgItemText(hwndDlg,IDC_FROMCHAR,regReadString(_T("fromString")));
		SetDlgItemText(hwndDlg,IDC_TOCHAR,regReadString(_T("toString")));
		SetDlgItemText(hwndDlg,IDC_TRUNC,regReadString(_T("truncate")));
		return TRUE;
	case WM_COMMAND: 
		switch (LOWORD(wParam)) {
		case IDOK:
			GetDlgItemText(hwndDlg,IDC_FROMCHAR,Buffer,sizeof(Buffer)-1);
			regWriteString(Buffer,_T("fromString"));
			GetDlgItemText(hwndDlg,IDC_TOCHAR,Buffer,sizeof(Buffer)-1);
			regWriteString(Buffer,_T("toString"));
			GetDlgItemText(hwndDlg,IDC_TRUNC,Buffer,sizeof(Buffer)-1);
			regWriteString(Buffer,_T("truncate"));
			EndDialog(hwndDlg,IDOK);
			break;
		case IDCANCEL:
			EndDialog(hwndDlg, IDCANCEL);
			break;
		}
		return TRUE;
	} 
	return FALSE;
}

VOID CenterWindow(HWND hwnd)
{
	RECT    rect;
	LONG    dx, dy;
	LONG    dxParent, dyParent;
	LONG    Style;

	GetWindowRect(hwnd, &rect);

	dx = rect.right - rect.left;
	dy = rect.bottom - rect.top;

	Style = GetWindowLong(hwnd, GWL_STYLE);
	if ((Style & WS_CHILD) == 0) {
		dxParent = GetSystemMetrics(SM_CXSCREEN);
		dyParent = GetSystemMetrics(SM_CYSCREEN);
	} else {
		HWND    hwndParent;
		RECT    rectParent;

		hwndParent = GetParent(hwnd);
		if (hwndParent == NULL) {
			hwndParent = GetDesktopWindow();
		}

		GetWindowRect(hwndParent, &rectParent);

		dxParent = rectParent.right - rectParent.left;
		dyParent = rectParent.bottom - rectParent.top;
	}

	rect.left = (dxParent - dx) / 2;
	rect.top  = (dyParent - dy) / 3;

	SetWindowPos(hwnd, HWND_TOPMOST, rect.left, rect.top, 0, 0, SWP_NOSIZE);

	SetForegroundWindow(hwnd);
}

/**
 * Char handling
 */
char * convert_to_char(const TCHAR *from)
{
	char *to = (char *) malloc(_tcslen(from) * sizeof(TCHAR));

	WideCharToMultiByte(CP_ACP,
						NULL,
						(LPCWSTR)from,
						-1,
						to,
						_tcslen(from) * sizeof(TCHAR),
						NULL,
						NULL);

	return to;
}

TCHAR * convert_to_wchar(const char *from)
{
	TCHAR *to;
	if (from == NULL)
		return NULL;

	to = (TCHAR *) malloc(strlen(from)*sizeof(TCHAR));
	MultiByteToWideChar(CP_ACP,
						NULL,
						(LPCSTR)from,
						strlen(from),
						(LPWSTR)to,
						strlen(from)*sizeof(TCHAR));                    

	return to;
}

