using System;
using System.Net;
using System.IO;
using System.Text.RegularExpressions;

namespace getPage
{
	//HOW TO: Retrieve a web page from an outside server 
	//        via the .NET framework

	public class SampleHttpRequest
	{
		public static void getUri(string strToGet)
		{
			HttpWebRequest  httprequest;
			HttpWebResponse httpresponse;
			Uri             responseUri = null, requestUri;
			StreamReader    bodyreader;
			string          bodytext = "", line;
			Stream          responsestream;

			//the page to retrieve as a Uri object.
			requestUri = new Uri(strToGet);

			//create the web request.
			httprequest = (HttpWebRequest) WebRequest.Create(requestUri);

			//fill in the properties of the request. All properties
			//are optional

			//allow auto redirects from redirect headers
			httprequest.AllowAutoRedirect=true;

			//maximum of 10 auto redirects
			httprequest.MaximumAutomaticRedirections=10;

			//use the GET method
			httprequest.Method="GET";

			//use HTTP/1.0
			httprequest.ProtocolVersion=new Version(1,0);

			//30 second timeout for request
			httprequest.Timeout=(int) new TimeSpan(0,0,30).TotalMilliseconds;

			//give the crawler a name.
			httprequest.UserAgent="Mozilla/3.0 (compatible; My Browser/1.0)";

			//retrieve the response from the requestUri.
			httpresponse = (HttpWebResponse) httprequest.GetResponse();

			//remember the url that responded because it may not
			//be the same as the input url because of the autoredirect
			//property setting above.
			responseUri = httpresponse.ResponseUri;

			//get the body stream of the requested page.
			responsestream = httpresponse.GetResponseStream();
			if (responsestream != null)
			{
				//read the stream and build it into a string
				//representing the body of the web page.
				bodyreader = new StreamReader(responsestream);

				Regex r;
				Match m;

				r = new Regex("href\\s*=\\s*(?:\"(?<1>[^\"]*)\"|(?<1>\\S+))",
					RegexOptions.IgnoreCase|RegexOptions.Compiled);				

				//loop the body, reading line by line
				while((line=bodyreader.ReadLine())!=null)
				{
					bodytext+=line+"\r\n";
				}

				//close the body stream
				bodyreader.Close();

				//cleanup
				bodyreader = null;
				
				for (m = r.Match(bodytext); m.Success; m = m.NextMatch()) 
				{
					Console.WriteLine("Found href " + m.Groups[1] + " at " 
						+ m.Groups[1].Index);
				}
			}

			//close response which cleans up the socket used to
			//retrieve the page.
			httpresponse.Close();

			//cleanup
			httpresponse = null;
			httprequest = null;

			//print the responding url and the text of the body
			/*Console.WriteLine(
				"{0} returns:\r\n{1}", 
				responseUri.ToString(), 
				bodytext
				);*/
		}
	}
}
