using System;
using System.Net;
using System.IO;
using System.Text.RegularExpressions;
using System.Diagnostics;

namespace getPage
{
	public class HttpRequest
	{
		public static String getUri(string strToGet)
		{
			HttpWebRequest  httprequest;
			HttpWebResponse httpresponse;
			Uri             responseUri = null, requestUri;
			Stream          responsestream;
			String			return_string = null;


			//the page to retrieve as a Uri object.
			requestUri = new Uri(strToGet);
			Debug.WriteLine("strToGet: " + strToGet);

			//create the web request.
			httprequest = (HttpWebRequest) WebRequest.Create(requestUri);

			//fill in the properties of the request. All properties
			//are optional

			//allow auto redirects from redirect headers
			httprequest.AllowAutoRedirect=true;

			//maximum of 10 auto redirects
			httprequest.MaximumAutomaticRedirections=10;

			//use the GET method
			httprequest.Method="GET";

			//use HTTP/1.0
			httprequest.ProtocolVersion=new Version(1,0);

			//30 second timeout for request
			httprequest.Timeout=(int) new TimeSpan(0,0,30).TotalMilliseconds;

			//give the crawler a name.
			httprequest.UserAgent="pGina BlueSocket Plugin 1.0";

			Debug.WriteLine("About to try for response");
			//retrieve the response from the requestUri.
			try
			{
				httpresponse = (HttpWebResponse) httprequest.GetResponse();
			}
			catch (System.Net.WebException e)
			{
				Debug.WriteLine(e.ToString());
				return_string = "UNREACHABLE";
				return return_string; //This must be in here or VS will outsmart itself.
			}

			//remember the url that responded because it may not
			//be the same as the input url because of the autoredirect
			//property setting above.
			responseUri = httpresponse.ResponseUri;

			//get the body stream of the requested page.
			responsestream = httpresponse.GetResponseStream();

			StreamReader    bodyreader;
			string          bodytext = "", line;

			if (responsestream != null)
			{
				//read the stream and build it into a string
				//representing the body of the web page.
				bodyreader = new StreamReader(responsestream);

				//loop the body, reading line by line
				while((line=bodyreader.ReadLine())!=null)
				{
					bodytext+=line+"\r\n";
				}

				Debug.WriteLine("Bodytext: " + bodytext);
				return_string = bodytext;

				//close the body stream
				bodyreader.Close();

				//cleanup
				bodyreader = null;
			}


			//close response which cleans up the socket used to
			//retrieve the page.
			httpresponse.Close();

			//cleanup
			httpresponse = null;
			httprequest = null;

			//print the responding url and the text of the body
			//Console.WriteLine("{0} returns:\r\n{1}", responseUri.ToString(), bodytext);
			return return_string;
		}
		
		public static String parseRegex(String body_string, String keyword)
		{
			string	return_string = null;

			if (body_string != null)
			{
				Regex	r;
				Match	m;
				String	regex;
						
				regex = String.Concat("\\<", keyword, "\\>(.*)\\</", keyword, "\\>");
				r = new Regex(regex);

				for (m = r.Match(body_string); m.Success; m = m.NextMatch()) 
				{
					return_string = m.Groups[1].Value;
				}
			}

			return return_string;
		}
	}
}