/*
	pGina - Pluggable GINA authentication DLL for Windows
    Copyright (C) 2001  Nathan Yocom, Michael Wright

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Email: yocomnw@plu.edu
    Web: http://pgina.cs.plu.edu
    Mail: 
       Nathan Yocom @ CSCE
       Pacific Lutheran University
       Tacoma, WA 98447 (USA)
    Phone: 253-535-7400
*/
// configDiag.cpp : implementation file
//

#include "StdAfx.h"
#include "configDiag.h"
#include "fileDiag.h"
#include "PluginSetupDlg.h"
#include "pGinaInterface.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// configDiag dialog


configDiag::configDiag(CWnd* pParent /*=NULL*/)
	: CDialog(configDiag::IDD, pParent)
{
	//{{AFX_DATA_INIT(configDiag)
	m_mapPaths = _T("");
	m_pathMSGina = _T("");
	m_pathPlugin = _T("");
	m_pathProfile = _T("");
	m_PluginInfo = _T("");
	//}}AFX_DATA_INIT
	pGinaFuncs = new pGinaInterface();
}


void configDiag::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(configDiag)
	DDX_Text(pDX, IDC_mapPaths, m_mapPaths);
	DDX_Text(pDX, IDC_pathMSGina, m_pathMSGina);
	DDX_Text(pDX, IDC_pathPlugin, m_pathPlugin);
	DDX_Text(pDX, IDC_pathProfile, m_pathProfile);
	DDX_Text(pDX, IDC_PluginInfo, m_PluginInfo);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(configDiag, CDialog)
	//{{AFX_MSG_MAP(configDiag)
	ON_BN_CLICKED(IDSave, OnSave)
	ON_BN_CLICKED(IDLoad, OnLoad)
	ON_EN_CHANGE(IDC_pathPlugin, OnChangepathPlugin)
	ON_BN_CLICKED(IDC_BrowseMSGina, OnBrowseMSGina)
	ON_BN_CLICKED(IDC_BrowsePlugin, OnBrowsePlugin)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// configDiag message handlers

// Save the entered settings
void configDiag::OnSave() 
{
	// Save the information from the dialog to the registry
	
	// Create our pGinaInfo structure
	pGinaInfo settingsInfo;
	
	// Fill it with the appropriate values
	UpdateData(TRUE);
    settingsInfo.mapPaths = (LPSTR) strdup((LPCSTR)m_mapPaths);
	settingsInfo.pathMSGina = (LPSTR) strdup((LPCSTR)m_pathMSGina);
	settingsInfo.pathPlugin = (LPSTR) strdup((LPCSTR)m_pathPlugin);
	settingsInfo.pathProfile = (LPSTR) strdup((LPCSTR)m_pathProfile);

	// Save them
	pGinaFuncs->pGinaSaveInfo(&settingsInfo);
	
	// Call our parents OK function to destroy Dialog
	CDialog::OnOK();
}

// Load the settings from the Registry
void configDiag::OnLoad() 
{
	// Setup our struct
	pGinaInfo settingsInfo;
	// Load the info
	
	pGinaFuncs->pGinaLoadInfo(&settingsInfo);
		

	// Map the struct to our dialog
	m_pathPlugin = settingsInfo.pathPlugin;
	m_mapPaths = settingsInfo.mapPaths;
	m_pathMSGina = settingsInfo.pathMSGina;
	m_pathProfile = settingsInfo.pathProfile;

	// Display
	UpdateData(FALSE);

	// Load/Test the plugin
	OnChangepathPlugin();
}

// Load the plugin's AboutPlugin() method to get information/test
void configDiag::getPluginInfo(void)
{
	// Handle to our DLL's lookup table
	HINSTANCE hDll;

	// Make sure its a DLL and set the handle
	if(!(hDll = LoadLibrary(m_pathPlugin)))
	{
		// Notify the user if there were issues
		m_PluginInfo = "Unable to load plugin, or no plugin yet selected.";
		return;		
	}
	else
	{
		// Map the methods
		pAboutPlugin = (DLLABOUTPLUGIN) GetProcAddress(hDll,"AboutPlugin");
		
		// If any of them were NOT loaded
		if(!pAboutPlugin)
		{
			// Tell the user something was missing/wrong
			m_PluginInfo = "This does not appear to be a valid or complete pGina plugin.";
			return;
		}
		else
		{
			// If we get here things are cool...
			m_PluginInfo = pAboutPlugin();
			return;
		}
	}
	// if we get this far something is DEFINETLY wrong!
	return; 
}

// When text in the pathPlugin box changes, try to load it as 
//   if it were a plugin. 
void configDiag::OnChangepathPlugin() 
{
	// Get the data from the dialog
	UpdateData(TRUE);
	// Try to load the plugin
	getPluginInfo();
	// Write any new data to the dialog
	UpdateData(FALSE);

}

// Browse button pushed, provides file selection dialog 
//   and sets selected file as the MSGina path
void configDiag::OnBrowseMSGina() 
{
	// Setup a file selection dialog box inherited from CFileDialog
	fileDiag *selectFileDiag = new fileDiag(TRUE,NULL,NULL,OFN_HIDEREADONLY, "MSGina (MSGINA.DLL)|msgina.dll|DLL files (*.dll)|*.dll|All files (*.*)|*.*", NULL);
	
	if(selectFileDiag->DoModal() == IDOK)
	{
		// set the path to the selected file
		m_pathMSGina = selectFileDiag->GetPathName();
	}
	else
	{
		// Nothing selected or they hit cancel
		//   Dont show any paths
		m_pathMSGina = "";
	}
	// Update the dialog with the latest info
	UpdateData(FALSE);
	
}

// Browse button pushed, provides file selection dialog 
//   and sets selected file as the plugin path
void configDiag::OnBrowsePlugin() 
{
	// Setup a file selection dialog box inherited from CFileDialog
	fileDiag *selectFileDiag = new fileDiag(TRUE,NULL,NULL,OFN_HIDEREADONLY, "pGina Plugin (*.pGina)|*.pGina|DLL files (*.dll)|*.dll|All files (*.*)|*.*", NULL);
	
	// If the user selects something and/or presses Okay
	if(selectFileDiag->DoModal() == IDOK)
	{
		// set the path to the selected file
		m_pathPlugin = selectFileDiag->GetPathName();
	}
	else
	{
		// Nothing selected or they hit cancel
		//   Dont show any paths
		m_pathPlugin = "";
	}
	// Update the dialog with the latest info
	UpdateData(FALSE);

	// We have to force this call as UpdateData() does not.
	//  this will allow us to try and load the plugin to test
	//  for compatability
	OnChangepathPlugin();
}
