/*************************************************************************

         AFS Plugin for pGina version 0.8 beta
         (c) 2003 by Martin Decky

         Distributed under the terms of GNU General Public License,
         see COPYING for details.

*************************************************************************/

#include "StdAfx.h"
#include "afsplugin.h"

#include <stdlib.h>
#include <tchar.h>
#include <winsock2.h>

extern "C" {
	#include <afs/param.h>
	#include <afs/stds.h>

	#include <afs/kautils.h>
}

#define MAX_STR_LEN 1024

BOOL get_default_cell(char * Cell, const size_t Len) { // return the current client cell, FIXME: better use cm_GetRootCellName()
	LONG res;
	HKEY hKey;
	TCHAR result[MAX_STR_LEN];
	DWORD rd = sizeof(result);

	res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, _T("SYSTEM\\CurrentControlSet\\Services\\TransarcAFSDaemon\\Parameters"), 0, KEY_QUERY_VALUE, &hKey);
	if (res == ERROR_SUCCESS) {
		res = RegQueryValueEx(hKey, _T("Cell"), NULL, NULL, (LPBYTE) &result, &rd);

		if ((res == ERROR_SUCCESS) && (rd != 0)) {
			wcstombs(Cell, result, Len);
			RegCloseKey(hKey);
			return true;
		}
	}

	RegCloseKey(hKey);
	return false;
}

BOOL autostart_afs(void) { // start the AFS service
	SC_HANDLE scm;
	SC_HANDLE svc;
	BOOL flag;
	BOOL result = false;
	LPQUERY_SERVICE_CONFIG config;
	DWORD rd;
	LONG ret;

	scm = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if (scm) {
		svc = OpenService(scm, _T("TransarcAFSDaemon"), SERVICE_QUERY_CONFIG);
		if (svc) {
			QueryServiceConfig(svc, NULL, 0, &rd);
			ret = GetLastError();
			if (ret == ERROR_INSUFFICIENT_BUFFER) {
				config = (LPQUERY_SERVICE_CONFIG) GlobalAlloc(GMEM_FIXED, rd);
				if (config) {
					flag = QueryServiceConfig(svc, config, rd, &rd);
					if ((flag) && (config->dwStartType < SERVICE_DEMAND_START))
						result = true;
					GlobalFree(config);
				}
			}
			CloseServiceHandle(svc);
		}
		CloseServiceHandle(scm);
	}

	return result;
}

BOOL is_service_running(void) { // check whether the AFS service is currently running
	SC_HANDLE scm;
	SC_HANDLE svc;
	SERVICE_STATUS status;

	memset(&status, 0, sizeof(status));
	status.dwCurrentState = SERVICE_STOPPED;

	scm = OpenSCManager(NULL, NULL, GENERIC_READ);
	if (scm) {
		svc = OpenService(scm, _T("TransarcAFSDaemon"), GENERIC_READ);
		if (svc) {
			QueryServiceStatus(svc, &status);
			CloseServiceHandle(svc);
		}
		CloseServiceHandle(scm);
	}

	return (status.dwCurrentState == SERVICE_RUNNING);
}

BOOL init_sockets() { // initialize WinSock
	WORD version = MAKEWORD(2, 0);
	WSADATA data;
	int err = WSAStartup(version, &data);

	return err == 0;
}

BOOL done_sockets() { // cleanup WinSock
	int err = WSACleanup();

	return err == 0;
}

AFSPLUGIN_API BOOL UserLogin(LPTSTR Username, LPTSTR Password, pGinaInfo * settingsInfo) {
	char uname[MAX_STR_LEN];
	char passw[MAX_STR_LEN];
	char cell[MAX_STR_LEN];
	long pw_exp;
	char * reason;
	int ret;
	int cnt;

	if (_wcsicmp(Username, TEXT("administrator")) == 0) { // don't check administrator's password against AFS
		settingsInfo->errorString = _tcsdup(TEXT("Administrator not checked against AFS"));
		return false;
	}

	ret = rx_Init();
	if (ret != 0) {
		settingsInfo->errorString = _tcsdup(TEXT("Unable to initialize rx client"));
		return false;
	}
	ret = ka_Init(0);
	if (ret != 0) {
		settingsInfo->errorString = _tcsdup(TEXT("Unable to initialize Kerberos client"));
		return false;
	}

	wcstombs(uname, Username, MAX_STR_LEN); // convert username and password to Unicode
	wcstombs(passw, Password, MAX_STR_LEN);

	ret = get_default_cell(cell, MAX_STR_LEN);
	if (!ret) {
		settingsInfo->errorString = _tcsdup(TEXT("Unable to get default cell"));
		return false;
	}

	ret = autostart_afs();
	if (!ret) {
		settingsInfo->errorString = _tcsdup(TEXT("Unable to start AFS service"));
		return false;
	}

	cnt = 0;
	while (!is_service_running()) { // check whether the AFS service is running, with 10 sec timeout
		Sleep(1000);
		cnt++;
		if (cnt > 10) {
			settingsInfo->errorString = _tcsdup(TEXT("AFS service start timeout"));
			return false;
		}
	}

	if (!init_sockets()) {
		settingsInfo->errorString = _tcsdup(TEXT("Unable to initialize WinSock"));
		return false;
	}

	ret = ka_UserAuthenticateGeneral2(KA_USERAUTH_VERSION + KA_USERAUTH_AUTHENT_LOGON, uname, "", cell, passw, uname, 0, &pw_exp, 0, &reason); // do the authentication here
	
	if (ret) { // report authentication error
		size_t slen = _mbstrlen(reason) + 1;
		settingsInfo->errorString = (LPTSTR) malloc(slen * sizeof(TCHAR));
		mbstowcs(settingsInfo->errorString, reason, slen);
		done_sockets();
		return false;
	}

	settingsInfo->allowPassChange = false; // TODO
	settingsInfo->isAdmin = false;
	settingsInfo->userGroups = _tcsdup(TEXT("Users")); // TODO: configurable AFS->Windows users mapping
	
	// TODO: fill in userFullName (is this actually stored somewhere in Kerberos?)
	
	done_sockets();
	return true;
}

AFSPLUGIN_API BOOL ChangeUserPassword(LPTSTR Username, LPTSTR OldPassword, LPTSTR NewPassword) { // TODO
	return false;
}

AFSPLUGIN_API LPCTSTR AboutPlugin(void) { // TODO: a better About dialog
	return TEXT("AFS authentication plugin");
}

AFSPLUGIN_API void ChangePluginSettings(void) { // TODO: configure some plugin options (service timeout, groups mapping, etc)
}

AFSPLUGIN_API void LoginHook(pGinaInfo * settingsInfo) { // TODO: home directory mapping, network profiles (?)
}

AFSPLUGIN_API void LogoutHook(pGinaInfo * settingsInfo) {
}

AFSPLUGIN_API BOOL IsRequired(void) {
	return true;
}
