/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */

// See auth.h for function descriptions

#include "stdafx.h"
#include "pgftp.h"
#include "auth.h"
#include "misc.h"
#include "resource.h"

// Used to display security warnings
int ShowSecurityWarning(LPCTSTR msg, int buttons) {
	TCHAR title[32];

	GetString(IDS_TITLE_SECURITY, title);
	return MessageBox(NULL, msg, title, buttons | MB_ICONWARNING | MB_SETFOREGROUND | MB_TOPMOST);
}

BOOL ValidateHostKey(LPCTSTR server, LPCTSTR key) {
	TCHAR msgFmt[1024];
	TCHAR msg[1024];
	LPTSTR savedKey = RegReadString(S_SFTP_HOST_KEY);
	int button;
	BOOL ret = FALSE;

	if (!savedKey) {
		GetString(IDS_MSG_NEW_HOST_KEY, msgFmt);
		StringCbPrintf(msg, sizeof(msg), msgFmt, server, key);
		button = ShowSecurityWarning(msg, MB_YESNOCANCEL);

		if (IDCANCEL == button) {
			ret = FALSE;
		}
		else {
			if (IDYES == button) {
				RegWriteString(S_SFTP_HOST_KEY, key);
			}
			ret = TRUE;
		}
	}
	else {
		// Check host's key against our version
		if (lstrcmp(key, savedKey)) {
			GetString(IDS_MSG_BAD_HOST_KEY, msgFmt);
			StringCbPrintf(msg, sizeof(msg), msgFmt, server, key, savedKey);
			ret = (IDYES == ShowSecurityWarning(msg, MB_YESNO));
			if (ret) {
				// Overwrite old key value
				RegWriteString(S_SFTP_HOST_KEY, key);
			}
		}
		else
			ret = TRUE;
	}

	LocalFree(savedKey);
	return ret;
}