/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */

#include "stdafx.h"
#include "pgftp.h"
#include "misc.h"
#include "resource.h"

// Dialog procedure for the Plugin settings tab
INT_PTR CALLBACK PluginSettingsProc(HWND hWndDlg,
    UINT msg,
    WPARAM wParam,
    LPARAM lParam)
{
	TCHAR buffer[MAX_SETTING];
	BOOL checked;
	int role;
	PLUGIN_SETTINGS settings;

	switch (msg) {
		case WM_INITDIALOG:
			// Populate Role combo
			SendDlgItemMessage(hWndDlg, IDC_COMBO_ROLE, CB_ADDSTRING, 0, (LPARAM)TEXT("Profiles Only"));
			SendDlgItemMessage(hWndDlg, IDC_COMBO_ROLE, CB_ADDSTRING, 0, (LPARAM)TEXT("Authentication Only"));
			SendDlgItemMessage(hWndDlg, IDC_COMBO_ROLE, CB_ADDSTRING, 0, (LPARAM)TEXT("Both"));

			// Setup compression slider
			SendDlgItemMessage(hWndDlg, IDC_SLIDER_COMPRESSION, TBM_SETRANGEMIN, FALSE, MIN_COMPRESSION);
			SendDlgItemMessage(hWndDlg, IDC_SLIDER_COMPRESSION, TBM_SETRANGEMAX, TRUE, MAX_COMPRESSION);

			// Initialize controls from registry settings
			LoadSettings(&settings);
			SetDlgItemText(hWndDlg, IDC_EDIT_PATH, settings.profileTempRoot);
			SendDlgItemMessage(hWndDlg, IDC_COMBO_ROLE, CB_SETCURSEL, settings.role - 1, 0);
			EnableWindow(GetDlgItem(hWndDlg, IDC_STATIC_PATH), settings.role != ROLE_AUTH);
			EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT_PATH), settings.role != ROLE_AUTH);
			if (settings.compression) 
				CheckDlgButton(hWndDlg, IDC_CHECK_COMPRESSION, BST_CHECKED);
			else
				CheckDlgButton(hWndDlg, IDC_CHECK_COMPRESSION, BST_UNCHECKED);
			SendMessage(GetDlgItem(hWndDlg, IDC_SLIDER_COMPRESSION), TBM_SETPOS, TRUE, settings.compression);
			SendMessage(hWndDlg, WM_COMMAND, MAKEWPARAM(IDC_CHECK_COMPRESSION, BN_CLICKED), 
				(LPARAM)GetDlgItem(hWndDlg, IDC_CHECK_COMPRESSION));
			CheckDlgButton(hWndDlg, IDC_CHECK_REQUIRED, settings.required ? BST_CHECKED : BST_UNCHECKED);
			FreeSettings(&settings);
			return TRUE;

		case WM_COMMAND:
			switch (HIWORD(wParam)) {
				case BN_CLICKED:
					if (LOWORD(wParam) == IDC_CHECK_COMPRESSION) {
						checked = IsDlgButtonChecked(hWndDlg, IDC_CHECK_COMPRESSION);
						EnableWindow(GetDlgItem(hWndDlg, IDC_SLIDER_COMPRESSION), checked);
						EnableWindow(GetDlgItem(hWndDlg, IDC_STATIC_COMPRESSION1), checked);
						EnableWindow(GetDlgItem(hWndDlg, IDC_STATIC_COMPRESSION2), checked);						
					}
					// Fall through ...
				case EN_CHANGE:
				case CBN_SELCHANGE:
					if (LOWORD(wParam) == IDC_COMBO_ROLE) {
						role = SendDlgItemMessage(hWndDlg, IDC_COMBO_ROLE, CB_GETCURSEL, 0, 0) + 1;
						EnableWindow(GetDlgItem(hWndDlg, IDC_STATIC_PATH), role != ROLE_AUTH);
						EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT_PATH), role != ROLE_AUTH);
					}

					// Signal that the settings have changed to enable the "apply" button
					PropSheet_Changed((HWND) GetWindowLong(hWndDlg, GWL_USERDATA), hWndDlg);
					return TRUE;
			}
			break;

		case WM_NOTIFY:
			switch (((NMHDR*) lParam)->code) {
				case PSN_SETACTIVE:
					// Save the property sheet handle for future use
					SetWindowLong(hWndDlg, GWL_USERDATA, (LONG) ((NMHDR*) lParam)->hwndFrom);
					return TRUE;
				case PSN_APPLY:
					// Validate the user's input
					role = SendDlgItemMessage(hWndDlg, IDC_COMBO_ROLE, CB_GETCURSEL, 0, 0) + 1;
					GetDlgItemText(hWndDlg, IDC_EDIT_PATH, buffer, MAX_SETTING);
					if (role != ROLE_AUTH && !lstrlen(buffer)) {
						MessageBox(hWndDlg, TEXT("You must specify a temporary profile path"), 
							TEXT("Settings"), MB_OK | MB_ICONSTOP);
						SetFocus(GetDlgItem(hWndDlg, IDC_EDIT_PATH));
						SetWindowLong(hWndDlg, DWL_MSGRESULT, PSNRET_INVALID);
						return TRUE;
					}

					// Write settings to the registry
					RegWriteString(S_PROFILE_TEMP_ROOT, buffer);
					RegWriteDWORD(S_ROLE, role);
					if (IsDlgButtonChecked(hWndDlg, IDC_CHECK_COMPRESSION))
						RegWriteDWORD(S_COMPRESSION, 
							SendMessage(GetDlgItem(hWndDlg, IDC_SLIDER_COMPRESSION), TBM_GETPOS, 0, 0));
					else
						RegWriteDWORD(S_COMPRESSION, 0);
					RegWriteDWORD(S_REQUIRED, IsDlgButtonChecked(hWndDlg, IDC_CHECK_REQUIRED));
					return TRUE;
			}
			break;
	}

	return FALSE;
}

// Dialog procedure for the Server settings tab
INT_PTR CALLBACK ServerSettingsProc(HWND hWndDlg,
    UINT msg,
    WPARAM wParam,
    LPARAM lParam)
{
	PLUGIN_SETTINGS settings;
	DWORD protocol;
	DWORD ftpFlags = 0;
	TCHAR buffer[MAX_SETTING];

	switch (msg) {
		case WM_INITDIALOG:
			// Populate Protocol combo
			SendDlgItemMessage(hWndDlg, IDC_COMBO_PROTOCOL, CB_ADDSTRING, 0, (LPARAM)TEXT("FTP"));
			SendDlgItemMessage(hWndDlg, IDC_COMBO_PROTOCOL, CB_ADDSTRING, 0, (LPARAM)TEXT("SFTP"));

			// Initialize controls from registry settings
			LoadSettings(&settings);
			SetDlgItemText(hWndDlg, IDC_EDIT_SERVER, settings.ftpServer);
			SendDlgItemMessage(hWndDlg, IDC_COMBO_PROTOCOL, CB_SETCURSEL, settings.protocol, 0);
			_itot(settings.port, buffer, 10);
			SetDlgItemText(hWndDlg, IDC_EDIT_PORT, buffer);
			if (settings.protocol == PROTOCOL_FTP) {
				CheckDlgButton(hWndDlg, IDC_CHECK_PASSIVE, 
					settings.flags & INTERNET_FLAG_PASSIVE ? BST_CHECKED : BST_UNCHECKED);
			}
			else {
				ShowWindow(GetDlgItem(hWndDlg, IDC_CHECK_PASSIVE), SW_HIDE);
				ShowWindow(GetDlgItem(hWndDlg, IDC_STATIC_HOSTKEY), SW_SHOW);
				ShowWindow(GetDlgItem(hWndDlg, IDC_EDIT_HOSTKEY), SW_SHOW);
				SetDlgItemText(hWndDlg, IDC_EDIT_HOSTKEY, settings.sftpHostKey);
			}
				
			if (settings.ftpUserName) {
				CheckDlgButton(hWndDlg, IDC_RADIO_USERNAME_SPECIFY, BST_CHECKED);
				SetDlgItemText(hWndDlg, IDC_EDIT_USERNAME, settings.ftpUserName);
				EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT_USERNAME), TRUE);
			}
			else
				CheckDlgButton(hWndDlg, IDC_RADIO_USERNAME_USEDEFAULT, BST_CHECKED);
			
			if (settings.ftpPassword) {
				CheckDlgButton(hWndDlg, IDC_RADIO_PASSWORD_SPECIFY, BST_CHECKED);
				SetDlgItemText(hWndDlg, IDC_EDIT_PASSWORD, settings.ftpPassword);
				EnableWindow(GetDlgItem(hWndDlg, IDC_EDIT_PASSWORD), TRUE);
			}
			else
				CheckDlgButton(hWndDlg, IDC_RADIO_PASSWORD_USEDEFAULT, BST_CHECKED);

			FreeSettings(&settings);
			return TRUE;

		case WM_COMMAND:
			switch (HIWORD(wParam)) {
				case BN_CLICKED:
					switch (LOWORD(wParam)) {
						case IDC_RADIO_USERNAME_USEDEFAULT:
						case IDC_RADIO_PASSWORD_USEDEFAULT:
							SetDlgItemText(hWndDlg, LOWORD(wParam) + 2, TEXT(""));
							EnableWindow(GetDlgItem(hWndDlg, LOWORD(wParam) + 2), FALSE);
							break;
						case IDC_RADIO_USERNAME_SPECIFY:
						case IDC_RADIO_PASSWORD_SPECIFY:
							EnableWindow(GetDlgItem(hWndDlg, LOWORD(wParam) + 1), TRUE);
							break;
					}
					// Fall through ...
				case EN_CHANGE:
				case CBN_SELCHANGE:
					if (LOWORD(wParam) == IDC_COMBO_PROTOCOL) {
						protocol = SendDlgItemMessage(hWndDlg, IDC_COMBO_PROTOCOL, 
							CB_GETCURSEL, 0, 0);
						// Change port to the default for the selected protocol
						_itot(protocol == PROTOCOL_SFTP ? DEFAULT_SFTP_PORT : DEFAULT_FTP_PORT,
							buffer, 10);
						SetDlgItemText(hWndDlg, IDC_EDIT_PORT, buffer);

						if (PROTOCOL_FTP == protocol) {
							// Check passive mode by default
							CheckDlgButton(hWndDlg, IDC_CHECK_PASSIVE, BST_CHECKED);
						}

						// Show protocol-specific options
						ShowWindow(GetDlgItem(hWndDlg, IDC_CHECK_PASSIVE), 
							protocol == PROTOCOL_FTP ? SW_SHOW : SW_HIDE);
						ShowWindow(GetDlgItem(hWndDlg, IDC_STATIC_HOSTKEY),
							protocol == PROTOCOL_SFTP ? SW_SHOW : SW_HIDE);
						ShowWindow(GetDlgItem(hWndDlg, IDC_EDIT_HOSTKEY),
							protocol == PROTOCOL_SFTP ? SW_SHOW : SW_HIDE);
					}

					// Signal that the settings have changed to enable the "apply" button
					PropSheet_Changed((HWND) GetWindowLong(hWndDlg, GWL_USERDATA), hWndDlg);
					return TRUE;
			}
			break;

		case WM_NOTIFY:
			switch (((NMHDR*) lParam)->code) {
				case PSN_SETACTIVE:
					// Save the property sheet handle for future use
					SetWindowLong(hWndDlg, GWL_USERDATA, (LONG) ((NMHDR*) lParam)->hwndFrom);
					return TRUE;
				case PSN_APPLY:
					// Validate the user's input
					GetDlgItemText(hWndDlg, IDC_EDIT_SERVER, buffer, MAX_SETTING);
					if (!lstrlen(buffer)) {
						MessageBox(hWndDlg, TEXT("You must specify a server"), 
							TEXT("Settings"), MB_OK | MB_ICONSTOP);
						SetFocus(GetDlgItem(hWndDlg, IDC_EDIT_SERVER));
						SetWindowLong(hWndDlg, DWL_MSGRESULT, PSNRET_INVALID);
						return TRUE;
					}
					
					// Write settings to the registry
					GetDlgItemText(hWndDlg, IDC_EDIT_SERVER, buffer, MAX_SETTING);
					RegWriteString(S_FTP_SERVER, buffer);
					RegWriteDWORD(S_PROTOCOL, 
						SendDlgItemMessage(hWndDlg, IDC_COMBO_PROTOCOL, CB_GETCURSEL, 0, 0));
					GetDlgItemText(hWndDlg, IDC_EDIT_PORT, buffer, MAX_SETTING);
					RegWriteDWORD(S_PORT, _ttoi(buffer));
					GetDlgItemText(hWndDlg, IDC_EDIT_HOSTKEY, buffer, MAX_SETTING);
					RegWriteString(S_SFTP_HOST_KEY, buffer);
					GetDlgItemText(hWndDlg, IDC_EDIT_USERNAME, buffer, MAX_SETTING);
					RegWriteString(S_FTP_USERNAME, buffer);
					GetDlgItemText(hWndDlg, IDC_EDIT_PASSWORD, buffer, MAX_SETTING);
					RegWriteString(S_FTP_PASSWORD, buffer);
					if (IsDlgButtonChecked(hWndDlg, IDC_CHECK_PASSIVE) == BST_CHECKED)
						ftpFlags = INTERNET_FLAG_PASSIVE;
					RegWriteDWORD(S_FTP_FLAGS, ftpFlags);
					return TRUE;
			}
			break;
	}
	return FALSE;
}

// Dialog procedure for the progress dialog
INT_PTR CALLBACK ProgressProc(HWND hWndDlg,
    UINT msg,
    WPARAM wParam,
    LPARAM lParam)
{
	return FALSE;
}

// Dialog procedure for the About tab
INT_PTR CALLBACK AboutProc(HWND hWndDlg,
    UINT msg,
    WPARAM wParam,
    LPARAM lParam)
{
	TCHAR buffer[1024];
	
	switch (msg) {
		case WM_INITDIALOG:
			GetAboutText(buffer, sizeof(buffer) / sizeof(TCHAR));
			SetWindowText(GetDlgItem(hWndDlg, IDC_STATIC_ABOUT), buffer);
			return TRUE;
	}
	return FALSE;
}

void ShowSettingsDlg() {
	PROPSHEETHEADER psh;
	PROPSHEETPAGE psp[3];
	int i;

	// Create and show a Windows property sheet with 3 tabs
	InitCommonControls();
	ZeroMemory(&psp, sizeof(psp));
	ZeroMemory(&psh, sizeof(psh));

	for (i = 0; i < sizeof(psp) / sizeof(PROPSHEETPAGE); i++) {
		psp[i].dwSize = sizeof(PROPSHEETPAGE);
		psp[i].hInstance = g_hModule;
	}
	psp[0].pszTemplate = MAKEINTRESOURCE(IDD_DIALOG_PLUGIN);
	psp[0].pfnDlgProc = PluginSettingsProc;
	psp[1].pszTemplate = MAKEINTRESOURCE(IDD_DIALOG_SERVER);
	psp[1].pfnDlgProc = ServerSettingsProc;
	psp[2].pszTemplate = MAKEINTRESOURCE(IDD_DIALOG_ABOUT);
	psp[2].pfnDlgProc = AboutProc;

	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPTITLE | PSH_PROPSHEETPAGE;
	psh.hInstance = g_hModule;
	psh.pszCaption = TEXT("pgFTP Plugin");
	psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGE);
	psh.ppsp = psp;

	PropertySheet(&psh);
}

HWND ShowProgressDlg() {
	HWND hWnd;
	RECT rect;
	int w, h;

	// Note that this is a modeless dialog
	hWnd = CreateDialog(g_hModule, MAKEINTRESOURCE(IDD_DIALOG_PROGRESS), NULL, ProgressProc);
	// Center the dialog on the screen
	GetWindowRect(hWnd, &rect);
	w = rect.right - rect.left;
	h = rect.bottom - rect.top;
	MoveWindow(hWnd, GetSystemMetrics(SM_CXSCREEN) / 2 - w / 2,
		GetSystemMetrics(SM_CYSCREEN) / 2 - h / 2, w, h, TRUE);
	ShowWindow(hWnd, SW_SHOW);

	return hWnd;
}