/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */

#include "stdafx.h"
#include "ftpfuncs.h"
#include "misc.h"

// See ftpfuncs.h for function descriptions

// Used to package a progress callback and its context together for
// InternetStatusCallback below
typedef struct {
	PFN_PROGRESS pfnProgress;
	LPVOID pContext;
} PROGRESS_CALLBACK_INFO;

// wininet calls this function to update ftp transfer progress,
// which this function then delegates to another application-defined callback
void CALLBACK InternetStatusCallback(
  HINTERNET hInternet,
  DWORD_PTR dwContext,
  DWORD dwInternetStatus,
  LPVOID lpvStatusInformation,
  DWORD dwStatusInformationLength) 
{
	PROGRESS_CALLBACK_INFO* pci = (PROGRESS_CALLBACK_INFO*)dwContext;
	
	if (pci && (INTERNET_STATUS_RESPONSE_RECEIVED == dwInternetStatus ||
			INTERNET_STATUS_REQUEST_SENT == dwInternetStatus)) 
		pci->pfnProgress(*(DWORD*)lpvStatusInformation, pci->pContext);
}

BOOL pgFtpConnect(LPVOID* pphConnect, LPCTSTR server, int port, LPCTSTR user, LPCTSTR password, DWORD flags) {

	HINTERNET hInternet = InternetOpen(TEXT("pGina"), INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);
	
	if (!hInternet) {
		*pphConnect = NULL;
		return FALSE;
	}
	
	*pphConnect = InternetConnect(hInternet, server, port, 
		user, password, INTERNET_SERVICE_FTP, flags, 0);
				
	return (*pphConnect != NULL);
}

DWORD pgFtpGetLastError(LPVOID hConnect) {
	return GetLastError();	
}

BOOL pgFtpGetFileSize(LPVOID hConnect, LPCTSTR filename, PUINT64 pnSize) {
	HINTERNET hFind = NULL;
	WIN32_FIND_DATA findData;

	ZeroMemory(&findData, sizeof(WIN32_FIND_DATA));
	hFind = FtpFindFirstFile(hConnect, filename, &findData, 0, 0);
	InternetCloseHandle(hFind);
	if (hFind)
		*pnSize = (((UINT64)findData.nFileSizeHigh) << 32) | findData.nFileSizeLow;

	return hFind != NULL;
}

BOOL pgFtpCreatePath(HINTERNET hConnect, LPCTSTR path) {
	LPTSTR tempPath;
	int len = lstrlen(path);
	BOOL ret = TRUE;
	int i;
	
	tempPath = (LPTSTR)LocalAlloc(LMEM_FIXED, (len + 2) * sizeof(TCHAR));
	StringCchCopy(tempPath, len + 2, path);
	if (path[len - 1] != TEXT('/')) {
		// Force path to have terminating '/'
		len++;
		tempPath[len - 1] = TEXT('/');
		tempPath[len] = TEXT('\0');
	}
	
	for (i = 0; i < len; i++) {
		if (tempPath[i] == TEXT('/')) {
			tempPath[i] = TEXT('\0');
			ret = FtpCreateDirectory(hConnect, tempPath);
			tempPath[i] = TEXT('/');
		}
	}
	LocalFree(tempPath);

	return ret; 
}

BOOL pgFtpGet(LPVOID hConnect, LPCTSTR remoteFile, LPCTSTR localFile, 
			  PFN_PROGRESS pfnProgress, LPVOID pContext)
{
	PROGRESS_CALLBACK_INFO pci;
	BOOL ret;

	if (pfnProgress) {
		// Register for progress updates with wininet
		InternetSetStatusCallback(hConnect, InternetStatusCallback);
		// Package the information needed by the wininet callback
		// to call pfnProgress
		pci.pfnProgress = pfnProgress;
		pci.pContext = pContext;
	}
	ret = FtpGetFile(hConnect, remoteFile, localFile, FALSE, 0, 
		FTP_TRANSFER_TYPE_BINARY | INTERNET_FLAG_RELOAD, 
		(DWORD_PTR)&pci);
	// Unregister wininet progress function
	InternetSetStatusCallback(hConnect, NULL);
	return ret;
}

BOOL pgFtpPut(LPVOID hConnect, LPCTSTR localFile, LPCTSTR remoteFile, 
			  PFN_PROGRESS pfnProgress, LPVOID pContext)
{
	PROGRESS_CALLBACK_INFO pci;
	BOOL ret;

	if (pfnProgress) {
		// Register for progress updates with wininet
		InternetSetStatusCallback(hConnect, InternetStatusCallback);
		// Package the information needed by the wininet callback
		// to call pfnProgress
		pci.pfnProgress = pfnProgress;
		pci.pContext = pContext;
	}
	ret = FtpPutFile(hConnect, localFile, remoteFile, FTP_TRANSFER_TYPE_BINARY, (DWORD_PTR)&pci);
	// Unregister wininet progress function
	InternetSetStatusCallback(hConnect, NULL);
	return ret;
}

BOOL pgFtpDelete(LPVOID hConnect, LPCTSTR filename) {
	return FtpDeleteFile(hConnect, filename);
}

BOOL pgFtpRename(LPVOID hConnect, LPCTSTR filename, LPCTSTR newFilename) {
	return FtpRenameFile(hConnect, filename, newFilename);
}

void pgFtpClose(LPVOID hConnect) {
	InternetCloseHandle(hConnect);
}
