/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */

#ifndef __ftpfuncs_h_
#define __ftpfuncs_h_

/*********************************************************
 * ftpfuncs.h - FTP related functions
 *********************************************************/

// Callback function prototype for file transfer progress
typedef void (*PFN_PROGRESS) (DWORD bytesTransferred, LPVOID pContext);

/**
 * Opens a connection to the FTP server
 **/
BOOL pgFtpConnect(LPVOID* pphConnect, LPCTSTR server, int port, LPCTSTR user, LPCTSTR password, DWORD flags);

/**
 * Gets a description of the last error that occured. You should call this function
 * immediately before calling any other Win32 functions. Returns TRUE if successful,
 * FALSE otherwise.
 **/
DWORD pgFtpGetLastError(LPVOID hConnect);

/**
 * Gets the size of a remote file.
 * Returns TRUE on success, FALSE otherwise.
 **/
BOOL pgFtpGetFileSize(LPVOID hConnect, LPCTSTR filename, PUINT64 pnSize);

/**
 * Creates a directory and all parent directories on a server
 */
BOOL pgFtpCreatePath(HINTERNET hConnect, LPCTSTR path);

/**
 * Downloads a file from the ftp server using binary mode. If pfnProgress is non-NULL,
 * it specifies a callback that gets periodically called as progress is made.
 * pContext is a pointer to application-defined data that gets passed to 
 * the progress function whenever it's called.
 * Returns TRUE on success, or FALSE otherwise.
 */
BOOL pgFtpGet(LPVOID hConnect, LPCTSTR remoteFile, LPCTSTR localFile,  
			   PFN_PROGRESS pfnProgress, LPVOID pContext);

/**
 * Uploads a file to the ftp server using binary mode. If pfnProgress is non-NULL,
 * it specifies a callback that gets periodically called as progress is made.
 * pContext is a pointer to application-defined data that gets passed to 
 * the progress function whenever it's called.
 * Returns TRUE on success, or FALSE otherwise.
 */
BOOL pgFtpPut(LPVOID hConnect, LPCTSTR localFile, LPCTSTR remoteFile,  
			   PFN_PROGRESS pfnProgress, LPVOID pContext);

/**
 * Renames a remote file on the server.
 * Returns TRUE on success, or FALSE otherwise.
 */
BOOL pgFtpRename(LPVOID hConnect, LPCTSTR filename, LPCTSTR newFilename);

/**
 * Deletes a remote file on the server.
 * Returns TRUE on success, or FALSE otherwise.
 */
BOOL pgFtpDelete(LPVOID hConnect, LPCTSTR filename);

/**
 * Closes a connection to the remote server.
 */
void pgFtpClose(LPVOID hConnect);


#endif //__ftpfuncs_h_