/*

	Copyright (C) 2004 University of Calgary

	

	Contact: Eric Parsons (ericpar@math.ucalgary.ca)



	This file is part of pgftp.



    pgftp is free software; you can redistribute it and/or modify

    it under the terms of the GNU General Public License as published by

    the Free Software Foundation; either version 2 of the License, or

    (at your option) any later version.



    pgftp is distributed in the hope that it will be useful,

    but WITHOUT ANY WARRANTY; without even the implied warranty of

    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

    GNU General Public License for more details.



    You should have received a copy of the GNU General Public License

    along with pgftp; if not, write to the Free Software

    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



 */



#ifndef __ftpfuncs_h_

#define __ftpfuncs_h_



/*********************************************************

 * ftpfuncs.h - FTP related functions

 *********************************************************/



/**

 * Opens a connection to the FTP server used for profiles.

 **/

HINTERNET OpenProfileFtp(LPCTSTR server, LPCTSTR user, LPCTSTR password);



/**

 * Creates a directory and all parent directories on a server

 */

BOOL FtpCreatePath(HINTERNET hConnect, LPCTSTR path);





// The following function are no longer used. Their implementation is 

// being kept in ftpfuncs.c in case they are one day needed again.



/**

 * Downloads one or more files (and/or folders) from an open

 * FTP connection

 * @param hConnect The ftp connection handle

 * @param sourcePath The path to copy files from. Can be relative or

 *		absolute. If NULL, the current FTP directory is used.

 * @param destPath Where to copy the files (the directory must exist)

 * @param fileName Name of file or directory to download. Wildcards are

 *		permitted, in which case all matches will be downloaded.

 * @param includeDirectories Specifies whether to recursively download

 *		directories as well. Any directories which don't exist in destPath

 *		are created (even if they are emtpy)

 * @param pFilesCopied [out] Returns the number of files that were copied.

 * @return TRUE if successful, otherwise FALSE. If FALSE, you can try calling

 *		GetLastError().

 **/

BOOL FtpDownload(HINTERNET hConnect, 

								 LPCTSTR sourcePath, 

								 LPCTSTR destPath, 

								 LPCTSTR fileName,

								 BOOL includeDirectories,

								 DWORD *pFilesCopied);



/**

 * Uploads one or more files (and/or folders) to an open

 * FTP connection.

 *

 * @param hConnect The ftp connection handle

 * @param sourcePath The path to copy files from (the directory must exist).

 * @param destPath Where to copy the files. Can be relative or

 * 		absolute. If NULL, the current FTP directory is used.

 * @param fileName Name of file or directory to upload. Wildcards are

 * 		permitted, in which case all matches will be uploaded.

 * @param includeDirectories Specifies whether to recursively upload

 *		directories as well. Any directories which don't exist in destPath

 *		are created (even if they are emtpy)

 * @param pFilesCopied [out] Returns the number of files that were copied.

 * @return TRUE if successful, otherwise FALSE. If FALSE, you can try calling

 * 		GetLastError().

 **/

BOOL FtpUpload(HINTERNET hConnect, 

							LPCTSTR sourcePath,

							LPCTSTR destPath,

							LPCTSTR fileName,

							BOOL includeDirectories,

							DWORD *pFilesCopied);





/**

 * Similar to the WinInet function FtpFindFirstFile, except that it

 * works correctly with paths that contain spaces. It is, however,

 * somewhat slower. It will still fail if the filename contains 

 * spaces (you must use wildcards in this case)	

 **/

HINTERNET FtpFindFirstFileInPath(HINTERNET hConnect,

																 LPCTSTR path,

																 LPCTSTR fileName,

																 LPWIN32_FIND_DATA findData);



/**

 * Deletes a directory from an FTP server, including all 

 * content files and subdirectories. Use with caution.

 *

 * @return TRUE on success, FALSE otherwise. If the function fails

 *	for some reason, the contents of the directory may or may not

 *	be partially deleted.

 **/

BOOL FtpDeleteDirectory(HINTERNET hConnect, LPCTSTR path);



#endif //__ftpfuncs_h_