/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */

#include "stdafx.h"
#include "misc.h"
#include "pgftp.h"
#include "resource.h"
#include "ftpfuncs.h"
#include "sftpfuncs.h"

// See misc.h for function documentation

LPTSTR lstrdup(LPCTSTR str) {
	LPTSTR dup;
	int size = lstrlen(str) + 1;

	if (NULL == str)
		return NULL;

	dup = (TCHAR *)LocalAlloc(LMEM_FIXED, size * sizeof(TCHAR));
	if (dup) 
		StringCchCopy(dup, size, str);

	return dup;
}


LPTSTR RegReadString(LPCTSTR name)
{
	HKEY hKey = NULL;
	DWORD dwBytesRead = 2048;
	TCHAR result[2048];

	if (ERROR_SUCCESS == RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("Software\\pGina\\pgftp"), 0, KEY_QUERY_VALUE, &hKey))
		if (ERROR_SUCCESS == RegQueryValueEx(hKey, name, NULL, NULL, (LPBYTE)&result, &dwBytesRead)) 
			if (dwBytesRead > 0 && lstrlen(result)) { 
				RegCloseKey(hKey);
				return lstrdup(result);                                                                                 
			}
				
  RegCloseKey(hKey);
  return NULL;
}

DWORD RegReadDWORD(LPCTSTR name, int defaultValue) {
	HKEY hKey = NULL;
	DWORD result;
	DWORD dwBytesRead = sizeof(result);

	if (ERROR_SUCCESS == RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("Software\\pGina\\pgftp"), 0, KEY_QUERY_VALUE, &hKey))
		if (ERROR_SUCCESS == RegQueryValueEx(hKey, name, NULL, NULL, (LPBYTE)&result, &dwBytesRead)) {
			RegCloseKey(hKey);
			return result;
		}
	RegCloseKey(hKey);
	return defaultValue;
}

LONG RegWriteString(LPCTSTR name, LPCTSTR value)
{
	HKEY hKey = NULL;
	LONG ret;
	
	ret = RegCreateKeyEx(HKEY_LOCAL_MACHINE, TEXT("Software\\pGina\\pgftp"), 0, NULL, 
		REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, NULL);
	if (ret != ERROR_SUCCESS)
		return ret;

	ret = RegSetValueEx(hKey, name, 0, REG_SZ, (LPBYTE)value, (lstrlen(value) + 1) * sizeof(TCHAR));			
  RegCloseKey(hKey);
  return ret;
}

LONG RegWriteDWORD(LPCTSTR name, DWORD value) {
	HKEY hKey = NULL;
	LONG ret;

	ret = RegCreateKeyEx(HKEY_LOCAL_MACHINE, TEXT("Software\\pGina\\pgftp"), 0, NULL,
		REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, NULL);
	if (ret != ERROR_SUCCESS)
		return ret;

	ret = RegSetValueEx(hKey, name, 0, REG_DWORD, (LPBYTE)&value, sizeof(value));			
  RegCloseKey(hKey);
  return ret;
}

BOOL LoadSettings(PLUGIN_SETTINGS* pSettings) {
	ZeroMemory(pSettings, sizeof(PLUGIN_SETTINGS));

	pSettings->ftpServer = RegReadString(S_FTP_SERVER);
	if (!pSettings->ftpServer)
		return FALSE;

	pSettings->profileTempRoot = RegReadString(S_PROFILE_TEMP_ROOT);
	if (!pSettings->profileTempRoot)
		pSettings->profileTempRoot = lstrdup(DEFAULT_PROFILE_TEMP_ROOT);
	pSettings->ftpUserName = RegReadString(S_FTP_USERNAME);
	pSettings->ftpPassword = RegReadString(S_FTP_PASSWORD);
	pSettings->compression = RegReadDWORD(S_COMPRESSION, DEFAULT_COMPRESSION);
	pSettings->protocol = RegReadDWORD(S_PROTOCOL, DEFAULT_PROTOCOL);
	pSettings->port = RegReadDWORD(S_PORT, 
		pSettings->protocol == PROTOCOL_SFTP ? DEFAULT_SFTP_PORT : DEFAULT_FTP_PORT);
	pSettings->role = RegReadDWORD(S_ROLE, ROLE_PROFILE);

	if (pSettings->protocol == PROTOCOL_FTP)
		pSettings->flags = RegReadDWORD(S_FTP_FLAGS, INTERNET_FLAG_PASSIVE);
	else {
		pSettings->flags = RegReadDWORD(S_SFTP_FLAGS, 0); // Currently no flags are implemented
		pSettings->sftpHostKey = RegReadString(S_SFTP_HOST_KEY);
	}

	
	pSettings->required = RegReadDWORD(S_REQUIRED, (pSettings->role & ROLE_AUTH) > 0);

	return TRUE;
}

void FreeSettings(PLUGIN_SETTINGS* pSettings) {
	LocalFree(pSettings->ftpServer);
	LocalFree(pSettings->ftpUserName);
	LocalFree(pSettings->ftpPassword);
	LocalFree(pSettings->profileTempRoot);
	LocalFree(pSettings->sftpHostKey);
}

BOOL FormatError(DWORD err, LPTSTR buffer, DWORD cchBuffer) {
	DWORD cchMsg = 0;

	if (!buffer || !cchBuffer)
		return FALSE;

	if (err >= 12000 && err <= 12174) {
		// This is a wininet error code
		if (err == ERROR_INTERNET_EXTENDED_ERROR) {
			cchMsg = (DWORD) InternetGetLastResponseInfo(&err, buffer, &cchBuffer);
		}
		else {
			cchMsg = FormatMessage(FORMAT_MESSAGE_FROM_HMODULE, 
				GetModuleHandle(TEXT("wininet.dll")), err, 
				0, buffer, cchBuffer, NULL);
		}			
	}
	else if (err & 0x20000000) {
		// This is a user defined error. The error code corresponds to a string
		// resource in this module.
		cchMsg = LoadString(g_hModule, err & 0xDFFFFFFF, buffer, cchBuffer);
	}
	else {
		// Get error from system instead
		cchMsg = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 
			NULL, err, 0, buffer, cchBuffer, NULL);
	}

	if (cchMsg)
		return TRUE;
	else {
		StringCchCopy(buffer, cchBuffer, TEXT("Error description unavailable"));
		return FALSE;
	}
}

void GetAboutText(LPTSTR buffer, DWORD cchBufferSize) {
	TCHAR szModuleFileName[MAX_PATH];
	DWORD dwHandle;
	DWORD dwVersionInfoSize;
	BYTE* pVersionInfo;
	LPTSTR szCopyright;
	LPTSTR szProductName;
	LPTSTR szVersion;
	TCHAR szAuthor[128];
	UINT uSize;

	// Retrieve and format version info from the version resource
	GetModuleFileName(g_hModule, szModuleFileName, MAX_PATH);
	dwVersionInfoSize = GetFileVersionInfoSize(szModuleFileName, &dwHandle);
	pVersionInfo = LocalAlloc(LMEM_FIXED, dwVersionInfoSize);
	GetFileVersionInfo(szModuleFileName, dwHandle, dwVersionInfoSize, pVersionInfo);
	VerQueryValue(pVersionInfo, TEXT("\\StringFileInfo\\040904B0\\ProductName"), &szProductName, &uSize);
	VerQueryValue(pVersionInfo, TEXT("\\StringFileInfo\\040904B0\\ProductVersion"), &szVersion, &uSize);
	VerQueryValue(pVersionInfo, TEXT("\\StringFileInfo\\040904B0\\LegalCopyright"), &szCopyright, &uSize);
	LoadString(g_hModule, IDS_AUTHOR, szAuthor, sizeof(szAuthor));
	StringCchPrintf(buffer, cchBufferSize, TEXT("%s\r\n\r\nVersion %s\r\nAuthor: %s\r\n\r\n%s"), szProductName, szVersion, szAuthor, szCopyright);
	LocalFree(pVersionInfo);
}