/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */

#ifndef __misc_h_
#define __misc_h_

#include "pgftp.h"

/*********************************************************
 * misc.h - Miscellaneous functions
 *********************************************************/

// Macro used to load a string from the resource table
// into an array. Note that using a non-array pointer will
// fail since sizeof(arr) will always be sizeof(INT_PTR) !
#define GetString(sid, arr) LoadString(g_hModule, (sid), (arr), sizeof(arr) / sizeof(TCHAR))

// Converts a string resource id to a Win32 error code. Windows reserves
// bit 29 for user-defined errors.
#define MAKE_ERROR(s) (s | 0x20000000)

// These macros convert between TCHAR and ANSI strings or TCHAR and UNICODE strings.
// _alloca() allocates memory on the stack instead of the heap, so no cleanup is necessary.
// You must specify INIT_CONVERSION in the declaration section of a function before using
// either T2A or A2T.

#define INIT_CONVERSION LPVOID __conv_temp = NULL; 

#ifdef UNICODE

#define T2A(x) (__conv_temp = _alloca(lstrlen(x) + 1), \
	WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)x, -1, __conv_temp, lstrlen(x) + 1, NULL, NULL), \
	__conv_temp)

#define A2T(x) (__conv_temp = _alloca((lstrlen(x) + 1) * sizeof(wchar_t)), \
	MultiByteToWideChar(CP_ACP, 0, (LPCSTR)x, -1, __conv_temp, (lstrlen(x) + 1) * sizeof(wchar_t)), \
	__conv_temp)

#define T2W(x) x
#define W2T(x) x

#else

#define T2A(x) x
#define A2T(x) x

#define T2W(x) (__conv_temp = _alloca((lstrlen(x) + 1) * sizeof(wchar_t)), \
	MultiByteToWideChar(CP_ACP, 0, (LPCSTR)x, -1, __conv_temp, (lstrlen(x) + 1) * sizeof(wchar_t)), \
	__conv_temp)

#define W2T(x) (__conv_temp = _alloca(lstrlen(x) + 1), \
	WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)x, -1, __conv_temp, lstrlen(x) + 1, NULL, NULL), \
	__conv_temp)

#endif // UNICODE

/**
 * Duplicates a string. The resulting string must be freed
 * with LocalFree(). This function provides an alternative 
 * to _tcsdup, which requires linking to MSVCRT.DLL.
 * Obsolete since the C runtime is now required due to
 * libssh2.
 **/
LPTSTR lstrdup(LPCTSTR str);

/**
 * Reads a string value from the pgftp registry location.
 * The returned string must be freed with LocalFree().
 **/
LPTSTR RegReadString(LPCTSTR name);

/**
 * Reads a DWORD value from the pgftp registry location.
 **/
DWORD RegReadDWORD(LPCTSTR name, int defaultValue);

/**
 * Writes a string value to the pgftp registry location.
 **/
LONG RegWriteString(LPCTSTR name, LPCTSTR value);

/**
 * Writes a DWORD value to the pgftp registry location.
 **/
LONG RegWriteDWORD(LPCTSTR name, DWORD value);

/**
 * Populates a PLUGIN_SETTINGS struct with information from the registry
 **/
BOOL LoadSettings(PLUGIN_SETTINGS*);

/**
 * Must be called after LoadSettings()
 **/
void FreeSettings(PLUGIN_SETTINGS*);

/**
 * Retrieves a string describing the plugin
 **/
void GetAboutText(LPTSTR buffer, DWORD cchBufferSize);

/**
 * Formats an error code into a human-readable string
 **/
BOOL FormatError(DWORD err, LPTSTR buffer, DWORD cchBuffer);



#endif // __misc_h_