/*
* PAM client module for the pagina_pam UNIX server
* based on pam_imap
* pam_pgina v0.4
* http://pam-imap.sf.net
*
* Copyright (C) 2004 Cal Heldenbrand
*	<heldenca@mnstate.edu> or <calzplace@users.sf.net>
*
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*       Thanks to Nathan Yocom!  Borrowed helper functions for this
*	client.
* Email: nate.yocom@xpasystems.com
* Web: http://pgina.xpasystems.com
*
*/


/****    A note to programmers out there  ****
*  Please note, I do not code this cruel usually, this
*  module is the combination of copy/paste of some code
*  I've done previously -- a total of about 2 HR's work!
*
*  If you'd like to make improvments, GO RIGHT AHEAD!!!  :-)
*/

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <stdarg.h>
#include <alloca.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>

#include <errno.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

/* #include "parseconf.h"

#ifdef HAVE_CONFIG_H
  #include <config.h>
  #include "config.h"
#endif
#include "isync.h"
*/

/*
* here, we make definitions for the externally accessible functions
* in this file (these definitions are required for static modules
* but strongly encouraged generally) they are used to instruct the
* modules include file to define their prototypes.
*/

#define PAM_SM_AUTH
#define PAM_SM_ACCOUNT
#define PAM_SM_SESSION
#define PAM_SM_PASSWORD

// use pam/ rather than security/ for darwin
#ifdef DARWIN
#include <pam/pam_modules.h>
#include <pam/pam_misc.h>
#else
#include <security/pam_modules.h>
#include <security/pam_misc.h>
#endif


// Our messages for sending to the server
#define AUTH_ACTION "AUTH"
#define SUCCESS "SUCC"
#define EXPIRED "EXP"
#define ERROR "ERR"
#define CHANGE_PASS "CHPS"
#define GET_GROUP "GGRP"
#define LOCKEDOUT "LKD"
#define DONE "!"


/* Globals (YES, I know this is a bad programming practice) */
const char *pam_passwd = NULL; /* place to remember password */
const char *pam_username = NULL;


/* Prototypes */
int converse (pam_handle_t * pamh,
	int nargs,
	struct pam_message **message,
	struct pam_response **response);

int _set_auth_tok (pam_handle_t * pamh,
	int flags,
	int argc,
	const char **argv);


int askForPassword(pam_handle_t *pamh, int type, const char *message);

void send_string(SSL *,char *);
char * read_string(SSL *);
SSL * connect_to_server(char *);
char * server_talk(char *hostname, int num, ...);


/*** Cal's prototypes taken from pam_mysql.c ***/

PAM_EXTERN int pam_sm_authenticate(pam_handle_t *pamh,
		                int flags, int argc, const char **argv);
PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc,
		                const char **argv);
PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc,
		                const char **argv);
PAM_EXTERN int pam_sm_setcred(pam_handle_t *pamh, int flags, int argc,
		                const char **argv);
PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc,
		                const char **argv);
PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags, int argc,
		                const char **argv);


/* Global PAM functions stolen from other modules */


int converse(pam_handle_t *pamh, int nargs,
	struct pam_message **message,
	struct pam_response **response)
{
	int retval;
	struct pam_conv *conv;

	retval = pam_get_item( pamh, PAM_CONV, (const void **) &conv ) ;
	if ( retval == PAM_SUCCESS )
	{
		retval = conv->conv(nargs, ( const struct pam_message ** )message,
				response, conv->appdata_ptr);

		if ((retval != PAM_SUCCESS) && (retval != PAM_CONV_AGAIN))
		{
			syslog(LOG_DEBUG, "pam_pgina: conversation failure [%s]",
				pam_strerror(pamh, retval));
		}
	}
	else
	{
		syslog(LOG_ERR, "pam_pgina: couldn't obtain coversation function [%s]",
			pam_strerror(pamh, retval));
	}
	return retval; /* propagate error status */
}

int askForPassword(pam_handle_t *pamh, int type, const char *message)
{
	struct pam_message msg[3], *mesg[3];
	struct pam_response *resp=NULL;
	char *prompt = NULL;
	int i=0;
	int retval;

		//msg[i].msg = (char *)parseconf("PAM_PasswordString", config_file);
	msg[i].msg = strdup(message);
	msg[i].msg_style = PAM_PROMPT_ECHO_OFF;
	mesg[i] = &msg[0];
	retval = converse(pamh, ++i, mesg, &resp);
	if (prompt)
	{
		_pam_overwrite(prompt);
		_pam_drop(prompt);
	}
	if (retval != PAM_SUCCESS)
	{
		if (resp != NULL)
			_pam_drop_reply(resp,i);
	return ((retval == PAM_CONV_AGAIN) ? PAM_INCOMPLETE:PAM_AUTHINFO_UNAVAIL);
	}

	/* we have a password so set AUTHTOK */
	return pam_set_item(pamh, type, resp->resp);
}

/*  Functions stolen from pGina PAM module DLL */

void send_string(SSL *mySSL, char *message)
{
	int error = 0, wrote = 0;

	for(wrote = 0; wrote <= strlen(message); wrote++)
	{
		error = SSL_write(mySSL,&message[wrote],1);

		if(error <= 0)
			break;
	}
	return;
}
char * read_string(SSL *mySSL)
{
	char buffer[1024];
	char *ret;
	char this_one;
	int error = 0, read_in = 0;

	memset(buffer,'\0',sizeof(buffer));

	for(read_in = 0; read_in < sizeof(buffer); read_in += error)
	{
		error = SSL_read(mySSL,&this_one,1);
		buffer[read_in] = this_one;

		if(error <= 0 || this_one == '\0' || this_one == '\n')
			break;
	}

	// Buffer is good here
	ret = (char *) malloc(strlen(buffer) + 1);
	strcpy(ret,buffer);

	return ret;
}


//SSL * connect_to_server(pGinaInfo *settingsInfo)
SSL * connect_to_server(char *pam_host)
{
	SSL *ssl;
	SSL_CTX *ctx;
	BIO *conn;
	//char *pam_host;
	
#ifdef DEBUG
	printf("connect_to_server %s...\n", pam_host);
#endif
		// Init OpenSSL
	OpenSSL_add_all_algorithms();
	SSL_load_error_strings();
	/* NOTE:  Any SSL errors with pam_pgina v0.3 
	 * are most likely the result of this function
	 * call not being here.  Please email Cal
	 * (calzplace@users.sf.net) if you still get
	 * SSL errors with version 0.4
	 */
	SSL_library_init();

	// Init context
	ctx = SSL_CTX_new(TLSv1_method());
	/* uncomment if SSL v2 or v3 is needed */
	//ctx = SSL_CTX_new(SSLv23_client_method());

	if(ctx == NULL)
	{
		syslog(LOG_ERR, "pam_pgina: Error creating context\n");
#ifdef DEBUG
		printf("Error creating TLSv1 context \n");
		printf("SSL error: %s\n", ERR_error_string( ERR_get_error(), NULL));

#endif
		return NULL;
	}

	// Init SSL
	ssl = SSL_new(ctx);
	if(!ssl)
	{
		syslog(LOG_ERR, "pam_pgina: Error creating SSL.");
#ifdef DEBUG
		printf("Error creating SSL \n");
		printf("SSL error: %s\n", ERR_error_string( ERR_get_error(), NULL));
#endif
		return NULL;
	}

	conn = BIO_new_connect(pam_host);

	if(!conn)
	{
		syslog(LOG_ERR, "pam_pgina: Error in BIO_new_connect");
#ifdef DEBUG
		printf("Error in BIO_new_connect\n");
#endif
		return NULL;
	}

	if(BIO_do_connect(conn) <= 0)
	{
		syslog(LOG_ERR, "pam_pgina: Error in BIO_do_connect");
#ifdef DEBUG
		printf("Error in BIO_do_connect\n");
#endif
		return NULL;
	}

	SSL_set_bio(ssl,conn,conn);

	if(SSL_connect(ssl) <= 0)
	{
		syslog(LOG_ERR, "pam_pgina: Error in SSL handshake");
		return NULL;
	}

	return ssl;
}

/*  Written by Cal -- takes hostname to connect to,
 *  then the number of commands in the variable arg (char *) list.
 *      Ex:  server_talk("myhost:299", 3, "AUTH", "user", "pass");
 *  server will return whatever spits out of read_string.
 *  note that this function opens & closes the connection
 *  so no need to worry about that.
 *
 *  This function is intended for a single COMMAND that requires
 *  several writes to the server, then a single read.
 *  Trying to do an AUTH and a GGRP and a CHPS all in one
 *  function call will not work.
 */
char * server_talk(char *hostname, int num, ...)
{
	va_list list;
	char *temp;
	char *result;
	int i;
	SSL *ssl;

	va_start(list, num);

#ifdef DEBUG
	printf("connecting to %s...\n", hostname);
#endif

	ssl = connect_to_server(hostname);

	if ( !ssl )
	{
		syslog(LOG_ERR, "pam_pgina:\tconnect_to_server error at host [%s]\n", hostname);
#ifdef DEBUG
		printf("Error setting up SSL to %s...\n", hostname);
#endif
		return NULL; 
	}

	for ( i = 0 ; i < num ; i++)
	{
		(temp = va_arg(list, char *));
		send_string(ssl, temp);
	}

	result = read_string(ssl);

	/*  Be nice and close properly */
	send_string(ssl,"!");
	SSL_shutdown(ssl);

	va_end(list);

	return result;
}



/* PAM Authentication functions */

PAM_EXTERN int pam_sm_authenticate (pam_handle_t * pamh,
	int flags,
	int argc,
	const char **argv)
{
	int retval;
	const char *user;
	const char *passwd;
	char *result;
	char *hostname = NULL;

	/* if this is a successive call, reset our passwords
	 * example:  if the 1st login attempt is incorrect
	 * this will reset all passwd variables to allow
	 * a 2nd input attempt */
	passwd = NULL;
	//global.pass = NULL;
	pam_passwd = 0 ;

/*******************  Find get pgina_pam's servername:host ****************************/

	if ( argc == 0 )  /* called w/o any args */
	{
		/* be mean here, and exit with PAM_CRED_INSUFFICIENT */
		return PAM_CRED_INSUFFICIENT;
	}
	if ( argc == 1 )
	{  /* be cruel here, and simply copy the string */
		hostname = strdup(argv[0]);
	}
#ifdef DEBUG
	printf("host=%s\n", hostname);
#endif


/******************************************************************/
	/* Get User */

	retval = pam_get_user (pamh, &user, NULL);
	if (retval != PAM_SUCCESS || user == NULL)
	{
		syslog (LOG_ERR, "pam_pgina: no user specified");
		return PAM_USER_UNKNOWN;
	}
	pam_username = user;

	if (passwd != NULL)
	{
		printf("password is not NULL, resetting\n");
		pam_set_item(pamh, PAM_AUTHTOK, (const void**) &passwd);
	}
	retval = pam_get_item(pamh, PAM_AUTHTOK, (const void **) &passwd);
	if ( passwd == NULL )
		askForPassword(pamh, PAM_AUTHTOK, "Password: ");
	retval = pam_get_item(pamh, PAM_AUTHTOK, (const void **) &passwd);
	if ( passwd == NULL )
		return PAM_AUTHINFO_UNAVAIL;
	pam_passwd = strdup(passwd);
	if ( strcmp(passwd, "") == 0 )
	{ /*  DO we want null passwords?
	   *  Note:  In the future I may provide the option
	   *  to allow NULL pwd's sent, but I don't think it
	   *  would EVER be used ... */
		return PAM_AUTH_ERR;
	}
/*************************************************************************/


	/*   Cal's NOTES:  these are the strings the server protocol accepts */
	/****************************************
	// Our messages for sending to the server
	#define AUTH_ACTION "AUTH"
	#define SUCCESS "SUCC"
	#define EXPIRED "EXP"
	#define ERROR "ERR"
	#define CHANGE_PASS "CHPS"
	#define GET_GROUP "GGRP"
	#define LOCKEDOUT "LKD"
	#define DONE "!"
	***************************/

	result = server_talk(hostname, 3, "AUTH", pam_username, pam_passwd);

	if ( result == NULL )
	{
		return PAM_AUTH_ERR;
	}
	if(!strcmp(result,"SUCC"))
	{
		return PAM_SUCCESS;
	}
	else if ( !strcmp(result, "EXP"))
	{
		return PAM_CRED_INSUFFICIENT;
	}

	return PAM_AUTH_ERR;

}/* end pam_sm_authenticate */


/* --- account management functions --- */
PAM_EXTERN int pam_sm_acct_mgmt (pam_handle_t * pamh,
	int flags,
	int argc,
	const char **argv)
{
/*
#ifdef DEBUG
	syslog (LOG_INFO, "pam_pgina: acct_mgmt called but not implemented.");
#endif
*/
	int retval;
	const char *user;
	char *passwd;
	char *result;
	char *hostname = NULL;

	/* if this is a successive call, reset our passwords
	 * example:  if the 1st login attempt is incorrect
	 * this will reset all passwd variables to allow
	 * a 2nd input attempt */
	passwd = NULL;
	//global.pass = NULL;
	pam_passwd = 0 ;

/*******************  Find get pgina_pam's servername:host ****************************/

	if ( argc == 0 )  /* called w/o any args */
	{
		/* be mean here, and exit with PAM_CRED_INSUFFICIENT */
		return PAM_CRED_INSUFFICIENT;
	}
	if ( argc == 1 )
	{  /* be cruel here, and simply copy the string */
		hostname = strdup(argv[0]);
	}
#ifdef DEBUG
	printf("host=%s\n", hostname);
#endif


/******************************************************************/
	/* Get User */

	retval = pam_get_user (pamh, &user, NULL);
	if (retval != PAM_SUCCESS || user == NULL)
	{
		syslog (LOG_ERR, "pam_pgina: no user specified");
		return PAM_USER_UNKNOWN;
	}
	pam_username = user;

/*************************************************************************/

	/*  Okay, we need to verify if the ACCOUNT exists right?
	 *  Well, the easiest way to do this with pgina's server
	 *  is to just check the group of our current user.
	 *  If we get an error back, it probably means the
	 *  account is not valid.  If there's a better way to
	 *  do this -- tell me!
	 */

	result = server_talk(hostname, 2, "GGRP", pam_username);

	if ( !strcmp(result, "ERR" ) )
		return PAM_USER_UNKNOWN;
	else
		return PAM_SUCCESS;
}

PAM_EXTERN
int pam_sm_setcred(pam_handle_t *pamh,
	int flags,
	int argc,
	const char **argv)
{
#ifdef DEBUG
	syslog(LOG_INFO, "pam_pgina: setcred called but not implemented.");
#endif
	return PAM_SUCCESS;
}

/* --- password management --- */

PAM_EXTERN
int pam_sm_chauthtok(pam_handle_t *pamh,int flags,int argc
,const char **argv)
{

/*
#ifdef DEBUG
	syslog(LOG_INFO, "pam_pgina: chauthtok called but not implemented.  \
		Password NOT CHANGED!");
#endif
*/
	int retval;
	const char *user;
	char *passwd;
	char *newpasswd;
	char *sTmp;
	char *result;
	char *hostname = NULL;

	/* if this is a successive call, reset our passwords
	 * example:  if the 1st login attempt is incorrect
	 * this will reset all passwd variables to allow
	 * a 2nd input attempt */
	passwd = NULL;
	//global.pass = NULL;
	pam_passwd = 0 ;

/*******************  Find get pgina_pam's servername:host ****************************/

	if ( argc == 0 )  /* called w/o any args */
	{
		/* be mean here, and exit with PAM_CRED_INSUFFICIENT */
		return PAM_CRED_INSUFFICIENT;
	}
	if ( argc == 1 )
	{  /* be cruel here, and simply copy the string */
		hostname = strdup(argv[0]);
	}
#ifdef DEBUG
	printf("host=%s\n", hostname);
#endif


/******************************************************************/
	/* Get User */

	retval = pam_get_user (pamh, &user, NULL);
	if (retval != PAM_SUCCESS || user == NULL)
	{
		syslog (LOG_ERR, "pam_pgina: no user specified");
		return PAM_USER_UNKNOWN;
	}
	pam_username = user;

	if( flags & PAM_PRELIM_CHECK )
	{
		if ( getuid() == 0 && !(flags & PAM_CHANGE_EXPIRED_AUTHTOK) )
			retval = PAM_SUCCESS;
		else
		{
			retval = pam_get_item(pamh, PAM_OLDAUTHTOK,
				(const void **) &passwd);
			if (passwd == NULL )
			{
				askForPassword(pamh, PAM_OLDAUTHTOK,
					"(Current) Password: ");
			}
			retval = pam_get_item(pamh, PAM_OLDAUTHTOK,
				(const void **) &passwd);
			if (passwd == NULL )
				retval = PAM_AUTHTOK_ERR;
			else
			{  /* check the server */

				result = server_talk(hostname, 3, "AUTH", pam_username, passwd);
				if(!strcmp(result,"SUCC"))
				{
					return PAM_SUCCESS;
				}
				return PAM_AUTHTOK_ERR;
			}
		}
	}
	if( flags & PAM_UPDATE_AUTHTOK )
	{
		retval = pam_get_item(pamh, PAM_AUTHTOK,
			(const void **) &newpasswd);
		if( retval )
			return PAM_AUTHTOK_ERR;
		if( newpasswd == NULL )
		{
			askForPassword(pamh, PAM_AUTHTOK,
				"(New) Password: ");
			retval = pam_get_item(pamh, PAM_AUTHTOK,
				(const void **) &newpasswd);
			if( retval )
				return PAM_AUTHTOK_ERR;
			sTmp = (char *) malloc(sizeof(char) *
				(strlen(newpasswd) + 2));
			strncpy (sTmp, newpasswd, strlen(newpasswd) + 2);
			askForPassword(pamh, PAM_AUTHTOK,
				"Retype (new) Password: ");
			retval = pam_get_item(pamh, PAM_AUTHTOK,
				(const void **) &newpasswd);
			if( newpasswd == NULL || strcmp(sTmp, newpasswd) )
				return PAM_AUTHTOK_ERR;
			free(sTmp);
		}

		retval = pam_get_item(pamh, PAM_OLDAUTHTOK,
			(const void **) &passwd);
		if( retval )
			return PAM_AUTHTOK_ERR;

#ifdef DEBUG
		printf("updating password server-side\n");
#endif

		/* Here comes all our server negotiations */

		result = server_talk(hostname, 4, "CHPS", pam_username, passwd, newpasswd);

		if(!strcmp(result,"SUCC"))
		{
			return PAM_SUCCESS;
		}
		else
			return PAM_AUTHTOK_ERR;
	}
	
	// backup case
	return PAM_AUTHTOK_ERR;
}

/* --- session management --- */

PAM_EXTERN
int pam_sm_open_session(pam_handle_t *pamh,
	int flags,
	int argc,
	const char **argv)
{
#ifdef DEBUG
	syslog(LOG_INFO, "pam_pgina: open_session called but not implemented.");
#endif
	return PAM_SUCCESS;
}

PAM_EXTERN
int pam_sm_close_session(pam_handle_t *pamh,
	int flags,
	int argc,
	const char **argv)
{
#ifdef DEBUG
	syslog(LOG_INFO, "pam_pgina: close_session called but not implemented.");
#endif
	return PAM_SUCCESS;
}

/* end of module definition */

#ifdef PAM_STATIC

/* static module data */

struct pam_module _pam_permit_modstruct = {
"pam_permit",
pam_sm_authenticate,
pam_sm_setcred,
pam_sm_acct_mgmt,
pam_sm_open_session,
pam_sm_close_session,
pam_sm_chauthtok
};

#endif

