/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */

// pgftp.c : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "pgftp.h"
#include "ftpfuncs.h"
#include "sftpfuncs.h"
#include "profile.h"
#include "auth.h"
#include "misc.h"
#include "dialog.h"
#include "resource.h"

// Globals

HMODULE g_hModule;
// This gets set to TRUE only when a user successfully logs in with a 
// roaming profile
BOOL g_restoreProfile;
BOOL g_userIsLoggedIn;
// Saves the profile filename that was matched on login
TCHAR g_archiveType[4];
// Saves the original profile path for the user
LPTSTR g_oldProfilePath;


BOOL APIENTRY DllMain( HMODULE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved)
{
  switch (ul_reason_for_call) {
		case DLL_PROCESS_ATTACH:
			g_restoreProfile = FALSE;
			g_userIsLoggedIn = FALSE;
			g_oldProfilePath = NULL;
			g_hModule = hModule;
			break;
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
  }

  return TRUE;
}

// These functions make up the interface of the plugin

PGFTP_API BOOL UserLogin(LPTSTR userName, LPTSTR password, PGINA_INFO *pginaInfo)
{
	TCHAR tempProfilePath[MAX_PATH];
	LPTSTR ftpUserName;
	LPTSTR ftpPassword;
	HWND hWndDlg, hWndStatus;
	PLUGIN_SETTINGS settings;
	LPVOID hConnect = NULL;
	FTP_FN_TABLE* pfnTable;
	BOOL success = TRUE;
	TCHAR status[64];
	
	if (!LoadSettings(&settings)) {
		ShowPluginError(MAKE_ERROR(IDS_ERR_CONFIG));
		return FALSE;
	}

	pfnTable = GetFtpFnTable(settings.protocol);

	if (settings.role & ROLE_AUTH) {
		// Authenticate the user
		if (!pfnTable->FtpConnect(&hConnect, settings.ftpServer, settings.port,
			userName, password, settings.flags))
		{
			success = FALSE;
		}
		
	}

	if ((settings.role & ROLE_PROFILE) && !g_userIsLoggedIn) {
		if (!success) {
			// Plugin set to authenticate but authentication failed
			ShowProfileError(IDS_ERR_NO_CONNECTION, pfnTable->FtpGetLastError(hConnect), TRUE);
			goto cleanup;
		}
		ftpUserName = (settings.ftpUserName ? settings.ftpUserName : userName);
		ftpPassword = (settings.ftpPassword ? settings.ftpPassword : password);
		
		// Download and unzip the profile from the FTP server to a temp local directory,
		// then change the profile path to point to this directory. Windows will copy 
		// the profile to its proper place.
		DeleteProfileDir(settings.profileTempRoot, userName);
		if (!CreateProfileDir(settings.profileTempRoot, userName)) {
			ShowProfileError(IDS_ERR_CREATE_TEMP_DIR, GetLastError(), TRUE);
			goto cleanup;
		}
		StringCchPrintf(tempProfilePath, MAX_PATH, TEXT("%s\\%s"), settings.profileTempRoot, userName);
		hWndDlg = ShowProgressDlg();
		hWndStatus = GetDlgItem(hWndDlg, IDC_STATIC_STATUS);

		// Connect if we haven't already
		if (!hConnect) {
			GetString(IDS_STATUS_CONNECTING, status);
			SetWindowText(hWndStatus, status);
			if (!pfnTable->FtpConnect(&hConnect, settings.ftpServer, settings.port,
				ftpUserName, ftpPassword, settings.flags))
			{
				ShowProfileError(IDS_ERR_NO_CONNECTION, pfnTable->FtpGetLastError(hConnect), TRUE);
				DestroyWindow(hWndDlg);
				goto cleanup;
			}
		}

		g_restoreProfile = LoadProfile(hConnect, &settings, tempProfilePath, 
			pginaInfo->pathProfile, (LPTSTR)pginaInfo->pathProfileDefault, 
			hWndDlg, userName, g_archiveType);
		if (g_restoreProfile) {
			g_oldProfilePath = lstrdup(pginaInfo->pathProfile);
			pginaInfo->pathProfile = lstrdup(tempProfilePath);
			pginaInfo->pathProfileDefault = NULL;
		}
		else {
			// Reset the profile paths to NULL so that Windows doesn't throw an error 
			// message on top of the one pgftp shows
			pginaInfo->pathProfile = pginaInfo->pathProfileDefault = NULL;
		}
		DestroyWindow(hWndDlg);
	}

cleanup:
	pfnTable->FtpClose(hConnect);
	FreeSettings(&settings);
	return success;
}


PGFTP_API BOOL ChangeUserPassword(LPTSTR Username,LPTSTR OldPassword,LPTSTR NewPassword) {
	return TRUE;
}

PGFTP_API LPCTSTR AboutPlugin(void) {
	static TCHAR buffer[1024];
	
	GetAboutText(buffer, sizeof(buffer) / sizeof(TCHAR));
	return buffer; // OK since buffer is static, otherwise very BAD!
}

PGFTP_API void ChangePluginSettings(void) {
	ShowSettingsDlg();
}

PGFTP_API void LoginHook(PGINA_INFO *pginaInfo) {
	PLUGIN_SETTINGS settings;

	g_userIsLoggedIn = TRUE;

	// Windows copied the profile, so we can delete the temp directory (Windows
	// will recreate it and copy the profile back when the user logs out).
	// This is done for security reasons.
	LoadSettings(&settings);
	DeleteProfileDir(settings.profileTempRoot, pginaInfo->Username);
	if (g_restoreProfile) {
		LocalFree(pginaInfo->pathProfile);
		pginaInfo->pathProfile = NULL;
	}
	FreeSettings(&settings);
}

PGFTP_API void LogoutHook(PGINA_INFO *pginaInfo)
{
	TCHAR path[MAX_PATH];
	LPTSTR ftpUserName;
	LPTSTR ftpPassword;
	HWND hWndDlg;
	PLUGIN_SETTINGS settings;

	// Zip and upload the profile from the temp directory back to the FTP server
	LoadSettings(&settings);
	if (g_restoreProfile) {
		StringCchPrintf(path, MAX_PATH, TEXT("%s\\%s"), settings.profileTempRoot, pginaInfo->Username);
		ftpUserName = (settings.ftpUserName ? settings.ftpUserName : pginaInfo->Username);
		ftpPassword = (settings.ftpPassword ? settings.ftpPassword : pginaInfo->Password);
		hWndDlg = ShowProgressDlg();
		SaveProfile(&settings, ftpUserName, ftpPassword, path, 
			g_oldProfilePath, hWndDlg, g_archiveType);
		DestroyWindow(hWndDlg);
	}	
	DeleteProfileDir(settings.profileTempRoot, pginaInfo->Username);
	FreeSettings(&settings);
	g_restoreProfile = FALSE;
	g_userIsLoggedIn = FALSE;
}

PGFTP_API BOOL IsRequired(void)
{
	return RegReadDWORD(S_REQUIRED, FALSE);
}

// Wrapper for pgSftpConnect that uses host validation function defined in
// auth.h
BOOL SftpConnectV(LPVOID* pphConnect, LPCTSTR server, int port, LPCTSTR user, LPCTSTR password, DWORD flags) {
	return pgSftpConnect(pphConnect, server, port, user, password, ValidateHostKey);
}

FTP_FN_TABLE* GetFtpFnTable(ProtocolEnum p) {
	// Tables of FTP functions. A group of functions is defined for each type of
	// connection
	static FTP_FN_TABLE ftpFnTables[2] = {
		// PROTOCOL_FTP
		{
			pgFtpConnect, pgFtpGetLastError, pgFtpGetFileSize, pgFtpCreatePath, 
			pgFtpGet, pgFtpPut, pgFtpDelete, pgFtpRename, pgFtpClose
		},
		// PROTOCOL_SFTP
		{
			SftpConnectV, pgSftpGetLastError, pgSftpGetFileSize, pgSftpCreatePath, 
			pgSftpGet, pgSftpPut, pgSftpDelete, pgSftpRename, pgSftpClose
		}
	};

	return &(ftpFnTables[p]);
}

void ShowPluginError(DWORD err) {
	TCHAR msg[1024];

	FormatError(err, msg, sizeof(msg) / sizeof(TCHAR));
	MessageBox(NULL, msg, TEXT("pgFtp Plugin"), MB_OK | MB_ICONSTOP | MB_TOPMOST | MB_SETFOREGROUND);
}