/*

	Copyright (C) 2004 University of Calgary

	

	Contact: Eric Parsons (ericpar@math.ucalgary.ca)



	This file is part of pgftp.



    pgftp is free software; you can redistribute it and/or modify

    it under the terms of the GNU General Public License as published by

    the Free Software Foundation; either version 2 of the License, or

    (at your option) any later version.



    pgftp is distributed in the hope that it will be useful,

    but WITHOUT ANY WARRANTY; without even the implied warranty of

    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

    GNU General Public License for more details.



    You should have received a copy of the GNU General Public License

    along with pgftp; if not, write to the Free Software

    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



 */



// pgftp.cpp : Defines the entry point for the DLL application.

//



#include "stdafx.h"

#include "pgftp.h"

#include "profile.h"

#include "misc.h"

#include "resource.h"



// Globals



// This gets set to TRUE only when a user successfully logs in with a 

// roaming profile

BOOL g_restoreProfile;

TCHAR g_archiveType[4];



// Saves the original profile path for the user

LPTSTR g_oldProfilePath;



// Plugin settings

PLUGIN_SETTINGS g_pluginSettings;



// Local function prototypes



BOOL LoadSettings();



INT_PTR CALLBACK SettingsProc(HWND hwndDlg,

    UINT uMsg,

    WPARAM wParam,

    LPARAM lParam

);





BOOL APIENTRY DllMain( HMODULE hModule, 

                       DWORD  ul_reason_for_call, 

                       LPVOID lpReserved)

{

  switch (ul_reason_for_call) {

		case DLL_PROCESS_ATTACH:

			g_restoreProfile = FALSE;

			g_oldProfilePath = NULL;

			SecureZeroMemory(&g_pluginSettings, sizeof(PLUGIN_SETTINGS));

			g_pluginSettings.hModule = hModule;

			break;

		case DLL_THREAD_ATTACH:

		case DLL_THREAD_DETACH:

		case DLL_PROCESS_DETACH:

			break;

  }



	

  return TRUE;

}



PGFTP_API BOOL UserLogin(LPTSTR userName, LPTSTR password, PGINA_INFO *pginaInfo)

{

	TCHAR tempProfilePath[MAX_PATH];

	LPTSTR ftpUserName;

	LPTSTR ftpPassword;



	if (!LoadSettings()) {

		ShowProfileError(TEXT("pgftp is not properly configured."), TRUE);

		return TRUE;

	}



	// Download and unzip the profile from the FTP server to a temp local directory,

	// then change the profile path to point to this directory. Windows will copy 

	// the profile to its proper place.

	DeleteProfileDir(g_pluginSettings.profileTempRoot, userName);

	if (!CreateProfileDir(g_pluginSettings.profileTempRoot, userName))

		ShowProfileError(TEXT("Unable to create temporary profile directory."), TRUE);

	else {

		wsprintf(tempProfilePath, TEXT("%s\\%s"), g_pluginSettings.profileTempRoot, userName);

		ftpUserName = (g_pluginSettings.ftpUserName ? g_pluginSettings.ftpUserName : userName);

		ftpPassword = (g_pluginSettings.ftpPassword ? g_pluginSettings.ftpPassword : password);

		g_restoreProfile = LoadFtpProfile(g_pluginSettings.ftpServer, ftpUserName, ftpPassword,

			tempProfilePath, pginaInfo->pathProfile, (LPTSTR)pginaInfo->pathProfileDefault, 

			g_pluginSettings.hModule, g_archiveType);

		if (g_restoreProfile) {

			g_oldProfilePath = pginaInfo->pathProfile;

			pginaInfo->pathProfile = lstrdup(tempProfilePath);

			pginaInfo->pathProfileDefault = NULL;

		}

		else {

			// Reset the profile paths to NULL so that Windows doesn't throw an error 

			// message on top of the one pgftp shows

			pginaInfo->pathProfile = pginaInfo->pathProfileDefault = NULL;

		}

	}

					

	return TRUE;

}





PGFTP_API BOOL ChangeUserPassword(LPTSTR Username,LPTSTR OldPassword,LPTSTR NewPassword) {

	return TRUE;

}



PGFTP_API LPCTSTR AboutPlugin(void) {

	return TEXT("pGina FTP Profile plugin. Enables roaming profiles using an FTP server as the central storage. Created by Eric Parsons for pGina, ericpar@math.ucalgary.ca");

}



PGFTP_API void ChangePluginSettings(void) {

	DialogBox(g_pluginSettings.hModule, MAKEINTRESOURCE(IDD_DIALOG1), NULL, SettingsProc);

}



PGFTP_API void LoginHook(PGINA_INFO *pginaInfo) {

	

	// Windows copied the profile, so we can delete the temp directory (Windows

	// will recreate it and copy the profile back when the user logs out)

	DeleteProfileDir(g_pluginSettings.profileTempRoot, pginaInfo->Username);

	if (g_restoreProfile) {

		LocalFree(pginaInfo->pathProfile);

		pginaInfo->pathProfile = NULL;

	}

}



PGFTP_API void LogoutHook(PGINA_INFO *pginaInfo)

{

	TCHAR path[MAX_PATH];

	LPTSTR ftpUserName;

	LPTSTR ftpPassword;



	// Zip and upload the profile from the temp directory back to the FTP server

	if (g_restoreProfile) {

		wsprintf(path, TEXT("%s\\%s"), g_pluginSettings.profileTempRoot, pginaInfo->Username);

		ftpUserName = (g_pluginSettings.ftpUserName ? g_pluginSettings.ftpUserName : pginaInfo->Username);

		ftpPassword = (g_pluginSettings.ftpPassword ? g_pluginSettings.ftpPassword : pginaInfo->Password);

		SaveFtpProfile(g_pluginSettings.ftpServer, ftpUserName, ftpPassword, path, 

			g_oldProfilePath, g_pluginSettings.hModule, g_archiveType, g_pluginSettings.compression);

	}	

	DeleteProfileDir(g_pluginSettings.profileTempRoot, pginaInfo->Username);

	g_restoreProfile = FALSE;



	// Remove the current user from the registry permissions

}



PGFTP_API BOOL IsRequired(void)

{

	return FALSE;

}



BOOL LoadSettings() {

	if (g_pluginSettings.ftpServer)

		LocalFree(g_pluginSettings.ftpServer);

	g_pluginSettings.ftpServer = GetStringValue(TEXT("ftpServer"));

	if (!g_pluginSettings.ftpServer)

		return FALSE;



	if (g_pluginSettings.profileTempRoot)

		LocalFree(g_pluginSettings.profileTempRoot);

	g_pluginSettings.profileTempRoot = GetStringValue(TEXT("profileTempRoot"));

	if (!g_pluginSettings.profileTempRoot)

		g_pluginSettings.profileTempRoot = lstrdup(TEXT("c:\\pgina\\profile_temp"));



	if (g_pluginSettings.ftpUserName)

		LocalFree(g_pluginSettings.ftpUserName);

	g_pluginSettings.ftpUserName = GetStringValue(TEXT("ftpUserName"));



	if (g_pluginSettings.ftpPassword)

		LocalFree(g_pluginSettings.ftpPassword);

	g_pluginSettings.ftpPassword = GetStringValue(TEXT("ftpPassword"));



	g_pluginSettings.compression = GetIntValue(TEXT("compression"), 6);

	return TRUE;

}





// Window procedure for the settings dialog

INT_PTR CALLBACK SettingsProc(HWND hwndDlg,

    UINT msg,

    WPARAM wParam,

    LPARAM lParam)

{

	#define MAX_SETTING 128

	TCHAR buffer[MAX_SETTING];

	BOOL checked;



	switch (msg) {

		case WM_INITDIALOG:

			// Initialize controls from registry values

			LoadSettings();

			SendMessage(GetDlgItem(hwndDlg, IDC_SLIDER_COMPRESSION), TBM_SETRANGEMIN, FALSE, 1);

			SendMessage(GetDlgItem(hwndDlg, IDC_SLIDER_COMPRESSION), TBM_SETRANGEMAX, TRUE, 9);

			SetDlgItemText(hwndDlg, IDC_EDIT_SERVER, g_pluginSettings.ftpServer);

			SetDlgItemText(hwndDlg, IDC_EDIT_PATH, g_pluginSettings.profileTempRoot);

			if (g_pluginSettings.ftpUserName) {

				CheckDlgButton(hwndDlg, IDC_RADIO_USERNAME_SPECIFY, BST_CHECKED);

				SetDlgItemText(hwndDlg, IDC_EDIT_USERNAME, g_pluginSettings.ftpUserName);

				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_USERNAME), TRUE);

			}

			else

				CheckDlgButton(hwndDlg, IDC_RADIO_USERNAME_USEDEFAULT, BST_CHECKED);

			

			if (g_pluginSettings.ftpPassword) {

				CheckDlgButton(hwndDlg, IDC_RADIO_PASSWORD_SPECIFY, BST_CHECKED);

				SetDlgItemText(hwndDlg, IDC_EDIT_PASSWORD, g_pluginSettings.ftpPassword);

				EnableWindow(GetDlgItem(hwndDlg, IDC_EDIT_PASSWORD), TRUE);

			}

			else

				CheckDlgButton(hwndDlg, IDC_RADIO_PASSWORD_USEDEFAULT, BST_CHECKED);



			EnableWindow(GetDlgItem(hwndDlg, IDOK), 

				lstrlen(g_pluginSettings.ftpServer) &&

				lstrlen(g_pluginSettings.profileTempRoot));



			if (g_pluginSettings.compression) 

				CheckDlgButton(hwndDlg, IDC_CHECK_COMPRESSION, BST_CHECKED);

			else

				CheckDlgButton(hwndDlg, IDC_CHECK_COMPRESSION, BST_UNCHECKED);

			

			SendMessage(GetDlgItem(hwndDlg, IDC_SLIDER_COMPRESSION), TBM_SETPOS, TRUE, g_pluginSettings.compression);

			SendMessage(hwndDlg, WM_COMMAND, MAKEWPARAM(IDC_CHECK_COMPRESSION, BN_CLICKED), 

				(LPARAM)GetDlgItem(hwndDlg, IDC_CHECK_COMPRESSION));

			return TRUE;



		case WM_COMMAND:

			if (EN_CHANGE == HIWORD(wParam)) {

				// Check whether we should enable or disable the Save button

				EnableWindow(GetDlgItem(hwndDlg, IDOK), 

					SendMessage(GetDlgItem(hwndDlg, IDC_EDIT_SERVER), WM_GETTEXTLENGTH, 0, 0) &&

					SendMessage(GetDlgItem(hwndDlg, IDC_EDIT_PATH), WM_GETTEXTLENGTH, 0, 0));

			}

			else if (BN_CLICKED == HIWORD(wParam)) {

				switch (LOWORD(wParam)) {

					case IDC_RADIO_USERNAME_USEDEFAULT:

					case IDC_RADIO_PASSWORD_USEDEFAULT:

						SetDlgItemText(hwndDlg, LOWORD(wParam) + 2, TEXT(""));

						EnableWindow(GetDlgItem(hwndDlg, LOWORD(wParam) + 2), FALSE);

						break;

					case IDC_RADIO_USERNAME_SPECIFY:

					case IDC_RADIO_PASSWORD_SPECIFY:

						EnableWindow(GetDlgItem(hwndDlg, LOWORD(wParam) + 1), TRUE);

						break;

					case IDC_CHECK_COMPRESSION:

						checked = IsDlgButtonChecked(hwndDlg, IDC_CHECK_COMPRESSION);

						EnableWindow(GetDlgItem(hwndDlg, IDC_SLIDER_COMPRESSION), checked);

						EnableWindow(GetDlgItem(hwndDlg, IDC_STATIC_COMPRESSION1), checked);

						EnableWindow(GetDlgItem(hwndDlg, IDC_STATIC_COMPRESSION2), checked);

						break;

					case IDOK:

						GetDlgItemText(hwndDlg, IDC_EDIT_SERVER, buffer, MAX_SETTING);

						SetStringValue(TEXT("ftpServer"), buffer);

						GetDlgItemText(hwndDlg, IDC_EDIT_PATH, buffer, MAX_SETTING);

						SetStringValue(TEXT("profileTempRoot"), buffer);

						GetDlgItemText(hwndDlg, IDC_EDIT_USERNAME, buffer, MAX_SETTING);

						SetStringValue(TEXT("ftpUserName"), buffer);

						GetDlgItemText(hwndDlg, IDC_EDIT_PASSWORD, buffer, MAX_SETTING);

						SetStringValue(TEXT("ftpPassword"), buffer);

						if (IsDlgButtonChecked(hwndDlg, IDC_CHECK_COMPRESSION))

							SetIntValue(TEXT("compression"), 

								SendMessage(GetDlgItem(hwndDlg, IDC_SLIDER_COMPRESSION), TBM_GETPOS, 0, 0));

						else

							SetIntValue(TEXT("compression"), 0);

						// Fall through

					case IDCANCEL:

						EndDialog(hwndDlg, LOWORD(wParam));

				}

				return TRUE;

			}

	}



	return FALSE;

}