/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */
#ifndef __pgftp_h_
#define __pgftp_h_

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the PGFTP_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// PGFTP_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef PGFTP_EXPORTS
#define PGFTP_API __declspec(dllexport)
#else
#define PGFTP_API __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef struct PGINA_INFO {
		LPTSTR pathMSGina;
		LPTSTR pathPlugin;
		LPTSTR pathProfile;
		LPTSTR mapPaths;
		LPTSTR Username;
		LPTSTR Password;
		LPTSTR homeDir;
		BOOL isAdmin;
		BOOL disabled;
		int authType;
		HANDLE hUser;
		LPTSTR userGroups;
		LPTSTR userDescription;
		LPTSTR userFullName;
		BOOL allowPassChange;
		LPTSTR errorString;
		LPTSTR defaultDomain;
		LPTSTR pathProfileDefault;
		BOOL Reserved4;
} PGINA_INFO;

typedef struct {
	LPTSTR ftpUserName;
	LPTSTR ftpPassword;
	LPTSTR ftpServer;
	LPTSTR profileTempRoot;
	DWORD compression;
	DWORD role;
	DWORD protocol;
	int port;
	LPTSTR sftpHostKey;
	BOOL required;
	DWORD flags;
} PLUGIN_SETTINGS;

extern HMODULE g_hModule;

// Constants for registry key names
#define S_FTP_SERVER TEXT("ftpServer")
#define S_FTP_USERNAME TEXT("ftpUserName")
#define S_FTP_PASSWORD TEXT("ftpPassword")
#define S_PROFILE_TEMP_ROOT TEXT("profileTempRoot")
#define S_COMPRESSION TEXT("compression")
#define S_ROLE TEXT("role")
#define S_PROTOCOL TEXT("protocol")
#define S_PORT TEXT("port")
#define S_SFTP_HOST_KEY TEXT("sftpHostKey")
#define S_FTP_FLAGS TEXT("ftpFlags")
#define S_SFTP_FLAGS TEXT("sftpFlags")
#define S_REQUIRED TEXT("required")

// These are the exported functions that make up the interface to
// the plugin
PGFTP_API BOOL UserLogin(LPTSTR, LPTSTR, PGINA_INFO *);			
PGFTP_API BOOL ChangeUserPassword(LPTSTR,LPTSTR,LPTSTR);
PGFTP_API LPCTSTR AboutPlugin(void);
PGFTP_API void ChangePluginSettings(void);
PGFTP_API void LoginHook(PGINA_INFO *);
PGFTP_API void LogoutHook(PGINA_INFO *);

// Option method with the 1.6.x series and up of pGina
PGFTP_API BOOL IsRequired(void);

typedef enum {
	PROTOCOL_FTP,
	PROTOCOL_SFTP
} ProtocolEnum;

typedef enum {
	ROLE_PROFILE = 0x1,
	ROLE_AUTH = 0x2,
	ROLE_BOTH = ROLE_PROFILE | ROLE_AUTH
} RoleEnum;

#define MIN_COMPRESSION 1
#define MAX_COMPRESSION 9
#define DEFAULT_COMPRESSION 6
#define DEFAULT_PROTOCOL PROTOCOL_FTP
#define DEFAULT_FTP_PORT 21
#define DEFAULT_SFTP_PORT 22
#define DEFAULT_PROFILE_TEMP_ROOT TEXT("c:\\pGina\\Profile_Temp")
#define DEFAULT_ROLE ROLE_PROFILE
#define MAX_SETTING 256

// Callback function prototype for file transfer progress. pContext is
// the application-defined value originally passed into the FtpGet or 
// FtpPut method
typedef void (*PFN_PROGRESS) (DWORD bytesTransferred, LPVOID pContext);

// Defines a table of function pointers for FTP-related tasks. 
typedef struct {
	BOOL (*FtpConnect) (LPVOID* pphConnect, LPCTSTR server, int port, LPCTSTR user, LPCTSTR password, DWORD flags);
	DWORD (*FtpGetLastError) (LPVOID hConnect);
	BOOL (*FtpGetFileSize) (LPVOID hConnect, LPCTSTR filename, PUINT64 pnSize);
	BOOL (*FtpCreatePath) (LPVOID hConnect, LPCTSTR path);
	BOOL (*FtpGet) (LPVOID hConnect, LPCTSTR remoteFile, LPCTSTR localFile, PFN_PROGRESS pfnProgress, LPVOID pContext);
	BOOL (*FtpPut) (LPVOID hConnect, LPCTSTR localFile, LPCTSTR remoteFile, PFN_PROGRESS pfnProgress, LPVOID pContext);
	BOOL (*FtpDelete) (LPVOID hConnect, LPCTSTR filename);
	BOOL (*FtpRename) (LPVOID hConnect, LPCTSTR filename, LPCTSTR newFilename);
	void (*FtpClose) (LPVOID hConnect);
} FTP_FN_TABLE;

/**
 * Selects a table of functions based on the protocol used (FTP or SFTP)
 **/
FTP_FN_TABLE* GetFtpFnTable(ProtocolEnum p);

/**
 * Displays a generic plugin error to the user
 **/
void ShowPluginError(DWORD err);


#ifdef __cplusplus
}
#endif

#endif // __pgftp_h_