/*

	Copyright (C) 2004 University of Calgary

	

	Contact: Eric Parsons (ericpar@math.ucalgary.ca)



	This file is part of pgftp.



    pgftp is free software; you can redistribute it and/or modify

    it under the terms of the GNU General Public License as published by

    the Free Software Foundation; either version 2 of the License, or

    (at your option) any later version.



    pgftp is distributed in the hope that it will be useful,

    but WITHOUT ANY WARRANTY; without even the implied warranty of

    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

    GNU General Public License for more details.



    You should have received a copy of the GNU General Public License

    along with pgftp; if not, write to the Free Software

    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



 */



#include "stdafx.h"

#include "profile.h"

#include "ftpfuncs.h"

#include "pgftp.h"



// See profile.h for function descriptions



// Used by CreateTempProfileDir()

typedef BOOL (WINAPI *SET_SECURITY_DESCRIPTOR_CONTROL_FUNC) (

    IN PSECURITY_DESCRIPTOR pSecurityDescriptor,

    IN SECURITY_DESCRIPTOR_CONTROL ControlBitsOfInterest,

    IN SECURITY_DESCRIPTOR_CONTROL ControlBitsToSet

);





int ShowProfileError(LPCTSTR error, BOOL logon)

{

	TCHAR buffer[1024];

	LPCTSTR strOp;

	LPCTSTR strTail;

	UINT buttons; 

	int ret;

	LPCTSTR PROFILE_ERR = TEXT("pGina was unable to %s your roaming profile due to the following reason:\n%s\n%s");



	if (logon) {

		strOp = TEXT("load");

		strTail = TEXT("A local profile will be used instead.");

		buttons = MB_OK;

	} 

	else {

		strOp = TEXT("save");

		strTail = TEXT("Changes made to your profile will be lost.");

		buttons = MB_RETRYCANCEL;

	}

	wsprintf(buffer, PROFILE_ERR, strOp, error, strTail);

	ret = MessageBox(NULL, buffer, TEXT("pGina FTP Profiles"),

		buttons | MB_ICONSTOP | MB_SETFOREGROUND);

	return ret;

}





void DeleteProfileDir(LPCTSTR profileRootDir, LPCTSTR userName) {

	SHFILEOPSTRUCT fileOp;

	TCHAR path[MAX_PATH];



	wsprintf(path, TEXT("%s\\%s%c"), profileRootDir, userName, '\0');

	fileOp.hwnd = NULL;

	fileOp.wFunc = FO_DELETE;

	fileOp.pFrom = path;

	fileOp.pTo = NULL;

	fileOp.fFlags = FOF_NOCONFIRMATION | FOF_NOERRORUI | FOF_SILENT;

	fileOp.hNameMappings = NULL;

	fileOp.lpszProgressTitle = NULL;



	SHFileOperation(&fileOp);

}



BOOL LoadFtpProfile(LPCTSTR server,

					LPCTSTR user,

					LPCTSTR password,

					LPCTSTR localPath, 

					LPCTSTR remotePath, 

					LPCTSTR defaultRemotePath,

					HMODULE hModule,

			/*OUT*/ LPTSTR archiveType)

{

	HINTERNET hConnect = NULL;

	HINTERNET hFind = NULL;

	WIN32_FIND_DATA findData;

	TCHAR localFile[MAX_PATH];

	TCHAR remoteFile[MAX_PATH];

	TCHAR dllPath[MAX_PATH];

	TCHAR zipCmd[MAX_PATH];

	STARTUPINFO si;

	PROCESS_INFORMATION pi;

	LPCTSTR PATHS[] = {remotePath, defaultRemotePath};

	LPCTSTR EXTS[] = {TEXT("zip"), TEXT("7z")};

	DWORD retCode = 0;

	BOOL success = FALSE;



	int i, j;

	

	hConnect = OpenProfileFtp(server, user, password);

	if (!hConnect) {

		ShowProfileError(TEXT("Unable to connect to FTP server."), TRUE);

		return FALSE;

	}



	// Find the first match for the profile file

	for (i = 0; i < (sizeof(PATHS) / sizeof(LPCTSTR)) && !hFind; i++) {

		for (j = 0; j < (sizeof(EXTS) / sizeof(LPCTSTR)) && !hFind; j++) {

			wsprintf(remoteFile, TEXT("%s/profile.%s"), PATHS[i], EXTS[j]);

			hFind = FtpFindFirstFile(hConnect, remoteFile, &findData, 0, 0);

		}

	}



	if (!hFind) {

		ShowProfileError(TEXT("No suitable roaming profile found on server."), TRUE);

		goto cleanup;

	}

			

	InternetCloseHandle(hFind);

	wsprintf(localFile, TEXT("%s\\profile.%s"), localPath, EXTS[j-1]);

	lstrcpy(archiveType, EXTS[j-1]);



	if (!FtpGetFile(hConnect, remoteFile, localFile, FALSE, 0, FTP_TRANSFER_TYPE_BINARY | INTERNET_FLAG_RELOAD, 0)) 

	{

		ShowProfileError(TEXT("Error downloading profile from FTP server."), TRUE);

		goto cleanup;

	}

	InternetCloseHandle(hConnect);



	// Unzip the profile

	GetModuleFileName(hModule, dllPath, MAX_PATH);

	PathRemoveFileSpec(dllPath);

	wsprintf(zipCmd, TEXT("%s\\7za.exe x -y \"%s\""), dllPath, localFile);

	ZeroMemory(&si, sizeof(si));

	si.cb = sizeof(si);

	ZeroMemory(&pi, sizeof(pi));

	if (!CreateProcess(NULL, zipCmd, NULL, NULL, TRUE, 0, NULL, localPath, &si, &pi)) {

		ShowProfileError(TEXT("pgftp could not unarchive your roaming profile because it could not run the unarchiving utility.\n")

			TEXT("Check that 7za.exe is installed in the same directory as pgftp."), TRUE);

		goto cleanup;

	}

	WaitForSingleObject(pi.hProcess, INFINITE);

	GetExitCodeProcess(pi.hProcess, &retCode);

	if (retCode) {

		ShowProfileError(TEXT("An error was encountered while unarchiving the profile."), TRUE);

		goto cleanup;

	}

	

	success = TRUE;	



cleanup:

	if (hConnect) InternetCloseHandle(hConnect);

	if (hFind) InternetCloseHandle(hFind);

	if (pi.hProcess) CloseHandle(pi.hProcess);

	if (pi.hThread) CloseHandle(pi.hThread);

	DeleteFile(localFile);



	return success;

}







BOOL SaveFtpProfile(LPCTSTR server,

					LPCTSTR user,

					LPCTSTR password,

					LPCTSTR localPath,

					LPCTSTR remotePath,

					HMODULE hModule,

					LPCTSTR archiveType,

					int compressionLevel)

{

	HINTERNET hConnect = NULL;

	BOOL success;

	TCHAR localFile[MAX_PATH];

	TCHAR remoteFile[MAX_PATH];

	TCHAR remoteBackupFile[MAX_PATH];

	TCHAR dllPath[MAX_PATH];

	TCHAR zipCmd[MAX_PATH];

	STARTUPINFO si;

	PROCESS_INFORMATION pi;

	DWORD retCode = 0;



	// Zip up the profile using 7za.exe

	wsprintf(localFile, TEXT("%s\\profile.%s"), localPath, archiveType);

	GetModuleFileName(hModule, dllPath, MAX_PATH);

	PathRemoveFileSpec(dllPath);

	wsprintf(zipCmd, TEXT("%s\\7za.exe a -y -r -t%s -mx=%d \"%s\" \"*\""), dllPath, archiveType, compressionLevel, localFile);

	ZeroMemory(&si, sizeof(si));

	si.cb = sizeof(si);

	ZeroMemory(&pi, sizeof(pi));

	if (!CreateProcess(NULL, zipCmd, NULL, NULL, TRUE, 0, NULL, localPath, &si, &pi)) {

		ShowProfileError(TEXT("pgftp could not archive your roaming profile because it could not run the archiving utility.\n")

			TEXT("Check that 7za.exe is installed in the same directory as pgftp."), TRUE);

		return FALSE;

	}

	WaitForSingleObject(pi.hProcess, INFINITE);

	GetExitCodeProcess(pi.hProcess, &retCode);

	if (retCode) {

		ShowProfileError(TEXT("An error was encountered while archiving the profile."), TRUE);

		CloseHandle(pi.hProcess);

		CloseHandle(pi.hThread);

		return FALSE;

	}

	CloseHandle(pi.hProcess);

	CloseHandle(pi.hThread);



	// The loops give the user a chance to retry the operation in the case

	// of an error before their profile changes are lost

	do {

		hConnect = OpenProfileFtp(server, user, password);

		if (!hConnect && IDCANCEL == ShowProfileError(TEXT("Unable to connect to FTP server."), FALSE)) 

				return FALSE;

	} while (!hConnect);

	

	wsprintf(remoteFile, TEXT("%s/profile.%s"), remotePath, archiveType);

	wsprintf(remoteBackupFile, TEXT("%s.bak"), remoteFile);

	// Create the profile path in case it doesn't exist

	FtpCreatePath(hConnect, remotePath); // <- This is not a standard Win32 function, it is defined in ftp.c



	// Delete any existing backup file

	FtpDeleteFile(hConnect, remoteBackupFile);

	// Backup the most recent copy of the profile (if it exists)

	FtpRenameFile(hConnect, remoteFile, remoteBackupFile);



	do {

		if (!(success = FtpPutFile(hConnect, localFile, remoteFile,  FTP_TRANSFER_TYPE_BINARY, 0))

				&& IDCANCEL == ShowProfileError(TEXT("Error uploading profile."), FALSE))

		{

			InternetCloseHandle(hConnect);

			return FALSE;

		}

	} while (!success);

	

	InternetCloseHandle(hConnect);

	return TRUE;

}





BOOL CreateProfileDir(LPCTSTR profileRootDir, LPCTSTR userName)

{

	#define NUM_ACES 3



	SID_IDENTIFIER_AUTHORITY NtAuthority = SECURITY_NT_AUTHORITY;

	PSID administratorsSid = NULL;

	PSID localSystemSid = NULL;

	PSID userSid = NULL;

	EXPLICIT_ACCESS ace[NUM_ACES];

	PACL acl = NULL;

	SECURITY_DESCRIPTOR sd;

	SECURITY_ATTRIBUTES sa;

	int i, ret;

	SET_SECURITY_DESCRIPTOR_CONTROL_FUNC pSetSecurityDescriptorControl;

	HMODULE hModule = NULL;

	BOOL success = FALSE;

	TCHAR path[MAX_PATH];

	DWORD userSidSize = 0;

	LPTSTR domainName = NULL;

	DWORD domainNameSize = 0;

	//SID_NAME_USE snu;



#ifdef UNICODE

	LPCWSTR profileRootDirW = profileRootDir;

#else

	WCHAR profileRootDirW[MAX_PATH];



	MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, profileRootDir, -1, 

		profileRootDirW, MAX_PATH);

#endif



	// Create parent directory if it doesn't exist

	ret = SHCreateDirectory(NULL, profileRootDirW);

	if (ret != ERROR_SUCCESS && ret != ERROR_ALREADY_EXISTS && ret != ERROR_FILE_EXISTS)

		return FALSE;



	// Get SID for local Administrators group

	if (!AllocateAndInitializeSid(&NtAuthority, 2, SECURITY_BUILTIN_DOMAIN_RID, 

		DOMAIN_ALIAS_RID_ADMINS, 0, 0, 0, 0, 0, 0, &administratorsSid))

		return FALSE;



	// Get SID for special SYSTEM user

	if (!AllocateAndInitializeSid(&NtAuthority, 1, SECURITY_LOCAL_SYSTEM_RID,

		0, 0, 0, 0, 0, 0, 0, &localSystemSid))

		goto cleanup;

	

	// Get SID for the local Users group. Ideally, this should be the SID

	// of the user logging in, however pGina hasn't yet created the user's

	// account when this function is called. 

	if (!AllocateAndInitializeSid(&NtAuthority, 2, SECURITY_BUILTIN_DOMAIN_RID,

		DOMAIN_ALIAS_RID_USERS, 0, 0, 0, 0, 0, 0, &userSid))

		goto cleanup;



	// Create an ACL with Administrators, System and Users having 

	// full control

	ZeroMemory(&ace, NUM_ACES * sizeof(EXPLICIT_ACCESS));

	for (i = 0; i < NUM_ACES; i++) {

		ace[i].grfAccessMode = SET_ACCESS;

		ace[i].grfAccessPermissions = GENERIC_ALL;

		ace[i].grfInheritance = SUB_CONTAINERS_AND_OBJECTS_INHERIT;

		ace[i].Trustee.TrusteeForm = TRUSTEE_IS_SID;

	}



	ace[0].Trustee.ptstrName = (LPTSTR) administratorsSid;

	ace[0].Trustee.TrusteeType = TRUSTEE_IS_GROUP;

	ace[1].Trustee.ptstrName = (LPTSTR) localSystemSid;

	ace[1].Trustee.TrusteeType = TRUSTEE_IS_USER;

	ace[2].Trustee.ptstrName = (LPTSTR) userSid;

	ace[2].Trustee.TrusteeType = TRUSTEE_IS_GROUP;



	if (SetEntriesInAcl(NUM_ACES, ace, NULL, &acl))

		goto cleanup;



	// The function SetSecurityDescriptorControl is only available

	// on Win2K and above, so we need to try a runtime call

	hModule = LoadLibrary(TEXT("advapi32.dll"));

	pSetSecurityDescriptorControl = (SET_SECURITY_DESCRIPTOR_CONTROL_FUNC)

		GetProcAddress(hModule, "SetSecurityDescriptorControl");



	// Create a security descriptor for the new directory with the above ACL

	if (!InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION))

		goto cleanup;

	if (pSetSecurityDescriptorControl) {

		// This prevents automatic inheritance of ACE's from the parent 

		// object (Win2K, 2003, and XP only)

		if (!pSetSecurityDescriptorControl(&sd, SE_DACL_PROTECTED, SE_DACL_PROTECTED))

			goto cleanup;

	}

	if (!SetSecurityDescriptorDacl(&sd, TRUE, acl, FALSE))

		goto cleanup;



	// Set Administrators as owner of directory. This is necessary for

	// Windows to recognize this dir as a profile dir.

	if (!SetSecurityDescriptorOwner(&sd, administratorsSid, FALSE))

		goto cleanup;

	

	// Finally, create the folder with the user's name and above security

	sa.nLength = sizeof(SECURITY_ATTRIBUTES);

	sa.bInheritHandle = TRUE;

	sa.lpSecurityDescriptor = &sd;

	wsprintf(path, TEXT("%s\\%s"), profileRootDir, userName);

	if (!CreateDirectory(path, &sa))

		goto cleanup;



	success = TRUE;



cleanup:

	if (administratorsSid)

		FreeSid(administratorsSid);

	if (localSystemSid)

		FreeSid(localSystemSid);

	if (userSid)

		FreeSid(userSid); 

	if (domainName)

		LocalFree(domainName);

	if (hModule)

		FreeLibrary(hModule);

	if (acl)

		LocalFree(acl);



	return success;

}