/*

	Copyright (C) 2004 University of Calgary

	

	Contact: Eric Parsons (ericpar@math.ucalgary.ca)



	This file is part of pgftp.



    pgftp is free software; you can redistribute it and/or modify

    it under the terms of the GNU General Public License as published by

    the Free Software Foundation; either version 2 of the License, or

    (at your option) any later version.



    pgftp is distributed in the hope that it will be useful,

    but WITHOUT ANY WARRANTY; without even the implied warranty of

    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

    GNU General Public License for more details.



    You should have received a copy of the GNU General Public License

    along with pgftp; if not, write to the Free Software

    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



 */



#ifndef __profile_h_

#define __profile_h_



/*********************************************************

 * profile.h - Helper functions for saving and retrieving 

 *		the user's profile

 *********************************************************/



/**

 * Helper function for showing profile-related errors to the user

 *

 * @param error The error to display

 * @param logon Specifies whether the user is logging on (TRUE) or logging

 *		off (FALSE).

 * @return The ID of the button selected. For logon errors,

 * 		this will always be IDOK. For logoff errors, it will be either

 *		IDCANCEL or IDRETRY.

 *

 * @TODO: This is really a temporary function, since messages to the user 

 *		should eventually be localized instead of hard-coded in English.

 **/

int ShowProfileError(LPCTSTR error, BOOL logon);



/**

 * Deletes the temporary profile directory for the current user

 **/

void DeleteProfileDir(LPCTSTR profileRootDir, LPCTSTR userName);



/**

 * Creates the temporary profile directory parentDir\userName and

 * sets the permissions accordingly so that it works properly with

 * Windows profiles

 **/

BOOL CreateProfileDir(LPCTSTR profileRootDir, LPCTSTR userName);



/**

 * Loads the current user's roaming profile from an FTP server

 * to a temp directory

 **/

BOOL LoadFtpProfile(LPCTSTR server,

					LPCTSTR user,

					LPCTSTR password,

					LPCTSTR localPath, 

					LPCTSTR remotePath, 

					LPCTSTR defaultRemotePath,

					HMODULE hModule,

			/*OUT*/ LPTSTR archiveType);



/**

 * Saves the current user's roaming profile to an FTP server

 * from a local temp directory

 **/

BOOL SaveFtpProfile(LPCTSTR server,

					LPCTSTR user,

					LPCTSTR password,

					LPCTSTR localPath,

					LPCTSTR remotePath,

					HMODULE hModule,

					LPCTSTR archiveType,

					int compressionLevel);

#endif